/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.communication.jndi.agent;

import java.util.HashMap;
import java.util.Map;

import javax.naming.NamingException;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.petals.communication.jndi.agent.msg.request.RegistryRequest;
import org.objectweb.petals.communication.jndi.agent.util.RegistryUtil;
import org.objectweb.petals.communication.network.NetworkService;
import org.objectweb.petals.kernel.configuration.ConfigurationService;
import org.objectweb.petals.kernel.configuration.ContainerConfiguration;
import org.objectweb.petals.kernel.configuration.DomainConfiguration;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.api.Logger;

/**
 * The implementation of the PEtALS JNDI agent {@link JNDIAgentService}.
 *
 * @author Roland NAUDIN - EbmWebsourcing
 * @since Petals 2.0
 *
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = org.objectweb.petals.communication.jndi.agent.JNDIAgentService.class))
public class JNDIAgentServiceImpl implements JNDIAgentService {

    /**
     * Logging utilities
     */
    protected LoggingUtil log;

    @Monolog(name = "logger")
    protected Logger logger;

    /**
     * The Network service fractal component
     */
    @Requires(name = "network", signature = org.objectweb.petals.communication.network.NetworkService.class)
    private NetworkService networkService;

    /**
     * The Configuration service fractal component
     */
    @Requires(name = "configuration", signature = org.objectweb.petals.kernel.configuration.ConfigurationService.class)
    private ConfigurationService configurationService;

    /**
     * The domain configuration
     */
    private DomainConfiguration domainConfiguration;

    /**
     * The container configuration
     */
    private ContainerConfiguration containerConfiguration;

    /**
     * Data stored in this distributed registry
     */
    protected Map<String, HashMap<String, Object>> data;

    /**
     * Registry utility
     */
    protected RegistryUtil registryUtil;

    /**
     * The Agent server thread
     */
    protected JNDIServerThread jndiServerThread;

    @LifeCycle(on = LifeCycleType.START)
    protected void start() throws JNDIAgentException {
        log = new LoggingUtil(logger);
        log.start();

        containerConfiguration = configurationService.getContainerConfiguration();
        domainConfiguration = configurationService
            .getDomainConfiguration();

        // start the agent only if necessary
        if (domainConfiguration.getJndiFactory().equals(
            JNDIAgentService.JNDI_PETALS_FACTORY)) {
            try {

                initData();

                registryUtil = new RegistryUtil(this);

                initServer();

            } catch (JNDIAgentException e) {
                // log the error here as Fractal layer is removing the Exception
                // stack
                log.error("Failed to start JNDI agent service", e);
                throw e;
            }
        }

        log.end();
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() throws Exception {
        log.start();

        // stop the agent only if necessary
        if (domainConfiguration.getJndiFactory().equals(
            JNDIAgentService.JNDI_PETALS_FACTORY)
            && jndiServerThread != null) {
            jndiServerThread.shutdown();
        }

        log.end();
    }

    /**
     * Initialize the JNDI data
     */
    private void initData() {
        data = networkService.getReplicatedHashMap();

        // init the root context if this is the first node
        if (!data.containsKey("/")) {
            HashMap<String, Object> newMap = new HashMap<String, Object>();
            data.put("/", newMap);

        }
    }

    /**
     * Initialize the Agent server
     *
     * @throws JNDIAgentException
     *             Failed to start the server
     */
    private void initServer() throws JNDIAgentException {
        // start the server only if this PEtALS node is configured to be the
        // JNDI server
        if (domainConfiguration.getJndiContainer() != null &&
                domainConfiguration.getJndiContainer().equals(
            containerConfiguration.getHost())) {
            jndiServerThread = new JNDIServerThread(this, log,
                containerConfiguration.getHost(), domainConfiguration
                    .getJndiPort());
            jndiServerThread.start();

            // wait that the socketServerThread is fully started
            synchronized (jndiServerThread) {
                if (!jndiServerThread.isStarted()) {
                    try {
                        jndiServerThread.wait(3000);
                    } catch (InterruptedException e) {
                        // do nothing
                    }
                }
            }

            if (!jndiServerThread.isStarted()) {
                throw new JNDIAgentException("Cannot start the JNDI server");
            }
        }
    }

    /**
     * Process a JNDI request
     *
     * @param request
     *            request to process
     * @return output Object of the operation
     * @throws NamingException
     */
    public Object processRequest(RegistryRequest request)
        throws NamingException {
        Object output = null;
        switch (request.getType()) {
            /*
             * BIND
             */
            case bind:
                registryUtil.bind(request.getArg1(), request.getArg2(), request
                    .getArg3());
                break;
            /*
             * CREATE SUBCONTEXT
             */
            case createSubcontext:
                output = registryUtil.createSubcontext(request.getArg1(),
                    request.getArg2());
                break;
            /*
             * DESTROY SUBCONTEXT
             */
            case destroySubcontext:
                registryUtil.destroySubcontext(request.getArg1(), request
                    .getArg2());
                break;
            /*
             * LIST
             */
            case list:
                output = registryUtil
                    .list(request.getArg1(), request.getArg2());
                break;
            /*
             * LIST BINDINGS
             */
            case listBindings:
                output = registryUtil.listBindings(request.getArg1(), request
                    .getArg2(), request.isLocalConnection());
                break;
            /*
             * LOOKUP
             */
            case lookup:
                output = registryUtil.lookup(request.getArg1(), request
                    .getArg2(), request.isLocalConnection());
                break;
            /*
             * LOOKUP LINK
             */
            case lookupLink:
                output = registryUtil.lookupLink(request.getArg1(), request
                    .getArg2(), request.isLocalConnection());
                break;
            /*
             * REBIND
             */
            case rebind:
                registryUtil.rebind(request.getArg1(), request.getArg2(),
                    request.getArg3());
                break;
            /*
             * RENAME
             */
            case rename:
                registryUtil.rename(request.getArg1(), request.getArg2(),
                    request.getArg3(), request.isLocalConnection());
                break;
            /*
             * UNBIND
             */
            case unbind:
                registryUtil.unbind(request.getArg1(), request.getArg2());
                break;

            default:
                break;
        }
        return output;
    }

    public LoggingUtil getLog() {
        return log;
    }

    public DomainConfiguration getDomainConfiguration() {
        return domainConfiguration;
    }

    public Map<String, HashMap<String, Object>> getData() {
        return data;
    }

}
