/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.communication.jndi.client;

import java.util.Hashtable;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.petals.communication.jndi.agent.JNDIAgentService;
import org.objectweb.petals.kernel.configuration.ConfigurationService;
import org.objectweb.petals.kernel.configuration.DomainConfiguration;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.api.Logger;

/**
 * Fractal component used to retrieve the JNDI initial context.
 *
 * @author alouis
 * @author Christophe HAMERLING - eBM WebSourcing
 * @author Roland NAUDIN - eBM WebSourcing
 *
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = org.objectweb.petals.communication.jndi.client.JNDIService.class))
public class JNDIServiceImpl implements JNDIService {

    /**
     * Logging utilities.
     */
    protected LoggingUtil log;

    @Monolog(name = "logger")
    protected Logger logger;

    /**
     * The JNDI Agent service fractal component
     */
    @Requires(name = "jndiAgent", signature = org.objectweb.petals.communication.jndi.agent.JNDIAgentService.class)
    private JNDIAgentService jndiAgentService;

    /**
     * The Configuration service fractal component
     */
    @Requires(name = "configuration", signature = org.objectweb.petals.kernel.configuration.ConfigurationService.class)
    private ConfigurationService configurationService;

    /**
     * The domain configuration
     */
    private DomainConfiguration domainConfiguration;

    private InitialContextDecorator initialContextDecorator;

    @LifeCycle(on = LifeCycleType.START)
    public void start() {
        log = new LoggingUtil(logger);
        log.start();

        domainConfiguration = configurationService.getDomainConfiguration();

        log.end();
    }

    @LifeCycle(on = LifeCycleType.STOP)
    public void stop() {
        log.start();

        if (initialContextDecorator != null) {
            try {
                initialContextDecorator.closeInitialContextDecorator();
            } catch (NamingException e) {
                log.error(e);
            } finally {
                initialContextDecorator = null;
            }
        }

        log.end();
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.kernel.jndi.JNDIService#getJNDIContext()
     */
    public InitialContext getInitialContext() throws NamingException {
        log.start();

        if (initialContextDecorator == null) {
            log.debug("Create a new InitialContext decorator");
            Hashtable<String, String> env = getJNDIEnv();
            log.debug("JNDI properties used : " + env);

            initialContextDecorator = new InitialContextDecorator(env, log,
                jndiAgentService);

        }

        log.end();
        return initialContextDecorator;
    }

    /**
     * Get the JNDI environment values.
     *
     * @return The environment hashTable
     */
    private Hashtable<String, String> getJNDIEnv() {
        Hashtable<String, String> env = new Hashtable<String, String>();
        env.put(Context.INITIAL_CONTEXT_FACTORY, domainConfiguration
            .getJndiFactory());

        if (domainConfiguration.getJndiFactory().equals(
            JNDIAgentService.JNDI_PETALS_FACTORY)) {
            env.put(JNDIAgentService.ENVIRONMENT_LOCAL_CONNECTION, Boolean
                .toString(true));
        }

        if (domainConfiguration.getJndiHost() != null) {
            env.put("java.naming.factory.host", domainConfiguration
                .getJndiHost());
            env.put("java.naming.factory.port", String
                .valueOf(domainConfiguration.getJndiPort()));
        }
        return env;
    }

}
