/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.communication.topology;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import javax.naming.Binding;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.NotContextException;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.petals.PetalsException;
import org.objectweb.petals.communication.jndi.client.JNDIService;
import org.objectweb.petals.communication.network.NetworkService;
import org.objectweb.petals.kernel.configuration.ConfigurationService;
import org.objectweb.petals.kernel.configuration.ContainerConfiguration;
import org.objectweb.petals.kernel.configuration.DomainConfiguration;
import org.objectweb.petals.kernel.configuration.DomainConfiguration.DomainMode;
import org.objectweb.petals.topology.TopologyException;
import org.objectweb.petals.util.JNDIUtil;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.api.Logger;

/**
 * This {@link TopologyService} implementation is based on JNDI.
 *
 * @author Roland NAUDIN - eBMWebSourcing
 * @since Petals 2.0
 *
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = org.objectweb.petals.communication.topology.TopologyService.class))
public class TopologyServiceImpl implements TopologyService {

    /**
     * Name of the JNDI container context
     */
    private static final String TOPOLOGY_REF = "topology";

    private static final String DOMAIN_REF = "_domain";

    /**
     * The root context
     */
    private static final String ROOT_REF = "/";

    /**
     * Logging utilities
     */
    private LoggingUtil log;

    @Monolog(name = "logger")
    private Logger logger;

    /**
     * The JNDI service fractal component
     */
    @Requires(name = "jndi", signature = org.objectweb.petals.communication.jndi.client.JNDIService.class)
    private JNDIService jndiService;

    /**
     * The Network service fractal component
     */
    @Requires(name = "network", signature = org.objectweb.petals.communication.network.NetworkService.class)
    private NetworkService networkService;

    /**
     * The Configuration service fractal component
     */
    @Requires(name = "configuration", signature = org.objectweb.petals.kernel.configuration.ConfigurationService.class)
    private ConfigurationService configurationService;

    /**
     * The local container configuration
     */
    private ContainerConfiguration containerConfiguration;

    /**
     * The domain configuration
     */
    private DomainConfiguration domainConfiguration;

    @LifeCycle(on = LifeCycleType.START)
    protected void start() throws PetalsException, TopologyException {
        log = new LoggingUtil(logger);
        log.start();

        String response;
        try {
            response = networkService.joinNetwork();

            if (!response.equals(NetworkService.JOIN_ACCEPTED)) {
                throw new PetalsException(
                    "PEtALS network refused to accept the local container. Cause: "
                        + response);
            }

            containerConfiguration = configurationService.getContainerConfiguration();
            domainConfiguration = configurationService.getDomainConfiguration();

            Context topologyContext = getOrCreateTopologyContext();

            if (domainConfiguration.getMode().equals(DomainMode.DYNAMIC)) {
                Set<ContainerConfiguration> containers = getAllContainersConfiguration(null);
                verifyContainerDynamicTopology(containers);
            }

            registerLocalContainer(topologyContext);
        } catch (PetalsException e) {
            // log the error here as Fractal layer is removing the Exception
            // stack
            log.error("Failed to start Topology service", e);
            throw e;
        } catch (TopologyException e) {
            // log the error here as Fractal layer is removing the Exception
            // stack
            log.error("Failed to start Topology service", e);
            throw e;
        }

        log.end();
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() throws Exception {
        log.call();
    }

    /**
     * Get or create the topology JNDI context.<br>
     * Note that the {@code domainConfiguration} field must be non null.
     *
     * @return The topology context
     * @throws PetalsException
     *             Failed to create or retrieve the topology JNDI context
     */
    private Context getOrCreateTopologyContext() throws PetalsException {
        log.start();

        Context topologyContext;
        try {
            InitialContext rootContext = jndiService.getInitialContext();

            if (!JNDIUtil.isBound(rootContext, ROOT_REF, TOPOLOGY_REF)) {
                topologyContext = rootContext.createSubcontext(TOPOLOGY_REF);
                topologyContext.bind(DOMAIN_REF, domainConfiguration);
            } else {
                topologyContext = (Context) rootContext.lookup(TOPOLOGY_REF);
            }
            if (topologyContext == null) {
                throw new NotContextException();
            }
        } catch (NamingException e) {
            e.printStackTrace();
            throw new PetalsException(
                "Problem while trying to initialize the topology context", e);
        }

        log.end();
        return topologyContext;
    }

    /**
     * Verify the local container configuration against the dynamic topology.
     *
     * @param containers
     *            The list of the domain' containers
     * @throws PetalsException
     */
    private void verifyContainerDynamicTopology(
        Set<ContainerConfiguration> containers) throws TopologyException,
        PetalsException {
        Collection<Integer> ports = new HashSet<Integer>();
        ports.add(containerConfiguration.getNetworkBindingPort());
        if (containerConfiguration.getJmxRMIConnectorPort() != 0) {
            ports.add(containerConfiguration.getJmxRMIConnectorPort());
        }
        if (containerConfiguration.getJmxHTTPConnectorPort() != 0) {
            ports.add(containerConfiguration.getJmxHTTPConnectorPort());
        }
        ports.add(containerConfiguration.getJoramDomainPort());
        ports.add(containerConfiguration.getJoramTCPPort());
        ports.add(containerConfiguration.getDreamTCPPort());

        for (ContainerConfiguration remoteContainerConfiguration : containers) {
            if (remoteContainerConfiguration.getName().equals(
                containerConfiguration.getName())) {
                if (remoteContainerConfiguration.getState().equals(
                    ContainerConfiguration.ContainerState.STARTED)) {
                    throw new PetalsException("Container named '"
                        + containerConfiguration.getName()
                        + "' is already started");
                }
                break;
            } else if (remoteContainerConfiguration.getHost().equals(
                containerConfiguration.getHost())) {
                if (!ports.add(remoteContainerConfiguration
                    .getNetworkBindingPort())) {
                    throw new TopologyException("Duplicate port "
                        + remoteContainerConfiguration.getNetworkBindingPort());
                }
                if (remoteContainerConfiguration.getJmxRMIConnectorPort() != 0) {
                    if (!ports.add(remoteContainerConfiguration
                        .getJmxRMIConnectorPort())) {
                        throw new TopologyException("Duplicate port "
                            + remoteContainerConfiguration
                                .getJmxRMIConnectorPort());
                    }
                }
                if (remoteContainerConfiguration.getJmxHTTPConnectorPort() != 0) {
                    if (!ports.add(remoteContainerConfiguration
                        .getJmxHTTPConnectorPort())) {
                        throw new TopologyException("Duplicate port "
                            + remoteContainerConfiguration
                                .getJmxHTTPConnectorPort());
                    }
                }
                if (!ports.add(remoteContainerConfiguration
                    .getJoramDomainPort())) {
                    throw new TopologyException("Duplicate port "
                        + remoteContainerConfiguration.getJoramDomainPort());
                }
                if (!ports.add(remoteContainerConfiguration.getJoramTCPPort())) {
                    throw new TopologyException("Duplicate port "
                        + remoteContainerConfiguration.getJoramTCPPort());
                }
                if (!ports.add(remoteContainerConfiguration.getDreamTCPPort())) {
                    throw new TopologyException("Duplicate port "
                        + remoteContainerConfiguration.getDreamTCPPort());
                }
            }
        }

    }

    private void registerLocalContainer(Context topologyContext)
        throws PetalsException {
        log.start();

        try {
            InitialContext rootContext = jndiService.getInitialContext();

            if (!JNDIUtil.isBound(rootContext, TOPOLOGY_REF,
                containerConfiguration.getName())) {

                topologyContext.bind(containerConfiguration.getName(),
                    ContainerConfiguration.getExportedForm(containerConfiguration));
            } else {
                topologyContext.rebind(containerConfiguration.getName(),
                    ContainerConfiguration.getExportedForm(containerConfiguration));
            }
        } catch (NamingException e) {
            throw new PetalsException(
                "Error while binding container information", e);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.communication.topology.TopologyService#getAllContainersConfiguration(org.objectweb.petals.kernel.configuration.ContainerConfiguration.ContainerState)
     */
    public Set<ContainerConfiguration> getAllContainersConfiguration(
        ContainerConfiguration.ContainerState state) throws PetalsException {
        log.start("Retrieve containers configuration with state " + state);

        Set<ContainerConfiguration> containers = new HashSet<ContainerConfiguration>();
        try {
            InitialContext rootContext = jndiService.getInitialContext();
            NamingEnumeration<Binding> enums = rootContext
                .listBindings(TOPOLOGY_REF);
            while (enums.hasMoreElements()) {
                Binding binding = enums.next();
                if (binding.getClassName().equals(ContainerConfiguration.class.getName())) {
                    ContainerConfiguration container = (ContainerConfiguration) binding
                        .getObject();

                    // get the entire configuration when it is the local container
                    if (container.getName().equals(containerConfiguration.getName())) {
                        container = containerConfiguration;
                    }

                    if (state == null) {
                        containers.add(container);
                    } else if (state.equals(container.getState()) ) {
                            containers.add(container);
                    }
                }
            }
        } catch (NamingException e) {
            throw new PetalsException(
                "Failed to retrieve containers configuration");
        }

        log.end("Returns " + containers.size() + " containers");
        return containers;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.communication.topology.TopologyService#getContainerConfiguration(java.lang.String)
     */
    public ContainerConfiguration getContainerConfiguration(String containerName)
        throws PetalsException {
        log.start("Get container configuration : " + containerName);

        ContainerConfiguration conf = null;
        Context topologyContext = getOrCreateTopologyContext();

        try {
            conf = (ContainerConfiguration) topologyContext
                .lookup(containerName);
        } catch (NamingException e) {
            throw new PetalsException(
                "Container configuration has not been found for container : "
                    + containerName);
        }

        log.end("Returns : " + conf);
        return conf;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.communication.topology.TopologyService#setContainerState(java.lang.String,
     *      org.objectweb.petals.kernel.configuration.ContainerConfiguration.ContainerState)
     */
    public void setContainerState(String containerName,
        ContainerConfiguration.ContainerState state) throws PetalsException {
        log.start("Set state " + state + " to the container named "
            + containerName);

        if (containerName == null) {
            throw new IllegalArgumentException(
                "The container name must not be null");
        }
        if (state == null) {
            throw new IllegalArgumentException("The state must not be null");
        }

        ContainerConfiguration conf = null;
        if (containerName.equals(containerConfiguration.getName())) {
            conf = containerConfiguration;
        } else {
            conf = getContainerConfiguration(containerName);
        }

        if (!conf.getState().equals(state)) {
            conf.setState(state);
            Context topologyContext = getOrCreateTopologyContext();
            try {
                topologyContext.rebind(conf.getName(), conf);
            } catch (NamingException e) {
                throw new PetalsException(
                    "Failed to set a state for the container named '"
                        + containerName + "'");
            }
        }

        log.end();
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.kernel.registry.ContainerNetwork#isContainerStarted(java.lang.String)
     */
    public boolean isContainerStarted(String containerName)
        throws PetalsException {
        log.start("Container started? " + containerName);

        if (containerName == null) {
            throw new IllegalArgumentException(
                "The container name must not be null");
        }

        boolean isStarted = false;
        ContainerConfiguration conf = getContainerConfiguration(containerName);
        isStarted = conf.getState().equals(
            ContainerConfiguration.ContainerState.STARTED);

        log.end("Returns : " + isStarted);
        return isStarted;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.communication.topology.TopologyService#unregisterContainer(java.lang.String)
     */
    public void unregisterContainer(String containerName)
        throws PetalsException {
        log.start();

        try {
            InitialContext rootContext = jndiService.getInitialContext();
            Context topologyContext = getOrCreateTopologyContext();
            if (JNDIUtil.isBound(rootContext, TOPOLOGY_REF, containerName)) {
                topologyContext.unbind(containerConfiguration.getName());
            }
        } catch (NamingException e) {
            throw new PetalsException("Failed to unregister container", e);
        }

        log.end();
    }

}
