/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ContainerServiceImpl.java,v 1.1 2007/04/11 13:34:07 lam Exp $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.container;

import java.util.ArrayList;
import java.util.List;

import javax.jbi.JBIException;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalContentException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.NameController;
import org.objectweb.fractal.api.control.SuperController;
import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Service;
import org.objectweb.fractal.util.Fractal;
import org.objectweb.petals.PetalsException;
import org.objectweb.petals.container.lifecycle.ServiceAssemblyLifeCycle;
import org.objectweb.petals.container.lifecycle.SharedLibraryLifeCycle;
import org.objectweb.petals.jbi.descriptor.ComponentDescription;
import org.objectweb.petals.jbi.descriptor.ServiceAssembly;
import org.objectweb.petals.jbi.descriptor.SharedLibrary;
import org.objectweb.petals.kernel.server.Binding;
import org.objectweb.petals.kernel.server.FractalHelper;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.api.Logger;

/**
 * This {@link ContainerServiceImpl} implementation allow to create and handle
 * JBI containers.
 *
 * @author Nicolas SALATGE - eBMWebSourcing
 *
 */
@FractalComponent
@Provides(interfaces = @org.objectweb.fractal.fraclet.annotation.annotations.Interface(name = "service", signature = org.objectweb.petals.container.ContainerService.class))
public class ContainerServiceImpl implements ContainerService {

    /**
     * Fractal name of the container service.
     */
    public static final String FRACTAL_CONTAINER_SERVICE_NAME = "ContainerServiceImpl";

    /**
     * Fractal name of the installer.
     */
    public static final String FRACTAL_INSTALLER_NAME = "org.objectweb.petals.container.lifecycle.Installer";

    /**
     * Fractal name of the component life cycle.
     */
    public static final String FRACTAL_COMPONENT_LIFE_CYCLE_NAME = "org.objectweb.petals.container.lifecycle.ComponentLifeCycle";

    /**
     * Fractal name of the service assembly life cycle.
     */
    public static final String FRACTAL_SERVICE_ASSEMBLY_LIFE_CYCLE_NAME = "org.objectweb.petals.container.lifecycle.ServiceAssemblyLifeCycle";

    /**
     * Fractal name of the shared library life cycle.
     */
    public static final String FRACTAL_SHARED_LIBRARY_LIFE_CYCLE_NAME = "org.objectweb.petals.container.lifecycle.SharedLibraryLifeCycleImpl";

    /**
     * Prefix of the installler name.
     */
    public static final String PREFIX_INSTALLER_NAME = "Installer";

    /**
     * Prefix of the component life cycle name.
     */
    public static final String PREFIX_COMPONENT_LIFE_CYCLE_NAME = "ComponentLifeCycle";

    /**
     * Prefix of the service assembly life cycle name.
     */
    public static final String PREFIX_SERVICE_ASSEMBLY_LIFE_CYCLE_NAME = "ServiceAssemblyLifeCycle";

    /**
     * Prefix of the shared library life cycle name.
     */
    public static final String PREFIX_SHARED_LIBRARY_LIFE_CYCLE_NAME = "SharedLibraryLifeCycle";

    /**
     * Configuration interface name.
     */
    public static final String CONFIGURATION_ITF = "configuration";

    /**
     * Admin interface name.
     */
    public static final String ADMIN_ITF = "admin";

    /**
     * Classloader interface name.
     */
    public static final String CLASSLOADER_ITF = "classloader";

    /**
     * Router interface name.
     */
    public static final String ROUTER_ITF = "router";

    /**
     * Repository interface name.
     */
    public static final String REPOSITORY_ITF = "repository";

    /**
     * Endpoint interface name.
     */
    public static final String ENDPOINT_ITF = "endpoint";

    /**
     * Registry interface name.
     */
    public static final String REGISTRY_ITF = "registry";

    /**
     * Systemstate interface name.
     */
    public static final String SYSTEMSTATE_ITF = "systemstate";

    /**
     * Systemstate interface name.
     */
    public static final String CONTAINER_ITF = "container";

    /**
     * Separator.
     */
    public static final String SEPARATOR = "_";

    /**
     * The FractalHelper singleton.
     */
    private static FractalHelper fractalHelper;

    /**
     * the log.
     */
    private LoggingUtil log;

    /**
     * the logger.
     */
    @Monolog(name = "logger")
    private Logger logger;

    /**
     * The component.
     */
    @Service
    private Component comp;

    /**
     * The parent component.
     */
    private Component parentcontainer;

    /**
     * Default container.
     *
     */
    public ContainerServiceImpl() {
    }

    /**
     * Get the parent container.
     *
     * @return the parent container
     */
    public Component getParentContainer() {
        return this.parentcontainer;
    }

    /**
     * Create an intaller from the component description of the component.
     *
     * @param componentDescription
     *            the component description of the component
     * @return the fractal component of the installer
     * @throws PetalsException :
     *             impossible to create the installer
     */
    public Component createInstaller(
        final ComponentDescription componentDescription) throws PetalsException {
        Component installer = null;
        try {

            // Creation of the installer
            installer = ContainerServiceImpl.fractalHelper
                .createNewComponent(ContainerServiceImpl.FRACTAL_INSTALLER_NAME);

            // Change name of the installer
            NameController nameController = Fractal
                .getNameController(installer);
            nameController.setFcName(ContainerServiceImpl.PREFIX_INSTALLER_NAME
                + ContainerServiceImpl.SEPARATOR
                + componentDescription.getIdentification().getName());

            // Get the contentcontroller of the container
            ContentController containerContentController = Fractal
                .getContentController(this.parentcontainer);

            // Creation of the list of bindings of the component with others
            // components
            List<Binding> listOfBindings = new ArrayList<Binding>();
            listOfBindings
                .add(new Binding(
                    ContainerServiceImpl.CONFIGURATION_ITF,
                    (Interface) containerContentController
                        .getFcInternalInterface(ContainerServiceImpl.CONFIGURATION_ITF)));
            listOfBindings.add(new Binding(ContainerServiceImpl.ADMIN_ITF,
                (Interface) containerContentController
                    .getFcInternalInterface(ContainerServiceImpl.ADMIN_ITF)));
            listOfBindings
                .add(new Binding(
                    ContainerServiceImpl.CLASSLOADER_ITF,
                    (Interface) containerContentController
                        .getFcInternalInterface(ContainerServiceImpl.CLASSLOADER_ITF)));
            listOfBindings.add(new Binding(ContainerServiceImpl.ROUTER_ITF,
                (Interface) containerContentController
                    .getFcInternalInterface(ContainerServiceImpl.ROUTER_ITF)));
            listOfBindings
                .add(new Binding(
                    ContainerServiceImpl.REPOSITORY_ITF,
                    (Interface) containerContentController
                        .getFcInternalInterface(ContainerServiceImpl.REPOSITORY_ITF)));
            listOfBindings
                .add(new Binding(
                    ContainerServiceImpl.ENDPOINT_ITF,
                    (Interface) containerContentController
                        .getFcInternalInterface(ContainerServiceImpl.ENDPOINT_ITF)));
            listOfBindings
                .add(new Binding(
                    ContainerServiceImpl.REGISTRY_ITF,
                    (Interface) containerContentController
                        .getFcInternalInterface(ContainerServiceImpl.REGISTRY_ITF)));
            listOfBindings
                .add(new Binding(
                    ContainerServiceImpl.SYSTEMSTATE_ITF,
                    (Interface) containerContentController
                        .getFcInternalInterface(ContainerServiceImpl.SYSTEMSTATE_ITF)));

            // Search of component containerServiceImpl
            Component containerService = ContainerServiceImpl.fractalHelper
                .getComponentByName(containerContentController,
                    ContainerServiceImpl.FRACTAL_CONTAINER_SERVICE_NAME);

            listOfBindings.add(new Binding(ContainerServiceImpl.CONTAINER_ITF,
                (Interface) containerService.getFcInterface("service")));

            // Add the new intaller in the framework
            ContainerServiceImpl.fractalHelper.addComponent(installer,
                this.parentcontainer, listOfBindings);
        } catch (ADLException e) {
            throw new PetalsException(e);
        } catch (NoSuchInterfaceException e) {
            throw new PetalsException(e);
        } catch (IllegalContentException e) {
            throw new PetalsException(e);
        } catch (IllegalLifeCycleException e) {
            throw new PetalsException(e);
        } catch (IllegalBindingException e) {
            throw new PetalsException(e);
        }
        return installer;
    }

    /**
     * Create a component life cycle from the component description of the
     * component.
     *
     * @param componentDescription
     *            the component description of the component
     * @return the fractal component of the component life cycle
     * @throws PetalsException :
     *             impossible to create a compoenent life cycle
     */
    public Component createComponentLifeCycle(
        final ComponentDescription componentDescription) throws PetalsException {
        Component componentLifeCycle = null;
        try {

            // Get the contentcontroller of the container
            ContentController containerContentController = Fractal
                .getContentController(this.parentcontainer);

            // Creation of the list of bindings of the component with others
            // components
            List<Binding> listOfBindings = new ArrayList<Binding>();
            listOfBindings
                .add(new Binding(
                    ContainerServiceImpl.CONFIGURATION_ITF,
                    (Interface) containerContentController
                        .getFcInternalInterface(ContainerServiceImpl.CONFIGURATION_ITF)));
            listOfBindings
                .add(new Binding(
                    ContainerServiceImpl.ADMIN_ITF,
                    (org.objectweb.fractal.api.Interface) containerContentController
                        .getFcInternalInterface(ContainerServiceImpl.ADMIN_ITF)));
            listOfBindings
                .add(new Binding(
                    ContainerServiceImpl.ROUTER_ITF,
                    (org.objectweb.fractal.api.Interface) containerContentController
                        .getFcInternalInterface(ContainerServiceImpl.ROUTER_ITF)));
            listOfBindings
                .add(new Binding(
                    ContainerServiceImpl.REPOSITORY_ITF,
                    (org.objectweb.fractal.api.Interface) containerContentController
                        .getFcInternalInterface(ContainerServiceImpl.REPOSITORY_ITF)));
            listOfBindings
                .add(new Binding(
                    ContainerServiceImpl.ENDPOINT_ITF,
                    (org.objectweb.fractal.api.Interface) containerContentController
                        .getFcInternalInterface(ContainerServiceImpl.ENDPOINT_ITF)));
            listOfBindings
                .add(new Binding(
                    ContainerServiceImpl.REGISTRY_ITF,
                    (org.objectweb.fractal.api.Interface) containerContentController
                        .getFcInternalInterface(ContainerServiceImpl.REGISTRY_ITF)));
            listOfBindings
                .add(new Binding(
                    ContainerServiceImpl.SYSTEMSTATE_ITF,
                    (org.objectweb.fractal.api.Interface) containerContentController
                        .getFcInternalInterface(ContainerServiceImpl.SYSTEMSTATE_ITF)));

            // Creation of the installer
            componentLifeCycle = ContainerServiceImpl.fractalHelper
                .createNewComponent(ContainerServiceImpl.FRACTAL_COMPONENT_LIFE_CYCLE_NAME);

            // Rename of the component life cycle
            NameController nameController = Fractal
                .getNameController(componentLifeCycle);
            nameController
                .setFcName(ContainerServiceImpl.PREFIX_COMPONENT_LIFE_CYCLE_NAME
                    + ContainerServiceImpl.SEPARATOR
                    + componentDescription.getIdentification().getName());

            // Add the new lifecycle in the framework
            ContainerServiceImpl.fractalHelper.addComponent(componentLifeCycle,
                this.parentcontainer, listOfBindings);

        } catch (ADLException e) {
            throw new PetalsException(e);
        } catch (NoSuchInterfaceException e) {
            throw new PetalsException(e);
        } catch (IllegalContentException e) {
            throw new PetalsException(e);
        } catch (IllegalLifeCycleException e) {
            throw new PetalsException(e);
        } catch (IllegalBindingException e) {
            throw new PetalsException(e);
        }
        return componentLifeCycle;
    }

    /**
     * Remove a component life cycle from the component description of the
     * component.
     *
     * @param componentDescription
     *            the component description of the component
     * @throws PetalsException :
     *             impossible to remove a component life cycle
     */
    public void removeComponentLifeCycle(
        final ComponentDescription componentDescription) throws PetalsException {
        try {

            // Get the contentcontroller of the container
            ContentController containerContentController = Fractal
                .getContentController(this.getParentContainer());

            // Get the component lifecycle component
            org.objectweb.fractal.api.Component componentLifeCycle = ContainerServiceImpl.fractalHelper
                .getComponentByName(containerContentController,
                    ContainerServiceImpl.PREFIX_COMPONENT_LIFE_CYCLE_NAME
                        + ContainerServiceImpl.SEPARATOR
                        + componentDescription.getIdentification().getName());

            // Stop the component
            ContainerServiceImpl.fractalHelper
                .stopComponent(componentLifeCycle);

            // Remove of all bind
            BindingController lifeCycleBindingController = Fractal
                .getBindingController(componentLifeCycle);
            for (int i = 0; i < lifeCycleBindingController.listFc().length; i++) {
                lifeCycleBindingController.unbindFc(lifeCycleBindingController
                    .listFc()[i]);
            }

            // Remove the component lifecycle component
            containerContentController.removeFcSubComponent(componentLifeCycle);

        } catch (NoSuchInterfaceException e) {
            throw new PetalsException(e);
        } catch (IllegalContentException e) {
            throw new PetalsException(e);
        } catch (IllegalLifeCycleException e) {
            throw new PetalsException(e);
        } catch (IllegalBindingException e) {
            throw new PetalsException(e);
        }
    }

    /**
     * Remove an installer from the name of the component.
     *
     * @param name
     *            the name of the component
     * @throws PetalsException
     *             impossible to remove the installer
     */
    public void removeInstaller(String name) throws PetalsException {
        try {

            // Get the contentcontroller of the container
            ContentController containerContentController = Fractal
                .getContentController(this.getParentContainer());

            // Get the component installer component
            org.objectweb.fractal.api.Component installer = ContainerServiceImpl.fractalHelper
                .getComponentByName(containerContentController,
                    ContainerServiceImpl.PREFIX_INSTALLER_NAME
                        + ContainerServiceImpl.SEPARATOR + name);

            // Stop the component
            ContainerServiceImpl.fractalHelper.stopComponent(installer);

            // Remove of all bind
            BindingController lifeCycleBindingController = Fractal
                .getBindingController(installer);
            for (int i = 0; i < lifeCycleBindingController.listFc().length; i++) {
                lifeCycleBindingController.unbindFc(lifeCycleBindingController
                    .listFc()[i]);
            }

            // Remove the component
            containerContentController.removeFcSubComponent(installer);

        } catch (NoSuchInterfaceException e) {
            throw new PetalsException(e);
        } catch (IllegalContentException e) {
            throw new PetalsException(e);
        } catch (IllegalLifeCycleException e) {
            throw new PetalsException(e);
        } catch (IllegalBindingException e) {
            throw new PetalsException(e);
        }
    }

    /**
     * Create a service assembly life cycle from the component description of
     * the component.
     *
     * @param sa
     *            the component description of the component
     * @return the fractal component of the service assembly life cycle
     * @throws PetalsException
     *             impossible to create a service assembly
     */
    public Component createServiceAssemblyLifeCycle(final ServiceAssembly sa)
        throws PetalsException {
        Component serviceAssemblyLifeCycle = null;
        try {

            // Get the contentcontroller of the container
            ContentController containerContentController = Fractal
                .getContentController(this.parentcontainer);

            // Creation of the list of bindings of the component with others
            // components
            List<Binding> listOfBindings = new ArrayList<Binding>();

            listOfBindings
                .add(new Binding(
                    ContainerServiceImpl.SYSTEMSTATE_ITF,
                    (org.objectweb.fractal.api.Interface) containerContentController
                        .getFcInternalInterface(ContainerServiceImpl.SYSTEMSTATE_ITF)));

            // Creation of the installer
            serviceAssemblyLifeCycle = ContainerServiceImpl.fractalHelper
                .createNewComponent(ContainerServiceImpl.FRACTAL_SERVICE_ASSEMBLY_LIFE_CYCLE_NAME);

            // Rename of the component life cycle
            NameController nameController = Fractal
                .getNameController(serviceAssemblyLifeCycle);
            nameController
                .setFcName(ContainerServiceImpl.PREFIX_SERVICE_ASSEMBLY_LIFE_CYCLE_NAME
                    + ContainerServiceImpl.SEPARATOR
                    + sa.getIdentification().getName());

            // Add the new lifecycle in the framework
            ContainerServiceImpl.fractalHelper.addComponent(
                serviceAssemblyLifeCycle, this.parentcontainer, listOfBindings);

        } catch (ADLException e) {
            throw new PetalsException(e);
        } catch (NoSuchInterfaceException e) {
            throw new PetalsException(e);
        } catch (IllegalContentException e) {
            throw new PetalsException(e);
        } catch (IllegalLifeCycleException e) {
            throw new PetalsException(e);
        } catch (IllegalBindingException e) {
            throw new PetalsException(e);
        }
        return serviceAssemblyLifeCycle;
    }

    /**
     * remove a service assembly life cycle.
     *
     * @param sa
     *            the servie assembly life cycle
     * @throws PetalsException
     *             Impossible to remove a service assembly
     */
    public void removeServiceAssemblyLifeCycle(final ServiceAssemblyLifeCycle sa)
        throws PetalsException {
        try {

            // Get the contentcontroller of the container
            ContentController containerContentController = Fractal
                .getContentController(this.getParentContainer());

            // Get the component serviceAssemblyLifeCycle component
            org.objectweb.fractal.api.Component serviceAssemblyLifeCycle = sa
                .getComponent();

            // Stop the component
            ContainerServiceImpl.fractalHelper
                .stopComponent(serviceAssemblyLifeCycle);

            // Remove of all bind
            BindingController lifeCycleBindingController = Fractal
                .getBindingController(serviceAssemblyLifeCycle);
            for (int i = 0; i < lifeCycleBindingController.listFc().length; i++) {
                lifeCycleBindingController.unbindFc(lifeCycleBindingController
                    .listFc()[i]);
            }

            // Remove the component lifecycle component
            containerContentController
                .removeFcSubComponent(serviceAssemblyLifeCycle);

        } catch (NoSuchInterfaceException e) {
            throw new PetalsException(e);
        } catch (IllegalContentException e) {
            throw new PetalsException(e);
        } catch (IllegalLifeCycleException e) {
            throw new PetalsException(e);
        } catch (IllegalBindingException e) {
            throw new PetalsException(e);
        }
    }

    /**
     * Create a shared library life cycle component.
     *
     * @param sl
     *            the shared library
     * @return the fractal component of the shared library
     * @throws PetalsException :
     *             occurs when it is impossible to create a shared library
     */
    public Component createSharedLibraryLifeCycle(final SharedLibrary sl)
        throws PetalsException {
        Component sharedLibraryLifeCycle = null;
        try {

            // Get the contentcontroller of the container
            ContentController containerContentController = Fractal
                .getContentController(this.parentcontainer);

            // Creation of the list of bindings of the component with others
            // components
            List<Binding> listOfBindings = new ArrayList<Binding>();

            listOfBindings
                .add(new Binding(
                    ContainerServiceImpl.CLASSLOADER_ITF,
                    (org.objectweb.fractal.api.Interface) containerContentController
                        .getFcInternalInterface(ContainerServiceImpl.CLASSLOADER_ITF)));

            // Creation of the shared library life cycle
            sharedLibraryLifeCycle = ContainerServiceImpl.fractalHelper
                .createNewComponent(ContainerServiceImpl.FRACTAL_SHARED_LIBRARY_LIFE_CYCLE_NAME);

            // Rename of the component life cycle
            NameController nameController = Fractal
                .getNameController(sharedLibraryLifeCycle);
            nameController
                .setFcName(ContainerServiceImpl.PREFIX_SHARED_LIBRARY_LIFE_CYCLE_NAME
                    + ContainerServiceImpl.SEPARATOR
                    + sl.getIdentification().getName());

            // Add the new lifecycle in the framework
            ContainerServiceImpl.fractalHelper.addComponent(
                sharedLibraryLifeCycle, this.parentcontainer, listOfBindings);

        } catch (ADLException e) {
            throw new PetalsException(e);
        } catch (NoSuchInterfaceException e) {
            throw new PetalsException(e);
        } catch (IllegalContentException e) {
            throw new PetalsException(e);
        } catch (IllegalLifeCycleException e) {
            throw new PetalsException(e);
        } catch (IllegalBindingException e) {
            throw new PetalsException(e);
        }
        return sharedLibraryLifeCycle;
    }

    /**
     * Remove a shared library life cycle component.
     *
     * @param slName
     *            the shared library name
     * @throws PetalsException :
     *             occurs when it is impossible to remove a shared library
     */
    public void removeSharedLibraryLifeCycle(final String slName)
        throws PetalsException {
        try {

            // Get the contentcontroller of the container
            ContentController containerContentController = Fractal
                .getContentController(this.getParentContainer());

            // Get the component sharedLibraryLifeCycle component
            org.objectweb.fractal.api.Component sharedLibraryLifeCycle = ContainerServiceImpl.fractalHelper
                .getComponentByName(containerContentController,
                    ContainerServiceImpl.PREFIX_SHARED_LIBRARY_LIFE_CYCLE_NAME
                        + ContainerServiceImpl.SEPARATOR + slName);

            // unload of PetalsShared Libraries
            ((SharedLibraryLifeCycle) sharedLibraryLifeCycle
                .getFcInterface("service")).unLoadSharedLibrary();

            // Stop the component
            ContainerServiceImpl.fractalHelper
                .stopComponent(sharedLibraryLifeCycle);

            // Remove of all bind
            BindingController lifeCycleBindingController = Fractal
                .getBindingController(sharedLibraryLifeCycle);
            for (int i = 0; i < lifeCycleBindingController.listFc().length; i++) {
                lifeCycleBindingController.unbindFc(lifeCycleBindingController
                    .listFc()[i]);
            }

            // Remove the component lifecycle component
            containerContentController
                .removeFcSubComponent(sharedLibraryLifeCycle);

        } catch (NoSuchInterfaceException e) {
            throw new PetalsException(e);
        } catch (IllegalContentException e) {
            throw new PetalsException(e);
        } catch (IllegalLifeCycleException e) {
            throw new PetalsException(e);
        } catch (IllegalBindingException e) {
            throw new PetalsException(e);
        }
    }

    /**
     * Initialization of the petals component.
     *
     * @throws PetalsException :
     *             impossible to start the component
     * @throws JBIException :
     *             JBI error
     * @throws NoSuchInterfaceException :
     *             Fractal Error
     * @throws ADLException :
     *             Fractal error
     *
     */
    @LifeCycle(on = LifeCycleType.START)
    public void start() throws PetalsException, NoSuchInterfaceException,
        JBIException, ADLException {
        this.log = new LoggingUtil(this.logger);
        this.log.start();

        ContainerServiceImpl.fractalHelper = FractalHelper.getFractalHelper();

        SuperController sc = Fractal.getSuperController(this.comp);
        if (sc.getFcSuperComponents().length != 1) {
            throw new PetalsException("Error in fractal architecture");
        }
        this.parentcontainer = sc.getFcSuperComponents()[0];

        this.log.end();
    }

    /**
     * Stop of the petals component.
     *
     */
    @LifeCycle(on = LifeCycleType.STOP)
    public void stop() {
        this.log.start();

        this.logger = null;

        this.log.end();
    }

}
