/**
 * PETALS - PETALS Services Platform.
 * Copyright (C) 2007 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: LifeCycleSharedLibraryImpl
 *
 */

package org.objectweb.petals.container.lifecycle;

import java.net.URL;
import java.util.List;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.petals.PetalsException;
import org.objectweb.petals.jbi.descriptor.SharedLibrary;
import org.objectweb.petals.platform.classloader.ClassLoaderService;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.api.Logger;


/**
 * The <code>LifeCycle</code> implementation of the shared library.
 *
 * @author Nicolas SALATGE - EBM WebSourcing
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service",
        signature =
            org.objectweb.petals.container.lifecycle.SharedLibraryLifeCycle.class))
public class SharedLibraryLifeCycleImpl implements SharedLibraryLifeCycle {


    /**
     * state flag: the shared library is installed.
     */
    public static final String INSTALLED = "Installed";

    /**
     * state flag: the shared library is installing.
     */
    public static final String INSTALLING = "Installing";

    /**
     * state flag: the shared library is uninstalled.
     */
    public static final String UNINSTALLED = "Uninstalled";

    /**
     * state flag: the shared library is uninstalling.
     */
    public static final String UNINSTALLING = "Uninstalling";

    /**
     * state flag: the state of the shared library is unknown.
     */
    public static final String UNKNOWN = "Unknown";

    /**
     * the state of the shared library.
     */
    private String state;

    /**
     *  the log.
     */
    private LoggingUtil log;

    /**
     * the logger.
     */
    @Monolog(name = "logger")
    private Logger logger;

    /**
     * the shared library.
     */
    private SharedLibrary sl;

    /**
     * the base urls.
     */
    private URL[] baseUrls;

    /**
     * the use parent first.
     */
    private boolean useParentFirst;

    /**
     * Platform Component :: Loader Service.
     */
    @Requires(name = "classloader",
            signature = org.objectweb.petals.platform.classloader.ClassLoaderService.class)
    private ClassLoaderService loaderService;


    /**
     * The default constructor.
     */
    public SharedLibraryLifeCycleImpl() {

    }

    /**
     * Initialization of sharedLibraryLifeCycle.
     * @param sl1
     *         the shared library
     * @param baseUrls1
     *         the base urls
     * @param useParentFirst1
     *         the use parent first
     */
    public void init(final SharedLibrary sl1,
            final URL[] baseUrls1, final boolean useParentFirst1) {
        this.sl = sl1;
        this.baseUrls = baseUrls1;
        this.useParentFirst = useParentFirst1;
    }

    /**
     * Test the installation flag.
     *
     * @return true if the shared library is already installed
     */
    public boolean isLoaded() {
        return this.state.equals(SharedLibraryLifeCycleImpl.INSTALLED);
    }

    /**
     * Install the jbi shared library. Call the Bootstrap onInstall() and cleanup()
     * methods, register the shared library object, and return its jmxName.
     *
     * @throws PetalsException
     *             the component is already installed
     */
    public void loadSharedLibrary() throws PetalsException {
        this.log.start();
        // create the class loader
        this.state = SharedLibraryLifeCycleImpl.INSTALLING;

        String classLoaderId = this.sl.getIdentification().getName();
        List<String> classpathUrls = this.sl.getSharedLibraryClassPath();

        this.loaderService.createSharedLibraryClassLoader(classLoaderId,
                this.baseUrls, classpathUrls, this.useParentFirst);

        this.state = SharedLibraryLifeCycleImpl.INSTALLED;
        this.log.end();


    }


    /**
     * Uninstall the jbi shared library. Call the Bootstrap onUninstall() and
     * cleanup() methods, deregister the jbiComponent object,
     *
     */
    public void unLoadSharedLibrary() {
        this.log.start();

        this.state = SharedLibraryLifeCycleImpl.UNINSTALLING;

        String classLoaderId = this.sl.getIdentification().getName();
        this.loaderService.deleteClassLoader(classLoaderId);

        this.state = SharedLibraryLifeCycleImpl.UNINSTALLED;

        this.log.end();
    }




    /**
     * Start of the shared library.
     *
     */
    @LifeCycle(on = LifeCycleType.START)
    public void start() {
        this.log = new LoggingUtil(this.logger);
        this.log.start();

        this.log.end();
    }


    /**
     * Stop of the shared library.
     *
     */
    @LifeCycle(on = LifeCycleType.STOP)
    public void stop() {
        this.log.start();

        this.sl = null;
        this.baseUrls = null;
        this.useParentFirst = false;

        this.log.end();
    }
}
