
package org.objectweb.petals.jbi.management.admin;

import java.util.Collections;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import javax.management.MBeanServer;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.objectweb.petals.container.lifecycle.ComponentLifeCycle;
import org.objectweb.petals.container.lifecycle.Installer;
import org.objectweb.petals.container.lifecycle.ServiceAssemblyLifeCycle;
import org.objectweb.petals.container.lifecycle.SharedLibraryLifeCycle;
import org.objectweb.petals.jbi.management.MBeanNames;
import org.objectweb.petals.jbi.management.MBeanNamesImpl;
import org.objectweb.petals.jbi.management.ManagementException;
import org.objectweb.petals.kernel.server.MBeanHelper;
import org.objectweb.petals.util.LoggingUtil;

/**
 * <code>LifeCycleManagerImpl</code> is the main class for JMX management. It
 * is the entry point to register and unregister elements in the JMX environment
 * (elements are JBI binding/engine components, service assemblies, installers
 * and systemServices components). Related MBeans are created when registering
 * and unregistering elements. <code>LifeCycleManagerImpl</code> owns the JMX
 * management services offered by the JBI container to administrative clients.
 * Management services are : - AdminServiceImpl (referenced in the JMX server as
 * "admin"); - InstallationServiceMBean (referenced in the JMX server as
 * "installation"); - DeploymentServiceMBean (referenced in the JMX server as
 * "deployment"); <code>LifeCycleManagerImpl</code> is the central point for
 * accessing the mBeans associated to the JBI container elements.
 * <code>ManagementException</code> exceptions are thrown when problems for
 * accessing the JMX server occur. TODO refactor to get System Services from
 * Fractal, and BC and SE from JMX TODO set all properties in a single file, and
 * add a FractalAttribute component to retrieve it (for joram, container name,
 * etc)
 *
 * @author Adrien LOUIS - EBM WebSourcing
 */
public class LifeCycleManagerImpl {

    private final static String PETALS_DOMAIN = "Petals";

    private final static String TYPE = "type";

    private final static String NAME = "name";

    private final static String SERVICE_TYPE = "service";

    protected Map<ObjectName, ComponentLifeCycle> bindingCompoLifeCycles;

    protected Map<ObjectName, ComponentLifeCycle> engineCompoLifeCycles;

    protected Map<String, ServiceAssemblyLifeCycle> serviceAssemblyLifeCycles;

    protected Map<String, SharedLibraryLifeCycle> sharedLibraryLifeCycles;

    protected Map<ObjectName, Installer> installers;

    protected MBeanNames mBeanNames;

    protected MBeanServer adminAttributes;

    /**
     * Logger wrapper
     */
    private LoggingUtil log;

    /**
     * Default constructor. Creates a <code>Manager</code> instance. JMX
     * server is created.
     *
     */
    public LifeCycleManagerImpl(MBeanServer adminAttributes, LoggingUtil log) {

        this.log = log;

        this.adminAttributes = adminAttributes;

        mBeanNames = new MBeanNamesImpl("org.objectweb.petals");

        bindingCompoLifeCycles = Collections
                .synchronizedMap(new HashMap<ObjectName, ComponentLifeCycle>());

        engineCompoLifeCycles = Collections
                .synchronizedMap(new HashMap<ObjectName, ComponentLifeCycle>());

        serviceAssemblyLifeCycles = Collections
                .synchronizedMap(new HashMap<String, ServiceAssemblyLifeCycle>());

        installers = Collections
                .synchronizedMap(new HashMap<ObjectName, Installer>());

        sharedLibraryLifeCycles = Collections
        .synchronizedMap(new HashMap<String, SharedLibraryLifeCycle>());

    }

    // /////////////////////////
    // /// retrieve platform component MBeanNames
    // /////////////////////////
    /**
     * Retrieve the AdminServieMBean ObjectName.
     */
    public ObjectName getAdminServiceMBeanName() throws ManagementException {
        ObjectName result = null;
        Set objNames;
        try {
            Hashtable<String, String> attributes = new Hashtable<String, String>();
            attributes.put(NAME, MBeanHelper.ADMIN_MBEAN);
            attributes.put(TYPE, SERVICE_TYPE);
            ObjectName objName = new ObjectName(PETALS_DOMAIN, attributes);
            objNames = adminAttributes.queryNames(objName, null);
        } catch (MalformedObjectNameException e) {
            String msg = "Error during AdminServiceImpl ObjName retrieval";
            log.error(msg, e);
            throw new ManagementException(msg, e);
        } catch (NullPointerException e) {
            String msg = "Error during AdminServiceImpl ObjName retrieval";
            log.error(msg, e);
            throw new ManagementException(msg, e);
        }
        if (objNames != null && objNames.size() == 1) {
            result = (ObjectName) objNames.iterator().next();
        } else {
            throw new ManagementException("AdminServiceImpl Mbean can't be "
                    + "correctly retrieved");
        }
        return result;
    }

    /**
     * Retrieve the DeploymentServiceMBean ObjectName. This is used by a
     * ServiceAssembly LifeCycle. As there is no MBean for the SA LifeCycle, the
     * DeploymentServiceMbean is used to represent it. All SA notifications are
     * reported to the DeploymentLifeCycleMB
     *
     * @return the jmx name of the deploymentServiceMBean
     */
    public ObjectName getDeploymentServiceMBeanName()
            throws ManagementException {
        ObjectName result = null;
        Set objNames;
        try {
            Hashtable<String, String> attributes = new Hashtable<String, String>();
            attributes.put(NAME, MBeanHelper.DEPLOYMENT_MBEAN);
            attributes.put(TYPE, SERVICE_TYPE);
            ObjectName objName = new ObjectName(PETALS_DOMAIN, attributes);
            objNames = adminAttributes.queryNames(objName, null);
        } catch (MalformedObjectNameException e) {
            String msg = "Error during DeploymentServiceMBean ObjName retrieval";
            log.error(msg, e);
            throw new ManagementException(msg, e);
        } catch (NullPointerException e) {
            String msg = "Error during DeploymentServiceMBean ObjName retrieval";
            log.error(msg, e);
            throw new ManagementException(msg, e);
        }
        if (objNames != null && objNames.size() == 1) {
            result = (ObjectName) objNames.iterator().next();
        } else {
            throw new ManagementException(
                    "DeploymentServiceMBean Mbean can't be "
                            + "correctly retrieved");
        }
        return result;
    }

    /**
     * Retrieve the InstallationServiceMBean ObjectName.
     */
    public ObjectName getInstallationServiceMBeanName()
            throws ManagementException {
        ObjectName result = null;
        Set objNames;
        try {
            Hashtable<String, String> attributes = new Hashtable<String, String>();
            attributes.put(NAME, MBeanHelper.INSTALLATION_MBEAN);
            attributes.put(TYPE, SERVICE_TYPE);
            ObjectName objName = new ObjectName(PETALS_DOMAIN, attributes);
            objNames = adminAttributes.queryNames(objName, null);
        } catch (MalformedObjectNameException e) {
            String msg = "Error during InstallationServiceMBean ObjName retrieval";
            log.error(msg, e);
            throw new ManagementException(msg, e);
        } catch (NullPointerException e) {
            String msg = "Error during InstallationServiceMBean ObjName retrieval";
            log.error(msg, e);
            throw new ManagementException(msg, e);
        }
        if (objNames != null && objNames.size() == 1) {
            result = (ObjectName) objNames.iterator().next();
        } else {
            throw new ManagementException(
                    "InstallationServiceMBean Mbean can't be "
                            + "correctly retrieved");
        }
        return result;
    }

    // ////////////////////////////
    // manage components : bindings and engines
    // ////////////////////////////

    /**
     * Register a mbean in the JMX server representing the specified binding
     * component.
     *
     * @param componentElement
     * @throws ManagementException
     */
    public ObjectName registerBindingComponent(ComponentLifeCycle lifeCycle)
            throws ManagementException {

        ObjectName name = lifeCycle.getMBeanName();

        Map<ObjectName, ComponentLifeCycle> bindings = getBindingCompoLifeCycles();

        synchronized (bindings) {
            try {
                adminAttributes.registerMBean(lifeCycle, name);

                bindings.put(name, lifeCycle);
            } catch (Exception e) {
                throw new ManagementException(e);
            }
        }
        return name;
    }

    /**
     * Register a mbean in the JMX server representing the specified engine
     * component
     *
     * @param componentElement
     * @throws ManagementException
     */
    public ObjectName registerEngineComponent(ComponentLifeCycle lifeCycle)
            throws ManagementException {
        ObjectName name = lifeCycle.getMBeanName();

        Map<ObjectName, ComponentLifeCycle> engines = getEngineCompoLifeCycles();

        synchronized (engines) {
            try {
                adminAttributes.registerMBean(lifeCycle, name);

                engines.put(name, lifeCycle);
            } catch (Exception e) {
                throw new ManagementException(e);
            }
        }
        return name;
    }

    /**
     * @see org.objectweb.petals.jbi.management.admin.LifeCycleManagerService#getComponentByName(java.lang.String)
     */
    public ComponentLifeCycle getComponentLifeCycleByName(String name) {
        log.call();

        ComponentLifeCycle on = null;

        // search in bindings

        on = getBindingComponentByName(name);

        if (on == null) {
            // search in engines

            on = getEngineComponentByName(name);
        }
        return on;
    }

    /**
     * Return the binding component associated with this name.
     *
     * @param name
     * @return The related objectName, or null
     */
    private ComponentLifeCycle getBindingComponentByName(String name) {
        log.call();

        ComponentLifeCycle result = null;

        Map bindings = this.getBindingCompoLifeCycles();

        synchronized (bindings) {
            for (Iterator iter = bindings.values().iterator(); iter.hasNext()
                    && result == null;) {
                ComponentLifeCycle lifeCycle = (ComponentLifeCycle) iter.next();

                if (lifeCycle.getName().equals(name)) {
                    result = lifeCycle;
                }
            }
        }
        return result;
    }

    /**
     * Return the engine component associated with this name.
     *
     * @param name
     * @return The related objectName, or null
     */
    private ComponentLifeCycle getEngineComponentByName(String name) {
        log.call();

        ComponentLifeCycle result = null;

        Map engines = this.getEngineCompoLifeCycles();

        synchronized (engines) {
            for (Iterator iter = engines.values().iterator(); iter.hasNext()
                    && result == null;) {
                ComponentLifeCycle lifeCycle = (ComponentLifeCycle) iter.next();

                if (lifeCycle.getName().equals(name)) {
                    result = lifeCycle;
                }
            }
        }
        return result;
    }

    /**
     * Unregister the binding component of the JMX server
     *
     * @throws ManagementException
     */
    public void unregisterBindingComponent(ObjectName name)
            throws ManagementException {
        Map bindings = getBindingCompoLifeCycles();

        synchronized (bindings) {
            try {
                adminAttributes.unregisterMBean(name);

                bindings.remove(name);
            } catch (Exception e) {
                throw new ManagementException(e);
            }
        }
    }

    /**
     * Unregister the engine component of the JMX server
     *
     * @throws ManagementException
     */
    public void unregisterEngineComponent(ObjectName name)
            throws ManagementException {
        Map engines = getEngineCompoLifeCycles();

        synchronized (engines) {
            try {
                adminAttributes.unregisterMBean(name);

                engines.remove(name);
            } catch (Exception e) {
                throw new ManagementException(e);
            }
        }
    }

    // ////////////////////////////
    // manage installers
    // ////////////////////////////
    /**
     * Register an installer mbean in the JMX server representing the specified
     * component installer.
     *
     * @param installerMBean
     * @throws ManagementException
     */
    public ObjectName registerInstaller(Installer installerMBean)
            throws ManagementException {

        ObjectName installerMBeanName = installerMBean.getMBeanName();

        Map<ObjectName, Installer> installers = getInstallers();

        synchronized (installers) {
            try {
                installers.put(installerMBeanName, installerMBean);
                try {
                    adminAttributes.registerMBean(installerMBean,
                            installerMBeanName);
                } catch (Exception e1) {
                    installers.remove(installerMBeanName);
                    throw e1;
                }
            } catch (Exception e) {
                throw new ManagementException(e);
            }
        }
        return installerMBeanName;
    }

    /**
     * Return the component installer associated with this componant name.
     *
     * @param name
     *            the component name
     * @return The related installer, or null
     */
    @SuppressWarnings("unchecked")
    public ObjectName getInstallerByName(String name) {
        log.call();

        ObjectName result = null;

        Map<ObjectName, Installer> installers = getInstallers();

        synchronized (installers) {
            for (Iterator iter = installers.entrySet().iterator(); iter
                    .hasNext()
                    && result == null;) {
                Entry<ObjectName, Installer> entry = (Entry<ObjectName, Installer>) iter
                        .next();
                ObjectName objName = entry.getKey();
                Installer installer = entry.getValue();

                if (installer.getComponentName().equals(name)) {
                    result = objName;
                }
            }
        }
        return result;
    }

    /**
     * Return the component installer MBean associated with this componant name.
     *
     * @param name
     *            the installer Object name
     * @return The related installer, or null
     */
    public Installer getInstallerMBeanByName(ObjectName name) {
        return installers.get(name);
    }

    /**
     * Unregister the component installer of the JMX server
     *
     * @throws ManagementException
     */
    public void unregisterInstaller(ObjectName name) throws ManagementException {
        Map<ObjectName, Installer> installers = getInstallers();

        synchronized (installers) {
            try {
                adminAttributes.unregisterMBean(name);

                installers.remove(name);
            } catch (Exception e) {
                throw new ManagementException(e);
            }
        }
    }

    // ////////////////////////////
    // manage service assemblies
    // ////////////////////////////
    /**
     * Register a ServiceAssemblyLifeCycle into the a synchronized Map<String,ServiceAssemblyLifeCycle>
     *
     * @param saName
     * @param serviceAssemblyLifeCycle
     */
    public void registerServiceAssembly(String saName,
            ServiceAssemblyLifeCycle serviceAssemblyLifeCycle) {
        Map<String, ServiceAssemblyLifeCycle> sas = getServiceAssemblyLifeCycles();
        synchronized (sas) {
            sas.put(saName, serviceAssemblyLifeCycle);
        }
    }

    /**
     * Return the service assembly associated with this name.
     *
     * @param saName
     * @return The related ServiceAssemblyLifeCycle, or null
     */
    public ServiceAssemblyLifeCycle getServiceAssemblyByName(String saName) {
        log.call();

        ServiceAssemblyLifeCycle result = null;

        Map<String, ServiceAssemblyLifeCycle> sas = this
                .getServiceAssemblyLifeCycles();

        synchronized (sas) {
            result = sas.get(saName);
        }
        return result;
    }

    /**
     * Unregister the ServiceAssembly life cycle from the sync map
     *
     * @throws ManagementException
     */
    public void unregisterServiceAssembly(String saName)
            throws ManagementException {
        Map sas = getServiceAssemblyLifeCycles();

        synchronized (sas) {
            try {
                sas.remove(saName);
            } catch (Exception e) {
                throw new ManagementException(e);
            }
        }
    }

    // ////////////////////////////
    // manage shared library
    // ////////////////////////////
    /**
     * Register a SharedLibraryLifeCycle into the a synchronized Map<String,SharedLibraryLifeCycle>
     *
     * @param slName
     * @param sharedLibraryLifeCycle
     */
    public void registerSharedLibrary(String slName,
            SharedLibraryLifeCycle sharedLibraryLifeCycle) {
        Map<String, SharedLibraryLifeCycle> sls = getSharedLibraryLifeCycles();
        synchronized (sls) {
            sls.put(slName, sharedLibraryLifeCycle);
        }
    }

    /**
     * Return the shared library associated with this name.
     *
     * @param slName
     * @return The related SharedLibraryLifeCycle, or null
     */
    public SharedLibraryLifeCycle getSharedLibraryByName(String slName) {
        log.call();

        SharedLibraryLifeCycle result = null;

        Map<String, SharedLibraryLifeCycle> sls = this
                .getSharedLibraryLifeCycles();

        synchronized (sls) {
            result = sls.get(slName);
        }
        return result;
    }

    /**
     * Unregister the Shared library life cycle from the sync map
     *
     * @throws ManagementException
     */
    public void unregisterSharedLibrary(String slName)
            throws ManagementException {
        Map sls = getSharedLibraryLifeCycles();

        synchronized (sls) {
            try {
                sls.remove(slName);
            } catch (Exception e) {
                throw new ManagementException(e);
            }
        }
    }

    // //////////////////////////////
    // / getters and setters
    // //////////////////////////////

    /**
     * Return the <code>MBeanNamesImpl</code> element which allows the
     * creation of mBeans names.
     *
     * @return
     */
    public MBeanNames getMBeanNames() {
        return mBeanNames;
    }

    /**
     * Return the JMX server used by the JBI container.
     *
     * @return
     */
    public MBeanServer getMBeanServer() {
        return adminAttributes;
    }

    /**
     * Return the synchronized map of binding components.
     *
     * @return
     */
    public Map<ObjectName, ComponentLifeCycle> getBindingCompoLifeCycles() {
        return bindingCompoLifeCycles;
    }

    /**
     * Return the synchronized map of engine components.
     *
     * @return
     */
    public Map<ObjectName, ComponentLifeCycle> getEngineCompoLifeCycles() {
        return engineCompoLifeCycles;
    }

    /**
     * Return the synchronized map of installers.
     *
     * @return
     */
    public Map<ObjectName, Installer> getInstallers() {
        return installers;
    }

    /**
     * Return the synchronized map of deployed service asse mbly.
     *
     * @return
     */
    public Map<String, ServiceAssemblyLifeCycle> getServiceAssemblyLifeCycles() {
        return serviceAssemblyLifeCycles;
    }

    /**
     * Return the synchronized map of deployed shared library.
     *
     * @return
     */
    public Map<String, SharedLibraryLifeCycle> getSharedLibraryLifeCycles() {
        return this.sharedLibraryLifeCycles;
    }

}
