/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: EndpointService.java 11:27:27 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.endpoint;

import javax.jbi.JBIException;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import org.objectweb.petals.jbi.messaging.endpoint.AbstractEndpoint;
import org.objectweb.petals.jbi.messaging.endpoint.ConsumerEndpoint;
import org.objectweb.petals.jbi.messaging.endpoint.InternalEndpoint;
import org.w3c.dom.Document;

/**
 * This class is used to manage everything that deals with endpoint in the
 * distributed environment
 *
 * @author ddesjardins - eBMWebsourcing
 */
public interface EndpointService {

    /**
     * Activates the named entpoint
     *
     * @param serviceName
     *            qualified name of the service endpoint; must be non null
     * @param endpointName
     *            the name of the endpoint to be activated; must be non null
     * @param address
     *            the consumer endpoint; must be non null
     * @return internal endpoint
     * @throws JBIException
     *             if a problem occurs in the registry
     */
    InternalEndpoint activateEndpoint(QName serviceName, String endpointName,
        ConsumerEndpoint address) throws JBIException;

    /**
     * Register a connection
     *
     * @param consInterface
     *            consumer interface name; must be non null
     * @param provService
     *            provider service name; must be non null
     * @param provEndpoint
     *            provider endpoint name; must be non null
     * @throws JBIException
     *             if a problem occurs in the registry
     */
    void createConnection(QName consInterface, QName provService,
        String provEndpoint) throws JBIException;

    /**
     * Register a connection
     *
     * @param consService
     *            consumer service name; must be non null
     * @param consEndpoint
     *            consumer endpoint name; must be non null
     * @param provService
     *            provider service name; must be non null
     * @param provEndpoint
     *            provider endpoint name; must be non null
     * @throws JBIException
     *             if a problem occurs in the registry
     */
    void createConnection(QName consService, String consEndpoint,
        QName provService, String provEndpoint) throws JBIException;

    /**
     * Deactivates the given endpoint
     *
     * @param endpoint
     *            endpoint to be deactivated; must be non null
     * @throws JBIException
     *             if the endpoint cannot be deactivated
     */
    void deactivateEndpoint(ServiceEndpoint endpoint) throws JBIException;

    /**
     * Delete a connection
     *
     * @param consInterface
     *            consumer interface name; must be non null
     * @param provService
     *            provider service name; must be non null
     * @param provEndpoint
     *            provider endpoint name; must be non null
     * @throws JBIException
     *             if the connection can not be deleted
     */
    void deleteConnection(QName consInterface, QName provService,
        String provEndpoint) throws JBIException;

    /**
     * Delete a connection
     *
     * @param consService
     *            consumer service name; must be non null
     * @param consEndpoint
     *            consumer endpoint name; must be non null
     *
     * @param provService
     *            provider service name; must be non null
     * @param provEndpoint
     *            provider endpoint name; must be non null
     * @throws JBIException
     *             if the connection can not be deleted
     */
    void deleteConnection(QName consService, String consEndpoint,
        QName provService, String provEndpoint) throws JBIException;

    /**
     * Deregister external endpoint
     *
     * @param endpoint
     *            external to be deregistered; must be non null
     * @throws JBIException
     *             if the endpoint cannot be deregistered
     */
    void deregisterExternalEndpoint(ServiceEndpoint endpoint)
        throws JBIException;

    /**
     * Get the service endpoint for the named activated endpoint, if any.
     * if the endpoint is a link, resolve it and return the real endpoint
     * @param service
     *            qualified name of the endpoint's service; must be non null
     * @param name
     *            name of the endpoint; must be non null
     * @return the named endpoint, or null if the named endpoint is not
     *         activated
     */
    ServiceEndpoint getEndpoint(QName service, String name);

    /**
     * Get the service endpoint for the named activated endpoint, if any
     *
     * @param service
     *            qualified name of the endpoint's service; must be non null
     * @param name
     *            name of the endpoint; must be non null
     * @param resolveLink
     *            if the Endpoint is a link, resolve it (true) or return the link (false)
     * @return the named endpoint, or null if the named endpoint is not
     *         activated
     */
    ServiceEndpoint getEndpoint(QName service, String name,boolean resolveLink);

    /**
     * Return the descriptor for a specified endpoint
     *
     * @param endpoint
     *            endpoint on which we want to retrieve the descriptor; must be
     *            non null
     * @return Document metadata describing the endpoint, or null if metadata is
     *         unavailable
     */
    Document getEndpointDescriptorForEndpoint(ServiceEndpoint endpoint)
        throws JBIException;

    /**
     * Queries the registry for external endpoints that implements the given
     * interface name
     *
     * @param interfaceName
     *            qualified name of interface implemented by the endpoints; must
     *            be non null
     * @return an array of availabe endpoints for the specified interface name;
     *         must be non null, may be empy
     */
    ServiceEndpoint[] getExternalEndpointsForInterface(QName interfaceName);

    /**
     * Queries the registry for external endpoints that are part of the given
     * service name
     *
     * @param serviceName
     *            qualified name of service that contains endpoints; must be non
     *            null
     * @return an array of availabe endpoints for the specified service name;
     *         must be non null, may be empy
     */
    ServiceEndpoint[] getExternalEndpointsForService(QName serviceName);

    /**
     * Return the interfaces for the specified endpoint
     *
     * @param endpoint
     *            endpoint on which we want to retrieve the interface names;
     *            must be non null
     * @return Array of interfaces QName
     */
    QName[] getInterfacesForEndpoint(AbstractEndpoint endpoint);

    /**
     * Queries the registry for active endpoints that implements the given
     * interface
     *
     * @param interfaceName
     *            qualified interface/portType that is implementsed by the
     *            endpoints; if null then all activated endpoints in the JBI
     *            environement must be returned
     * @return an array of available endpoints for the specified interface name;
     *         must be non null, may be empty
     */
    ServiceEndpoint[] getInternalEndpointsForInterface(QName interfaceName);

    /**
     * Queries the registry for active endpoints belonging to the given service
     *
     * @param serviceName
     *            qualified name of the service that the endpoints are part of;
     *            must be non null
     * @return an array of available endpoints for the specified service name;
     *         must be non null, may be empty
     */
    ServiceEndpoint[] getInternalEndpointsForService(QName serviceName);

    /**
     * Query a component to check if it is ready to accept the exchange as a
     * consumer
     *
     * @param internalEndpoint
     *            the endpoint to be used by the consumer; must be non null
     * @param exchange
     *            the proposed message exchange to be perfomed; must be non null
     * @return true if the provider component can perform the given exchange
     *         with the described consumer
     */
    boolean isExchangeWithConsumerOkayForComponent(
        InternalEndpoint internalEndpoint, MessageExchange exchange);

    /**
     * Query a component to check if it is ready to accept the exchange as a
     * provider
     *
     * @param internalEndpoint
     *            the endpoint to be used by the provider; must be non null
     * @param exchange
     *            the message exchange that is to be performed
     * @return true if this consumer component can interact with the described
     *         provider to perform the given exchange
     */
    boolean isExchangeWithProviderOkayForComponent(
        InternalEndpoint internalEndpoint, MessageExchange exchange);

    /**
     * Register this given external endpoint
     *
     * @param externalEndpoint
     *            the external endpoint to be registered; must be non null
     * @throws JBIException
     *             if an external endpoint wit the same name is already
     *             registered
     */
    void registerExternalEndpoint(ServiceEndpoint externalEndpoint)
        throws JBIException;


}
