/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SystemRecoveryServiceImpl.java 154 2 mai 2006 ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.recovery;

import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

import javax.jbi.JBIException;
import javax.jbi.management.LifeCycleMBean;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.management.ReflectionException;

import org.apache.commons.io.FileUtils;
import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.petals.PetalsException;
import org.objectweb.petals.container.ContainerService;
import org.objectweb.petals.container.lifecycle.Installer;
import org.objectweb.petals.container.lifecycle.ServiceAssemblyLifeCycle;
import org.objectweb.petals.jbi.descriptor.JBIDescriptor;
import org.objectweb.petals.jbi.management.ManagementException;
import org.objectweb.petals.jbi.management.admin.AdminServiceMBean;
import org.objectweb.petals.jbi.management.autoload.AutoLoaderService;
import org.objectweb.petals.jbi.management.endpoint.EndpointService;
import org.objectweb.petals.jbi.management.task.TaskContextConstants;
import org.objectweb.petals.jbi.management.task.deployment.DeploymentTaskFactory;
import org.objectweb.petals.jbi.management.task.deployment.DeploymentUtils;
import org.objectweb.petals.jbi.management.task.installation.InstallationContextConstants;
import org.objectweb.petals.jbi.management.task.installation.InstallationTaskFactory;
import org.objectweb.petals.jbi.management.task.installation.InstallationUtils;
import org.objectweb.petals.jbi.management.util.PackageHandler;
import org.objectweb.petals.jbi.messaging.routing.Router;
import org.objectweb.petals.kernel.server.FractalHelper;
import org.objectweb.petals.platform.classloader.ClassLoaderService;
import org.objectweb.petals.platform.repository.RepositoryService;
import org.objectweb.petals.platform.repository.RepositoryServiceImpl.EntityType;
import org.objectweb.petals.platform.systemstate.ComponentState;
import org.objectweb.petals.platform.systemstate.ServiceAssemblyState;
import org.objectweb.petals.platform.systemstate.SharedLibraryState;
import org.objectweb.petals.platform.systemstate.SystemStateService;
import org.objectweb.petals.processor.TaskProcessor;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.api.Logger;

/**
 * Allows to restore system state after shutdown or crash.
 *
 * @author Olivier Fabre - EBM Websourcing
 *
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = org.objectweb.petals.jbi.management.recovery.SystemRecoveryService.class))
public class SystemRecoveryServiceImpl implements SystemRecoveryService {

	/**
	 * logger wrapper
	 */
	protected LoggingUtil log;

	/**
	 * Monolog Logger instance.
	 */
	@Monolog(name = "logger")
	protected Logger logger;

	private DeploymentTaskFactory deploymentTaskFactory;

    protected InstallationTaskFactory installationTaskFactory;

    /**
     * Package Handler.
     */
    protected PackageHandler packageHandler;

	/**
	 * The endpointService is used to manage all the endpoint interfaces in a
	 * distributed context
	 */
	@Requires(name = "endpoint", signature = org.objectweb.petals.jbi.management.endpoint.EndpointService.class)
	protected EndpointService endpointService;

    /**
     * Communication :: JMX Agent Admin.
     */
    @Requires(name = "admin", signature = org.objectweb.petals.jbi.management.admin.AdminServiceMBean.class)
    protected AdminServiceMBean adminService;

    /**
     * JBI management : AutoLoader Service.
     */
    @Requires(name = "autoloader", signature = org.objectweb.petals.jbi.management.autoload.AutoLoaderService.class)
    protected AutoLoaderService autoLoaderService;

    /**
	 * JBI messaging : Router Service.
	 */
	@Requires(name = "router", signature = org.objectweb.petals.jbi.messaging.routing.Router.class)
	protected Router router;

    /**
     * Platform :: Loader Service
     */
    @Requires(name = "classloader", signature = org.objectweb.petals.platform.classloader.ClassLoaderService.class)
    protected ClassLoaderService loaderService;

    /**
     * Platform :: Repository Service.
     */
    @Requires(name = "repository", signature = org.objectweb.petals.platform.repository.RepositoryService.class)
	protected RepositoryService repositoryService;

    /**
     * Platform :: SystemState Service.
     */
    @Requires(name = "systemstate", signature = org.objectweb.petals.platform.systemstate.SystemStateService.class)
	protected SystemStateService systemStateService;

	/**
	 * Container :: Container Service.
	 */
	@Requires(name = "container", signature = org.objectweb.petals.container.ContainerService.class)
	protected ContainerService container;

	protected void createRepositoryDirectories() {
		File componentsRep = null;
		File serviceAssRep = null;
		File sharedLibRep = null;



		componentsRep = repositoryService
		.getComponentsDirectory();
		serviceAssRep = repositoryService
		.getServiceAssembliesDirectory();
		sharedLibRep = repositoryService.getSharedLibsDirectory();

		// Create components directory in Petals repository if it doesn't exist
		if (!componentsRep.exists()) {
			if (!componentsRep.mkdirs()) {
				String msg = "Failed to create components dir in Petals repository";
				log.error(msg);
				throw new RuntimeException(msg);
			}
		}

		// Create sharedlibs directory in Petals repository if it doesn't exist
		if (!sharedLibRep.exists()) {
			if (!sharedLibRep.mkdirs()) {
				String msg = "Failed to create sharedlibs dir in Petals repository";
				log.error(msg);
				throw new RuntimeException(msg);
			}
		}

		// Create serviceassemblies directory in Petals repository if it doesn't
		// exist
		if (!serviceAssRep.exists()) {
			if (!serviceAssRep.mkdirs()) {
				String msg = "Failed to create serviceassemblies dir in Petals repository";
				log.error(msg);
				throw new RuntimeException(msg);
			}
		}
	}

	public boolean recoverAllComponent() throws ManagementException {
		log.start();

		/*
		 * Retrieve all component directories in the components petals
		 * repository. This list will be used to delete components that have
		 * been unzipped in the Petals repository but no more persisted in the
		 * system recory file (system-state.xml). These unknown component
		 * directories will be put in the Petals "lost+found" directory.
		 */

		List<File> unknownDirs = new ArrayList<File>(Arrays.asList(
				repositoryService.getComponentsDirectory().listFiles()));



		/*
		 * Retrieve all components to recover
		 */

		List<ComponentState> cps = systemStateService
		.retrieveAllComponentStates();

		/*
		 * Recover state for each component
		 */
		for (ComponentState state : cps) {

			// The component may be corrupted and be in the UNKNOWN state
			// If it is the case, we just skip it
			if (state.getInstallState().equals(LifeCycleMBean.UNKNOWN)) {
				log.warning("A component is in UNKNOWN state : " + state.getName() + ". Skip it.");
			} else {
				URI installationRoot = null;
				try {
					installationRoot = packageHandler.processAndGetPackageURI(state
							.getInstallURL(), true);

					JBIDescriptor descriptor = packageHandler.loadDescriptor(state
							.getName(), EntityType.COMPONENT_TYPE);

					recoverComponent(installationRoot, descriptor, state
							.getInstallState(), state.getLifecycleState());
				} catch (PetalsException e) {
					StringBuffer msgBuffer = new StringBuffer("A component can't be recovered: ");
					msgBuffer.append(state.getName());
					msgBuffer.append(".");
					try {
						systemStateService.updateComponentLifeCycleState(state.getName(), LifeCycleMBean.UNKNOWN);
						systemStateService.updateComponentInstallationState(state.getName(), LifeCycleMBean.UNKNOWN);
						msgBuffer.append("\nIts state changes to UNKNOWN.");
					} catch (Exception e1) {
						// do nothing
					}
					msgBuffer.append("\nPlease check the repository consistency.");
					throw new ManagementException(msgBuffer.toString(), e);
				}  catch (IllegalArgumentException e) {
					StringBuffer msgBuffer = new StringBuffer("A component consistency is corrupted: ");
					msgBuffer.append(state.getName());
					msgBuffer.append(".");
					try {
						systemStateService.updateComponentLifeCycleState(state.getName(), LifeCycleMBean.UNKNOWN);
						systemStateService.updateComponentInstallationState(state.getName(), LifeCycleMBean.UNKNOWN);
						msgBuffer.append("\nIts state changes to UNKNOWN.");
					} catch (Exception e1) {
						// do nothing
					}
					msgBuffer.append("\nPlease check the repository consistency.");
					throw new ManagementException(msgBuffer.toString(), e);
				}

				// Delete component from the unknownDirs list because this
				// component is known now...
				if (unknownDirs != null) {
					File knownComp = new File(installationRoot).getParentFile();
					if (knownComp.exists()) {
						unknownDirs.remove(knownComp);
					}
				}
			}
		}

		// Now, move all unknown components from Petals repository to
		// "lost+found" dir
		moveUnknownDirsToLostPlusFoundDir(unknownDirs);

		log.end();
		return true;
	}

	public boolean recoverAllServiceAssembly() throws ManagementException {
		log.start();

		/*
		 * Retrieve all sa directories in the sa petals repository. This list
		 * will be used to delete sa that have been unzipped in the Petals
		 * repository but no more persisted in the system recory file
		 * (system-state.xml). These unknown sa directories will be put in the
		 * Petals "lost+found" directory.
		 */

		List<File> unknownDirs= new ArrayList<File>(Arrays.asList(
				repositoryService.getServiceAssembliesDirectory().listFiles()));



		/*
		 * Retrieve all service assemblies to recover
		 */

		List<ServiceAssemblyState> sas = systemStateService
		.retrieveAllServiceAssemblyStates();

		/*
		 * Recover state for each service assembly
		 */

		for (ServiceAssemblyState state : sas) {

		    //The component may be corrupted and be in the UNKNOWN state
            // If it is the case, we just skip it
            if (state.getLifecycleState().equals(LifeCycleMBean.UNKNOWN)) {
                log.warning("A service assembly is in UNKNOWN state : " + state.getName() + ". Skip it.");
            } else {
                URI installationRoot = null;
    			try {

    				installationRoot = packageHandler.processAndGetPackageURI(state
    						.getInstallURL(), true);

    				JBIDescriptor descriptor = packageHandler.loadDescriptor(state
    						.getName(), EntityType.SA_TYPE);

    				recoverServiceAssembly(installationRoot, descriptor, state
    						.getLifecycleState());
                } catch (IllegalArgumentException e) {
                    StringBuffer msgBuffer = new StringBuffer("A service assembly consistency is corrupted: ");
                    msgBuffer.append(state.getName());
                    msgBuffer.append(".");
                    try {
                        systemStateService.updateServiceAssemblyState(state.getName(), LifeCycleMBean.UNKNOWN);
                        msgBuffer.append("\nIts state changes to UNKNOWN.");
                    } catch (Exception e1) {
                        // do nothing
                    }
                    msgBuffer.append("\nPlease check the repository consistency.");
                    throw new ManagementException(msgBuffer.toString(), e);
                } catch (Exception e) {
                    StringBuffer msgBuffer = new StringBuffer("A service assembly can't be recovered: ");
                    msgBuffer.append(state.getName());
                    msgBuffer.append(".");
                    try {
                        systemStateService.updateServiceAssemblyState(state.getName(), LifeCycleMBean.UNKNOWN);
                        msgBuffer.append("\nIts state changes to UNKNOWN.");
                    } catch (Exception e1) {
                        // do nothing
                    }
                    msgBuffer.append("\nPlease check the repository consistency.");
                    throw new ManagementException(msgBuffer.toString(), e);
                }


    			// Delete sa from the unknownDirs list because this
    			// sa is known now...
    			if (unknownDirs != null) {
    				File knownComp = new File(installationRoot).getParentFile();
    				if (knownComp.exists()) {
    					unknownDirs.remove(knownComp);
    				}
    			}
            }
		}

		// Now, move all unknown sa from Petals repository to
		// "lost+found" dir
		moveUnknownDirsToLostPlusFoundDir(unknownDirs);

		log.end();
		return true;
	}

	public boolean recoverAllSharedLibrary() throws ManagementException {
		log.start();
		String msg = null;

		/*
		 * Retrieve all sl directories in the sl petals repository. This list
		 * will be used to delete sl that have been unzipped in the Petals
		 * repository but no more persisted in the system recory file
		 * (system-state.xml). These unknown sl directories will be put in the
		 * Petals "lost+found" directory.
		 */

		List<File> unknownDirs = new ArrayList<File>(Arrays.asList(
				repositoryService.getSharedLibsDirectory().listFiles()));


		/*
		 * Retrieve all shared library to recover
		 */

		List<SharedLibraryState> sls = systemStateService
		.retrieveAllSharedLibraryStates();

		/*
		 * Recover state for each shared library
		 */
		for (SharedLibraryState state : sls) {
			URI installationRoot = null;
			try {

				installationRoot = packageHandler.processAndGetPackageURI(state
						.getInstallURL(), true);

				JBIDescriptor descriptor = packageHandler.loadDescriptor(state
						.getName(), EntityType.SL_TYPE);

				recoverSharedLib(installationRoot, descriptor);
			} catch (PetalsException e) {
				msg = "A Shared lib can't be recovered : " + state.getName();
				log.error(msg, e);
				throw new ManagementException(msg, e);
			}

			// Delete sl from the unknownDirs list because this
			// sl is known now...
			if (unknownDirs != null) {
				File knownComp = new File(installationRoot).getParentFile();
				if (knownComp.exists()) {
					unknownDirs.remove(knownComp);
				}
			}
		}

		// Now, move all unknown sl from Petals repository to
		// "lost+found" dir
		moveUnknownDirsToLostPlusFoundDir(unknownDirs);

		log.end();
		return true;
	}

	/**
	 * Create the lost+found dir in the petals file system if it doesn't exist,
	 * and cleanup it if it exists
	 *
	 */
	protected void createLostPlusFoundDir() {

		File lostPlusFoundFile = new File(repositoryService
				.getLostPlusFoundDirectory());

		// Cleanup lost+found directory at Petals startup
		if (lostPlusFoundFile.exists()) {
			try {
				FileUtils.forceDelete(lostPlusFoundFile);
				if (!lostPlusFoundFile.mkdirs()) {
					String msg = "Failed to create lost+found directory, "
						+ "following Petals repository cleanup will fail!";
					log.warning(msg);
				}
			} catch (IOException e) {
				log
				.warning("\"lost+found\" can't be deleted, "
						+ "following Petals repository cleanup can have some troubles!");
			}
		} else {
			if (!lostPlusFoundFile.mkdirs()) {
				String msg = "Failed to create lost+found directory, "
					+ "following Petals repository cleanup will fail!";
				log.warning(msg);
			}
		}
	}

	/**
	 * Move all unknown components from Petals repository to "lost+found" dir
	 *
	 * @param unknownDirs
	 *            the {@link List} of unknown dirs that have to be moved.
	 */
	protected void moveUnknownDirsToLostPlusFoundDir(List<File> unknownDirs) {
		if (unknownDirs != null) {
			for (File unknownDir : unknownDirs) {
				if(!unknownDir.isFile()) {
					if (!unknownDir.exists()) {
						log.warning("The unknown dir ("
								+ unknownDir.getAbsolutePath() + ") doesn't exist.");
					}
					boolean moveDir = unknownDir.renameTo(new File(new File(
							repositoryService.getLostPlusFoundDirectory()), unknownDir
							.getName()));
					if (!moveDir) {
						log.warning("Failed to move unknown directory ("
								+ unknownDir.getAbsolutePath()
								+ ") to lost+found directory");
					}
				}
			}
		}
	}

	/**
	 * Recover the given component. Recovering a component consist in redoing
	 * installation process of a component (see
	 * <code>InstallationServiceMBean</code>), calling the "install" method
	 * on the created Installer and putting the component in its previous state
	 * by calling "start" or/and "stop" on the created component lifecycle.
	 *
	 * @param installationRoot
	 * @param installState
	 * @param lifecycleState
	 * @throws ManagementException
	 */
	protected void recoverComponent(URI installRoot, JBIDescriptor descriptor,
			String installState, String lifecycleState) throws ManagementException {
		log.start();
		String msg = null;

		String compName = descriptor.getComponent().getIdentification()
		.getName();

		/*
		 * Redo install
		 */

		/*
		 * Create installation execution context
		 */
		HashMap<String, Object> installationContext = createRecoveryContext(
				descriptor, installRoot);

		/*
		 * Recover component installer
		 */
		recoverComponent(installationContext);

		try {

			if (Installer.INSTALLED.equals(installState)) {

				/*
				 * Redo install
				 */
				MBeanServer beanServer = adminService.getMBeanServer();
				Object[] nullObjects = new Object[0];
				String[] nullStrings = new String[0];
				ObjectName clcName = (ObjectName) beanServer
				.invoke(
						(ObjectName) installationContext
						.get(InstallationContextConstants.INSTALLER_OBJECT_NAME),
						"install", nullObjects, nullStrings);

				if (LifeCycleMBean.STOPPED.equals(lifecycleState)) {
					/*
					 * Redo start followed by stop
					 */
					beanServer.invoke(clcName, "start", nullObjects,
							nullStrings);
					beanServer
					.invoke(clcName, "stop", nullObjects, nullStrings);
				} else if (LifeCycleMBean.STARTED.equals(lifecycleState)) {
					/*
					 * Redo start
					 */
					beanServer.invoke(clcName, "start", nullObjects,
							nullStrings);
				}

			}
		} catch (InstanceNotFoundException e) {
			msg = "Component can't be recovered : " + compName;
			log.error(msg, e);
			throw new ManagementException(msg, e);
		} catch (MBeanException e) {
			msg = "Component can't be recovered : " + compName;
			log.error(msg, e);
			throw new ManagementException(msg, e);
		} catch (ReflectionException e) {
			msg = "Component can't be recovered : " + compName;
			log.error(msg, e);
			throw new ManagementException(msg, e);
		}
		log.end();
	}

	/**
	 * Create recovery context for component and shared libraries
	 *
	 * @param descriptor
	 * @param installRoot
	 * @return
	 */
	protected HashMap<String, Object> createRecoveryContext(
			JBIDescriptor descriptor, URI installRoot) {
		HashMap<String, Object> installationContext = new HashMap<String, Object>();
		installationContext.put(TaskContextConstants.DESCRIPTOR, descriptor);
		installationContext.put(TaskContextConstants.INSTALL_ROOT, installRoot);
		installationContext.put(TaskContextConstants.UNZIP_ROOT, new File(
				installRoot));
		return installationContext;
	}

	/**
	 * Recover the given Service Assembly. Recovering a Service Assembly consist
	 * in redoing deployment process of a Service Assembly (see
	 * <code>DeploymentServiceMBean</code>) and putting the Service Assembly
	 * in its previous state by calling "start" or/and "stop" on the created
	 * component lifecycle.
	 *
	 * @param installationRoot
	 * @param lifecycleState
	 * @throws ManagementException
	 */
	protected void recoverServiceAssembly(URI installRoot,
			JBIDescriptor descriptor, String lifecycleState) throws Exception {
		log.start();
		String msg = null;

		String saName = descriptor.getServiceAssembly().getIdentification()
		.getName();
		try {

			/*
			 * Redo deployment
			 */

			// Create deployment execution context
			HashMap<String, Object> deploymentContext = createDeploymentContext(
					installRoot, descriptor);

			// Recover the given service assembly
			recoverSA(deploymentContext);

			if (LifeCycleMBean.STOPPED.equals(lifecycleState)) {
				/*
				 * Redo start followed by stop
				 */
				ServiceAssemblyLifeCycle salc = adminService
				.getServiceAssemblyByName(saName);
				//salc.start();
				//salc.stop();
				FractalHelper.getFractalHelper().startComponent(salc.getComponent());
				FractalHelper.getFractalHelper().stopComponent(salc.getComponent());
			} else if (LifeCycleMBean.STARTED.equals(lifecycleState)) {
				/*
				 * Redo start
				 */
				ServiceAssemblyLifeCycle salc = adminService
				.getServiceAssemblyByName(saName);
				FractalHelper.getFractalHelper().startComponent(salc.getComponent());
				//salc.start();
			}

		} catch (PetalsException e) {
			msg = "ServiceAssembly can't be recovered : " + saName;
			log.error(msg, e);
			throw new ManagementException(msg, e);
		} catch (JBIException e) {
			msg = "ServiceAssembly can't be recovered : " + saName;
			log.error(msg, e);
			throw new ManagementException(msg, e);
		}
		log.end();
	}

	/**
	 * Create the deployment context
	 *
	 * @param descriptor
	 *            the sa jbi descriptor
	 * @return the Deployment Context HashMap
	 */
	protected HashMap<String, Object> createDeploymentContext(URI installRoot,
			JBIDescriptor descriptor) {
		HashMap<String, Object> deploymentContext = new HashMap<String, Object>();
		deploymentContext.put(TaskContextConstants.UNZIP_ROOT, new File(
				installRoot));
		deploymentContext.put(TaskContextConstants.DESCRIPTOR, descriptor);
		return deploymentContext;
	}

	/**
	 * Recover the given shared library. Recovering a shared lib consist in
	 * redoing installation process of a shared lib (see
	 * <code>InstallationServiceMBean</code>).
	 *
	 * @param installationRoot
	 * @throws ManagementException
	 */
	protected void recoverSharedLib(URI installationRoot,
			JBIDescriptor descriptor) throws ManagementException {
		log.start();

		/*
		 * Create installation execution context
		 */
		HashMap<String, Object> installationContext = createRecoveryContext(
				descriptor, installationRoot);

		/*
		 * Redo install of the shared lib
		 */
		recoverSharedLib(installationContext);

		log.end();
	}

	public boolean recoverAllEntities() throws ManagementException {
		log.start();

		recoverAllSharedLibrary();
		recoverAllComponent();
		recoverAllServiceAssembly();

		log.end();
		return true;
	}

	/**
	 *
	 */
	@LifeCycle(on = LifeCycleType.START)
	protected void start() {
		log = new LoggingUtil(logger);
		log.start();

		/*
		 * Initialize package handler
		 */
		packageHandler = new PackageHandler(logger, repositoryService);

		/*
		 * Initialize task factory
		 */
		/*
		 * Initialize installation task factory
		 */
		installationTaskFactory = new InstallationTaskFactory(packageHandler,
				systemStateService, repositoryService, log,
				logger, adminService, router, loaderService, container, autoLoaderService);

		/*
		 * Initialize DeploymentTaskFactory
		 */
		deploymentTaskFactory = new DeploymentTaskFactory(packageHandler,
				endpointService, null, systemStateService, repositoryService, log,
				adminService, container, autoLoaderService);

		createLostPlusFoundDir();
		createRepositoryDirectories();
		log.end();
	}

	/**
	 * Process the deployment of a given Service assembly
	 *
	 * @param saDeploymentContext
	 *            the deployment context of the given SA
	 * @throws Exception
	 *             if an error occured during deployment process
	 */
	private void recoverSA(HashMap<String, Object> deploymentContext)
	throws Exception {
		JBIDescriptor descriptor = (JBIDescriptor) deploymentContext
		.get(TaskContextConstants.DESCRIPTOR);

		// Create deployment processor and fill with tasks
		TaskProcessor processor = createSARecoveryProcessor(deploymentContext);

		// Launch the process
		if (!processor.process()) {
			throw new Exception("An error occured during the deployment "
					+ "of the following service assembly: "
					+ DeploymentUtils.getServiceAssemblyName(descriptor));
		}
	}

	/**
	 * Create the deployment processor and add all required task for the
	 * complete recovery of a service assembly
	 *
	 * @param deploymentContext
	 *            the deployment context
	 * @return the filled deployment processor
	 */
	protected TaskProcessor createSARecoveryProcessor(
			HashMap<String, Object> deploymentContext) {
		TaskProcessor processor = new TaskProcessor(deploymentContext, log);

		processor.addTask(deploymentTaskFactory.getSaCheckPackageTask());
		processor.addTask(deploymentTaskFactory
				.getCreateXMLDeploymentResultTask());
		processor.addTask(deploymentTaskFactory
				.getRetrieveSAAndSUInstallRootsTask());
		processor.addTask(deploymentTaskFactory.getRegisterSALifeCycleTask());
		processor.addTask(deploymentTaskFactory.getDeployAllSUTask());

		processor.addTask(deploymentTaskFactory.getRegisterAllConnectionTask());
		processor.addTask(deploymentTaskFactory.getSuccessTask());

		return processor;
	}

	/**
	 * Load the installer for a previously successfuly installed component. Used
	 * only by the Recovery Service.
	 *
	 * @param installationContext
	 *            the installation context for the recory process : handles
	 *            descriptor, archive uri and installRoot information
	 * @throws ManagementException
	 *             if the component installer can't be recovered
	 */
	private void recoverComponent(HashMap<String, Object> installationContext)
	throws ManagementException {
		JBIDescriptor descriptor = (JBIDescriptor) installationContext
		.get(TaskContextConstants.DESCRIPTOR);

		/*
		 * Create installation processor and fill with tasks
		 */
		TaskProcessor processor = createComponentRecoveryProcessor(installationContext);

		/*
		 * Launch the process
		 */
		if (!processor.process()) {
			throw new ManagementException(
					"An error occured during the recover "
					+ "of the following component: "
					+ InstallationUtils
					.getComponentNameFromDescriptor(descriptor));
		}
	}

	/**
	 * Create the installation processor and add all required task for the
	 * complete recovery of a component
	 *
	 * @param installationContext
	 *            the installation context
	 * @return the filled installation processor
	 */
	private TaskProcessor createComponentRecoveryProcessor(
			HashMap<String, Object> installationContext) {
		TaskProcessor processor = new TaskProcessor(installationContext, log);

		processor.addTask(installationTaskFactory
				.getCheckInstalledSLForCompTask());

		processor.addTask(installationTaskFactory
				.getCreateAndRegisterInstallerMBeanTask());

		processor.addTask(installationTaskFactory.getSuccessTask());

		return processor;
	}

	/**
	 * Reload the shared lib classloader for the given shared lib
	 *
	 * @param installationContext
	 *            the installation context for the recory process : handles
	 *            descriptor, archive uri and installRoot information
	 * @throws ManagementException
	 *             if the shared library classloader can't be recovered
	 */
	private void recoverSharedLib(HashMap<String, Object> installationContext)
	throws ManagementException {
		JBIDescriptor descriptor = (JBIDescriptor) installationContext
		.get(TaskContextConstants.DESCRIPTOR);

		/*
		 * Create installation processor and fill with tasks
		 */
		TaskProcessor processor = createSLRecoveryProcessor(installationContext);

		/*
		 * Launch the process
		 */
		if (!processor.process()) {
			throw new ManagementException(
					"An error occured during the recover "
					+ "of the following shared lib: "
					+ InstallationUtils.getSLNameFromDescriptor(descriptor));
		}

	}

	/**
	 * Create the installation processor and add all required task for the
	 * complete recovery of a shared lib
	 *
	 * @param installationContext
	 *            the installation context
	 * @return the filled installation processor
	 */
	private TaskProcessor createSLRecoveryProcessor(
			HashMap<String, Object> installationContext) {
		TaskProcessor processor = new TaskProcessor(installationContext, log);

		processor.addTask(installationTaskFactory.getSlCheckPackageTask());
		processor.addTask(installationTaskFactory.getCreateSLClassloaderTask());

		processor.addTask(installationTaskFactory.getSuccessTask());

		return processor;
	}

	/**
	 * Stop the InstallationServiceImpl
	 */
	@LifeCycle(on = LifeCycleType.STOP)
	public void stop() {
		log.start();
		log.end();
	}



}
