/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.task.installation.uninstall.sl;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import javax.jbi.management.LifeCycleMBean;

import org.objectweb.petals.PetalsException;
import org.objectweb.petals.container.lifecycle.ComponentLifeCycle;
import org.objectweb.petals.jbi.descriptor.ComponentDescription;
import org.objectweb.petals.jbi.descriptor.SharedLibraryList;
import org.objectweb.petals.jbi.management.admin.AdminServiceMBean;
import org.objectweb.petals.jbi.management.task.TaskContextConstants;
import org.objectweb.petals.processor.Task;
import org.objectweb.petals.util.LoggingUtil;

/**
 * This task checks that all components that makes use of the given sl are in
 * the shutdown state <br/> Inputs : shared lib name
 * 
 * @author ofabre
 * 
 */
public class CheckComponentShutdownStateTask implements Task {

    private AdminServiceMBean jmxAdmin;

    private LoggingUtil log;

    public CheckComponentShutdownStateTask(AdminServiceMBean jmxAdmin,
        LoggingUtil log) {
        super();
        this.jmxAdmin = jmxAdmin;
        this.log = log;
    }

    public void execute(HashMap context) throws Exception {
        String slName = (String) context.get(TaskContextConstants.ENTITY_NAME);

        if (!checkComponentShutdownStateForSL(slName)) {
            throw new PetalsException(
                "Shared library can't be uninstalled because "
                    + "at least one component isn't shutdown");
        }
    }

    /**
     * Check that all components that makes use of the given sl are in the
     * shutdown state.
     * 
     * @param slName
     *            the sl identifier
     * @return true if all components that makes use of the given sl are in the
     *         shutdown state, false otherwise
     */
    protected boolean checkComponentShutdownStateForSL(String slName) {
        boolean result = true;

        /*
         * Check for bindings
         */
        Collection<ComponentLifeCycle> bindings = jmxAdmin
            .getBindingCompoLifeCycles().values();
        for (Iterator iter = bindings.iterator(); iter.hasNext() && result;) {
            ComponentLifeCycle cycle = (ComponentLifeCycle) iter.next();

            /*
             * Retrieve the list of used sl
             */
            ComponentDescription description = cycle.getComponentDescription();
            List<SharedLibraryList> sls = description.getSharedLibraryList();

            if (sls != null) {
                for (Iterator iter2 = sls.iterator(); iter2.hasNext() && result;) {
                    SharedLibraryList sl = (SharedLibraryList) iter2.next();

                    /*
                     * If a used component isn't in the shutdown state, shared
                     * lib can't be uninstalled
                     */
                    if (slName.equals(sl.getName())
                        && !(LifeCycleMBean.SHUTDOWN.equals(cycle
                            .getCurrentState()))) {
                        log
                            .error("The shared lib can't be uninstalled because the following "
                                + "component isn't in the shutdown state : "
                                + description.getIdentification().getName());
                        result = false;
                    }
                }
            }
        }

        /*
         * Check for engines
         */
        Collection<ComponentLifeCycle> engines = jmxAdmin
            .getEngineCompoLifeCycles().values();
        for (Iterator iter = engines.iterator(); iter.hasNext() && result;) {
            ComponentLifeCycle cycle = (ComponentLifeCycle) iter.next();

            /*
             * Retrieve the list of used sl
             */
            ComponentDescription description = cycle.getComponentDescription();
            List<SharedLibraryList> sls = description.getSharedLibraryList();

            if (sls != null) {
                for (Iterator iter2 = sls.iterator(); iter2.hasNext() && result;) {
                    SharedLibraryList sl = (SharedLibraryList) iter2.next();

                    /*
                     * If a used component isn't in the shutdown state, shared
                     * lib can't be uninstalled
                     */
                    if (slName.equals(sl.getName())
                        && !(LifeCycleMBean.SHUTDOWN.equals(cycle
                            .getCurrentState()))) {
                        log
                            .error("The shared lib "
                                + slName
                                + " can't be uninstalled because"
                                + " the following component isn't in the shutdown state : "
                                + description.getIdentification().getName());
                        result = false;
                    }
                }
            }
        }

        return result;
    }

    public void undo(HashMap context) throws Exception {

    }

}
