/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id:AbstractEndpoint.java 5140 2007-09-19 09:25:19Z rnaudin $
 * -------------------------------------------------------------------------
 * $Log: AbstractEndpoint.java,v $
 */

package org.objectweb.petals.jbi.messaging.endpoint;

import java.io.ByteArrayInputStream;
import java.io.Serializable;

import javax.jbi.JBIException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.objectweb.petals.util.StringHelper;
import org.objectweb.petals.util.XMLUtil;
import org.objectweb.petals.jbi.management.endpoint.EndpointService;
import org.objectweb.petals.util.NamingHelper;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.xml.sax.InputSource;

/**
 * @version $Rev:5140 $ $Date:2007-09-19 11:25:19 +0200 (mer., 19 sept. 2007) $
 * @since Petals 1.0
 * @author alouis,<a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
public abstract class AbstractEndpoint implements ServiceEndpoint, Serializable {

    public enum EndpointType {
        CONSUMER, EXTERNAL, INTERNAL, LINKED
    }

    public static final String EPR_NAMESPACE = "http://java.sun.com/xml/ns/jbi";

    protected String componentName;

    protected String containerName;

    protected String description;

    protected String endpointName;

    protected transient EndpointService endpointService;

    protected QName[] interfaces;

    protected QName serviceName;

    protected EndpointType type;

    /**
     * Creates a new instance of {@link AbstractEndpoint}
     *
     */
    public AbstractEndpoint() {
        // Do nothing
    }

    /**
     * Creates a new instance of {@link AbstractEndpoint}
     *
     * @param serviceName
     * @param endpointName
     * @param componentName
     * @param containerName
     */
    public AbstractEndpoint(QName serviceName, String endpointName,
            String componentName, String containerName, EndpointType type,
            EndpointService endpointService) {
        super();
        this.serviceName = serviceName;
        this.endpointName = endpointName;
        this.componentName = componentName;
        this.containerName = containerName;
        this.type = type;
        this.endpointService = endpointService;
    }

    /**
     * @see javax.jbi.servicedesc.ServiceEndpoint#getAsReference(javax.xml.namespace.QName)
     */
    public DocumentFragment getAsReference(QName operationName) {
        DocumentFragment result = null;
        try {
            String reference = "<end-point-reference xmlns='" + EPR_NAMESPACE
                    + "' service-name='" + serviceName.getLocalPart()
                    + "' end-point-name='" + endpointName
                    + "'></end-point-reference>";
            InputSource source = createSource(reference);
            DocumentBuilderFactory factory = DocumentBuilderFactory
                    .newInstance();
            DocumentBuilder builder = factory.newDocumentBuilder();
            Document document = builder.parse(source);
            result = document.createDocumentFragment();
        } catch (Exception e) {
            // Do nothing
        }
        return result;
    }

    public String getComponentName() {
        return componentName;
    }

    public String getContainerName() {
        return containerName;
    }

    public String getDescription() {
        return description;
    }

    public Document getEndpointDescriptor() {
        Document doc = null;
        if (description != null) {
            doc = XMLUtil.createDocumentFromString(description);
        }
        if ((doc == null)&&(endpointService != null)) {
            try {
                doc = endpointService.getEndpointDescriptorForEndpoint(this);
            } catch (JBIException e) {
                // Do nothing
            }
        }
        return doc;
    }

    /**
     * @see javax.jbi.servicedesc.ServiceEndpoint#getEndpointName()
     */
    public String getEndpointName() {
        return endpointName;
    }

    public EndpointService getEndpointService() {
        return endpointService;
    }

    /**
     * @see javax.jbi.servicedesc.ServiceEndpoint#getInterfaces()
     */
    public QName[] getInterfaces() {
        if (interfaces != null) {
            return interfaces;
        }
        if (endpointService != null) {
            return endpointService.getInterfacesForEndpoint(this);
        }
        return new QName[0];
    }

    /**
     * @see javax.jbi.servicedesc.ServiceEndpoint#getServiceName()
     */
    public QName getServiceName() {
        return serviceName;
    }

    public EndpointType getType() {
        return type;
    }

    /**
     *
     * @param interfaceName
     * @return
     */
    public boolean implementsInterface(QName interfaceName) {

        boolean implement = false;

        for (QName anInterface : getInterfaces()) {
            if (anInterface.equals(interfaceName)) {
                implement = true;
                break;
            }
        }

        return implement;
    }

    public void setComponentName(String componentName) {
        this.componentName = componentName;
    }

    public void setContainerName(String containerName) {
        this.containerName = containerName;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public void setEndpointName(String endpointName) {
        this.endpointName = endpointName;
    }

    public void setEndpointService(EndpointService endpointService) {
        this.endpointService = endpointService;
    }

    public void setInterfaces(QName[] interfaces) {
        this.interfaces = interfaces;
    }

    public void setServiceName(QName serviceName) {
        this.serviceName = serviceName;
    }

    public void setType(EndpointType type) {
        this.type = type;
    }

    @Override
    public String toString() {
        return serviceName + " ->" + endpointName + " (" + type.name() + "):"
                + componentName + "," + containerName;
    }

    /**
     * Return true if o is a ServiceEndpoint and endpointName and serviceName
     * are equals
     */
    @Override
    public boolean equals(Object o) {
        boolean result = false;

        if (o instanceof ServiceEndpoint) {
            ServiceEndpoint other = (ServiceEndpoint) o;

            boolean endpointsEqual = StringHelper.equal(endpointName, other
                    .getEndpointName());

            String serviceNameStr = NamingHelper.qNameToString(serviceName);
            String otherServiceNameStr = NamingHelper.qNameToString(other
                    .getServiceName());
            boolean servicesEqual = StringHelper.equal(serviceNameStr,
                    otherServiceNameStr);

            result = endpointsEqual && servicesEqual;
        }
        return result;
    }

    @Override
    public int hashCode() {
        int hashCode = 0;
        if (serviceName != null) {
            hashCode += serviceName.hashCode();
        }
        if (endpointName != null) {
            hashCode += endpointName.hashCode();
        }
        return hashCode;
    }

    private InputSource createSource(String msg) {
        InputSource source = new InputSource();
        byte[] msgByte = msg.getBytes();
        ByteArrayInputStream in = new ByteArrayInputStream(msgByte);
        source.setByteStream(in);
        return source;
    }

}
