/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: DeliveryChannelImpl.java,v 1.2 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.petals.jbi.messaging.exchange;

import javax.jbi.JBIException;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessageExchangeFactory;
import javax.jbi.messaging.MessagingException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import org.objectweb.petals.jbi.component.context.ComponentContextImpl;
import org.objectweb.petals.jbi.messaging.routing.RoutingException;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.api.Logger;

/**
 * 
 * @author Adrien LOUIS - EBM WebSourcing
 * @author Gael BLONDELLE - EBM WebSourcing
 * @author ofabre - EBM Websourcing
 * @author rnaudin - EBM Websourcing
 */
public class DeliveryChannelImpl implements DeliveryChannel {

    public static final String PROPERTY_SENDSYNC = "javax.jbi.messaging.sendSync";

    public static final String PROPERTY_NOACK = "org.objectweb.petals.messaging.noack";

    protected ComponentContextImpl context;

    protected LoggingUtil log;

    /**
     * Monolog Logger instance.
     */
    protected Logger logger;

    /**
     * This messageExchangeFactory is used to return
     * <code>MessageExchangeDecorator</code> on accept() methods.
     */
    protected MessageExchangeFactoryImpl messageExchangeFactory;

    protected boolean opened = true;

    /**
     * Constructor
     * 
     * @param context
     * @param router
     * @param logger
     */
    public DeliveryChannelImpl(ComponentContextImpl context) {
        this.logger = context.getLogger();

        log = new LoggingUtil(logger);

        messageExchangeFactory = new MessageExchangeFactoryImpl(this, context
            .getAddress(), logger);

        this.context = context;
    }

    /**
     * Return a MessageExchangeDecorator with 'consumer' or 'provider' observer
     * role (depending on the messageExchange getRole())
     * 
     * @see javax.jbi.messaging.DeliveryChannel#accept()
     */
    public MessageExchange accept() throws MessagingException {
        log.start();
        MessageExchange me = poll(-1);
        log.end();
        return me;
    }

    /**
     * @see javax.jbi.messaging.DeliveryChannel#accept(long)
     */
    public MessageExchange accept(final long timeoutMS)
        throws MessagingException {
        log.start();
        if (timeoutMS < 0) {
            throw new MessagingException("Timeout must be positive or null");
        }
        MessageExchange me = poll(timeoutMS);
        log.end();
        return me;
    }

    /**
     * Poll for a recept message exchange.
     * 
     * @param block
     * @param timeoutMS
     * 
     * @return The received message exchange
     * 
     * @throws MessagingException
     */
    private MessageExchange poll(long timeoutMS) throws MessagingException {
        log.start();

        checkDeliveryChannelIsOpened();

        MessageExchangeImpl msg = null;

        try {
            msg = context.getRouter().receive(context, timeoutMS);
        } catch (RoutingException e) {
            throw new MessagingException(e);
        }

        MessageExchange result = null;
        if (msg != null) {
            result = messageExchangeFactory.createExchangeDecorator(msg, msg
                .getPattern());
        }

        log.end();

        return result;
    }

    /**
     * Check if the <code>DeliveryChannel</code> is opened
     * 
     * @throws MessagingException
     */
    public synchronized void checkDeliveryChannelIsOpened()
        throws MessagingException {
        if (!opened) {
            throw new MessagingException("DeliveryChannel is closed.");
        }
    }

    /**
     * Check if the <code>MessageExchange</code> is Ok (not null and not
     * terminated)
     * 
     * @throws MessagingException
     */
    public void checkMessageExchange(MessageExchange exchange)
        throws MessagingException {
        if (exchange == null) {
            throw new MessagingException("MessageExchange is null.");
        }
        MessageExchangeDecorator exchangeDecorator = (MessageExchangeDecorator) exchange;

        if (exchangeDecorator.getMessageExchange().isTerminated()) {
            throw new MessagingException("MessageExchange is terminated.");
        }
    }

    /**
     * This method wake up all threads that are waiting for a message. This
     * method deregister all endpoints for this component.
     * 
     * @see javax.jbi.messaging.DeliveryChannel#close()
     */
    public synchronized void close() throws MessagingException {
        log.start();

        checkDeliveryChannelIsOpened();

        try {
            context.deregisterAllEndpoints();
        } catch (JBIException e) {
            throw new MessagingException(e);
        }

        opened = false;
        log.end();
    }

    /**
     * @see javax.jbi.messaging.DeliveryChannel#createExchangeFactory()
     */
    public MessageExchangeFactory createExchangeFactory() {
        log.call();

        MessageExchangeFactoryImpl mef = new MessageExchangeFactoryImpl(this,
            context.getAddress(), logger);

        return mef;
    }

    /**
     * @see javax.jbi.messaging.DeliveryChannel#createExchangeFactory(javax.xml.namespace.QName)
     */
    public MessageExchangeFactory createExchangeFactory(QName interfaceName) {
        log.call(interfaceName);

        MessageExchangeFactoryImpl mef = new MessageExchangeFactoryImpl(this,
            context.getAddress(), logger);

        mef.setDefaultInterfaceName(interfaceName);

        return mef;
    }

    /**
     * @see javax.jbi.messaging.DeliveryChannel#createExchangeFactory(javax.jbi.servicedesc.ServiceEndpoint)
     */
    public MessageExchangeFactory createExchangeFactory(ServiceEndpoint endpoint) {
        log.call(endpoint);

        MessageExchangeFactoryImpl mef = new MessageExchangeFactoryImpl(this,
            context.getAddress(), logger);

        mef.setDefaultServiceEndpoint(endpoint);

        return mef;
    }

    /**
     * @see javax.jbi.messaging.DeliveryChannel#createExchangeFactoryForService(javax.xml.namespace.QName)
     */
    public MessageExchangeFactory createExchangeFactoryForService(
        QName serviceName) {
        log.call(serviceName);

        MessageExchangeFactoryImpl mef = new MessageExchangeFactoryImpl(this,
            context.getAddress(), logger);

        mef.setDefaultServiceName(serviceName);

        return mef;
    }

    public synchronized boolean isOpened() {
        return opened;
    }

    /**
     * @see javax.jbi.messaging.DeliveryChannel#send(javax.jbi.messaging.MessageExchange)
     */
    public void send(MessageExchange exchange) throws MessagingException {
        log.start(exchange);

        sendExchange(exchange, 0, false);

        log.end();
    }

    /**
     * @see javax.jbi.messaging.DeliveryChannel#sendSynch(javax.jbi.messaging.MessageExchange)
     */
    public boolean sendSync(MessageExchange exchange) throws MessagingException {
        log.start(exchange);
        boolean res = sendSync(exchange, 0);
        log.end();
        return res;
    }

    /**
     * @see javax.jbi.messaging.DeliveryChannel#sendSync(javax.jbi.messaging.MessageExchange,
     *      long)
     */
    public boolean sendSync(MessageExchange exchange, long timeoutMS)
        throws MessagingException {
        log.start();

        MessageExchangeDecorator exchangeDecorator = (MessageExchangeDecorator) exchange;

        try {
            // prepare the send
            exchangeDecorator.setWaitingOnSynchronousSend(true);

            MessageExchangeImpl responseExchange = sendExchange(
                exchangeDecorator, timeoutMS, true);

            if (responseExchange != null) {
                exchangeDecorator.setMessageExchange(responseExchange);
                exchangeDecorator.setWaitingOnSynchronousSend(false);
            } else {
                // get the ownership of the ME
                exchangeDecorator.getMessageExchange().setRole(
                    exchangeDecorator.getObserverRole());
                exchangeDecorator.setStatus(ExchangeStatus.ERROR);
                exchangeDecorator.getMessageExchange().setTerminated(true);
            }

        } catch (MessagingException e) {
            log.error(e.getMessage(), e);
            // get the ownership of the ME
            exchangeDecorator.getMessageExchange().setRole(
                exchangeDecorator.getObserverRole());
            exchangeDecorator.setStatus(ExchangeStatus.ERROR);
            exchangeDecorator.getMessageExchange().setTerminated(true);

            // If the cause is an Interrupted exception, we return false
            Throwable causeException = e.getCause();
            while (causeException != null) {
                if (causeException instanceof InterruptedException) {
                    log.end();
                    return false;
                } else {
                    causeException = causeException.getCause();
                }
            }
            throw new MessagingException(e);
        }

        log.end();

        return !exchangeDecorator.isWaitingOnSynchronousSend();
    }

    /**
     * Send the exchange to the underlying layer.<br>
     * Unused data in the exchange are cleaned before.<br>
     * If the message to exchange is a mere status information, and the exchange
     * is set to not send acknowledgment, the message is not sent.<br>
     */
    protected MessageExchangeImpl sendExchange(MessageExchange exchange,
        long timeOut, boolean sync) throws MessagingException {
        log.start(exchange);

        checkDeliveryChannelIsOpened();
        checkMessageExchange(exchange);

        MessageExchangeDecorator exchangeDecorator = (MessageExchangeDecorator) exchange;
        MessageExchangeImpl exchangeImpl = exchangeDecorator
            .getMessageExchange();
        MessageExchangeImpl responseExchange = null;

        try {
            // check if the exchange must become terminated
            if (!ExchangeStatus.ACTIVE.equals(exchangeImpl.getStatus())) {
                exchangeImpl.setTerminated(true);
            }

            // check if the status information message must be send back
            boolean sendMessage = true;
            if (exchangeImpl.isTerminated()) {
                Object noAck = exchange.getProperty(PROPERTY_NOACK);
                sendMessage = (noAck == null || !noAck.toString().toLowerCase()
                    .equals("true"));
            }

            // send the message
            if (sendMessage) {
                if (sync) {
                    responseExchange = context.getRouter().sendSync(context,
                        exchangeImpl, timeOut);
                } else {
                    context.getRouter().send(context, exchangeImpl, timeOut);
                }
            }

        } catch (JBIException e) {
            throw new MessagingException(e);
        }

        log.end();
        return responseExchange;
    }

}
