/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2006 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): Adrien LOUIS
 * --------------------------------------------------------------------------
 * $Id: MessageExchangeDecorator.java,v 1.0 2006/04/11 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.petals.jbi.messaging.exchange;

import java.net.URI;
import java.util.Set;

import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

/**
 * <p>
 * This class exposes the <code>MessageExchange</code> for a consumer
 * component .
 * </p>
 * <p>
 * Each actor (consumer or provider) owns a particular
 * <code>MessageExchangeDecoratorForXXX</code> parametrized as a
 * 'MessageExchangeDecorator', or MessageExchangeDecoratorForProvider'.
 * </p>
 * <p>
 * The decorator checks that the methods which are called by the related
 * component can be used for the current exchange state.
 * </p>
 *
 * @author Adrien LOUIS - EBM WebSourcing
 */
public class MessageExchangeDecorator implements MessageExchange {

    protected MessageExchangeImpl messageExchange;

    /**
     * Role of the observer (consumer or provider). Used for checking that the
     * ME is in the good role before calling a method.
     */
    protected Role observerRole;

    /**
     * Used for SendSync. It is set to TRUE when a sendSync is performed with
     * this ME. When the response is sent, it is set to False. If timeout
     * occured before the response has been sent, this boolean is always set to
     * TRUE, and the requester knows that a timeout occured before the response
     * has been sent.
     */
    protected boolean waitingOnSynchronousSend;

    /**
     * Create a MessageExchangeDecorator. Set this object as the
     * consumerMEDecorator or providerMEDecorator of the MessageExchangeImpl,
     * depending on the given Role.
     *
     * @param consumerDeliveryChannel
     */
    public MessageExchangeDecorator(MessageExchangeImpl me, Role observerRole) {
        this.messageExchange = me;

        this.observerRole = observerRole;
    }

    /**
     * @see javax.jbi.messaging.MessageExchange#createFault()
     */
    public Fault createFault() throws MessagingException {
        checkComponentOwnsExchange();

        return messageExchange.createFault();
    }

    /**
     * @see javax.jbi.messaging.MessageExchange#createMessage()
     */
    public NormalizedMessage createMessage() throws MessagingException {
        checkComponentOwnsExchange();

        return messageExchange.createMessage();
    }

    /**
     * @see javax.jbi.messaging.MessageExchange#getEndpoint()
     */
    public ServiceEndpoint getEndpoint() {
        checkComponentOwnsExchange();

        return messageExchange.getEndpoint();
    }

    /**
     * @see javax.jbi.messaging.MessageExchange#getError()
     */
    public Exception getError() {
        // TODO Unecessary ownership test ?
        // checkComponentOwnsExchange();

        return messageExchange.getError();
    }

    /**
     * @see javax.jbi.messaging.MessageExchange#getExchangeId()
     */
    public String getExchangeId() {
        // TODO Unecessary ownership test ?
        // checkComponentOwnsExchange();

        return messageExchange.getExchangeId();
    }

    /**
     * @see javax.jbi.messaging.MessageExchange#getFault()
     */
    public Fault getFault() {
        checkComponentOwnsExchange();

        return messageExchange.getFault();
    }

    /**
     * @see javax.jbi.messaging.MessageExchange#getInterfaceName()
     */
    public QName getInterfaceName() {
        checkComponentOwnsExchange();

        return messageExchange.getInterfaceName();
    }

    /**
     * @see javax.jbi.messaging.MessageExchange#getMessage(java.lang.String)
     */
    public NormalizedMessage getMessage(String name) {
        checkComponentOwnsExchange();

        return messageExchange.getMessage(name);
    }

    /**
     * Return the real implementation of this MessageExchange.
     *
     * @return MessageExchangeImpl
     */
    public MessageExchangeImpl getMessageExchange() {
        return messageExchange;
    }

    /**
     * Return the Role that the owner of this object has in the exchange, which
     * is invariant ( unlike the Role in which the exchange is).
     *
     * @return
     */
    public Role getObserverRole() {
        return observerRole;
    }

    /**
     * @see javax.jbi.messaging.MessageExchange#getOperation()
     */
    public QName getOperation() {
        checkComponentOwnsExchange();

        return messageExchange.getOperation();
    }

    /**
     * @see javax.jbi.messaging.MessageExchange#getPattern()
     */
    public URI getPattern() {
        checkComponentOwnsExchange();

        return messageExchange.getPattern();
    }

    /**
     * @see javax.jbi.messaging.MessageExchange#getProperty(java.lang.String)
     */
    public Object getProperty(String name) {
        checkComponentOwnsExchange();

        return messageExchange.getProperty(name);
    }

    /**
     * @see javax.jbi.messaging.MessageExchange#getPropertyNames()
     */
    public Set getPropertyNames() {
        checkComponentOwnsExchange();

        return messageExchange.getPropertyNames();
    }

    /**
     * @see javax.jbi.messaging.MessageExchange#getRole()
     */
    public Role getRole() {
        checkComponentOwnsExchange();

        return messageExchange.getRole();
    }

    /**
     * @see javax.jbi.messaging.MessageExchange#getService()
     */
    public QName getService() {
        checkComponentOwnsExchange();

        return messageExchange.getService();
    }

    /**
     * @see javax.jbi.messaging.MessageExchange#getStatus()
     */
    public javax.jbi.messaging.ExchangeStatus getStatus() {
        return messageExchange.getStatus();
    }

    /**
     * Return if the component owns the message exchange. (Check that the role
     * of the observer is the same than the role of the message exchange)
     *
     * @throws IllegalStateException
     */
    public boolean hasOwnership() {
        return messageExchange.getRole().equals(observerRole);
    }

    /**
     * @see javax.jbi.messaging.MessageExchange#isTransacted()
     */
    public boolean isTransacted() {
        checkComponentOwnsExchange();

        return messageExchange.isTransacted();
    }

    public boolean isWaitingOnSynchronousSend() {
        return waitingOnSynchronousSend;
    }

    /**
     * @see javax.jbi.messaging.MessageExchange#setEndpoint(java.lang.Exception)
     */
    public void setEndpoint(ServiceEndpoint endpoint) {
        checkComponentOwnsExchange();

        messageExchange.setEndpoint(endpoint);
    }

    /**
     * @see javax.jbi.messaging.MessageExchange#setError(java.lang.Exception)
     */
    public void setError(Exception error) {
        checkComponentOwnsExchange();

        messageExchange.setError(error);
    }

    /**
     * @see javax.jbi.messaging.MessageExchange#setFault(javax.jbi.messaging.Fault)
     */
    public void setFault(Fault fault) throws MessagingException {
        checkComponentOwnsExchange();

        messageExchange.setFault(fault);
    }

    public void setInterfaceName(QName interfaceName) {
        checkComponentOwnsExchange();

        messageExchange.setInterfaceName(interfaceName);
    }

    /**
     * @see javax.jbi.messaging.MessageExchange#setMessage(javax.jbi.messaging.NormalizedMessage,
     *      java.lang.String)
     */
    public void setMessage(NormalizedMessage msg, String name) throws MessagingException {
        checkComponentOwnsExchange();

        messageExchange.setMessage(msg, name);
    }

    // //////////////////////////////////////////
    // Specific methods
    // //////////////////////////////////////////

    /**
     * @see javax.jbi.messaging.MessageExchange#setOperation(javax.xml.namespace.QName)
     */
    public void setOperation(QName name) {
        checkComponentOwnsExchange();

        messageExchange.setOperation(name);
    }

    /**
     * @see javax.jbi.messaging.MessageExchange#setProperty(java.lang.String,
     *      java.lang.Object)
     */
    public void setProperty(String name, Object obj) {
        checkComponentOwnsExchange();

        messageExchange.setProperty(name, obj);
    }

    /**
     * @see javax.jbi.messaging.MessageExchange#setService(javax.xml.namespace.QName)
     */
    public void setService(QName service) {
        checkComponentOwnsExchange();

        messageExchange.setService(service);
    }

    /**
     * @see javax.jbi.messaging.MessageExchange#setStatus(javax.jbi.messaging.ExchangeStatus)
     */
    public void setStatus(ExchangeStatus status) throws MessagingException {
        checkComponentOwnsExchange();

        messageExchange.setStatus(status);
    }

    public void setWaitingOnSynchronousSend(boolean waitingOnSynchronousSend) {
        this.waitingOnSynchronousSend = waitingOnSynchronousSend;
    }

    public void setMessageExchange(MessageExchangeImpl messageExchange) {
        this.messageExchange = messageExchange;
    }

    /**
     * Check that the component owns the message exchange. (Check that the role
     * of the observer is the same than the role of the message exchange)
     *
     * @throws IllegalStateException
     */
    protected void checkComponentOwnsExchange() throws IllegalStateException {
        if (!hasOwnership()) {
            if (messageExchange.isTerminated()) {
                throw new IllegalStateException("The MessageExchange has been closed "
                        + "by an other component (timeout)");
            } else {
                throw new IllegalStateException("The component does not owns the MessageExchange");
            }
        }
    }

}
