/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: MessageExchangeFactoryImpl.java,v 1.3 2005/07/25 07:44:35 alouis Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.petals.jbi.messaging.exchange;

import java.net.URI;

import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.InOptionalOut;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessageExchangeFactory;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.RobustInOnly;
import javax.jbi.messaging.MessageExchange.Role;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import org.objectweb.petals.jbi.messaging.endpoint.ConsumerEndpoint;
import org.objectweb.petals.jbi.messaging.exchange.types.InOnlyImpl;
import org.objectweb.petals.jbi.messaging.exchange.types.InOptionalOutImpl;
import org.objectweb.petals.jbi.messaging.exchange.types.InOutImpl;
import org.objectweb.petals.jbi.messaging.exchange.types.RobustInOnlyImpl;
import org.objectweb.petals.util.IDGenerator;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.api.Logger;

/**
 * @author Adrien LOUIS - EBM WebSourcing
 *
 */
public class MessageExchangeFactoryImpl implements MessageExchangeFactory {

    protected DeliveryChannelImpl channel;

    protected ConsumerEndpoint consumerEndpoint;

    protected QName defaultInterfaceName;

    protected ServiceEndpoint defaultServiceEndpoint;

    protected QName defaultServiceName;

    /**
     * Logger wrapper.
     */
    protected LoggingUtil log;

    public MessageExchangeFactoryImpl(DeliveryChannelImpl dc,
        ConsumerEndpoint ep, Logger logger) {
        log = new LoggingUtil(logger);
        this.channel = dc;
        this.consumerEndpoint = ep;
    }

    /**
     * TODO find an endpoint for this parameteters. Return a
     * <code>MessageExchangeDecorator</code> for Consumer
     *
     * @see javax.jbi.messaging.MessageExchangeFactory#createExchange(javax.xml.namespace.QName,
     *      javax.xml.namespace.QName)
     */
    public MessageExchange createExchange(QName serviceName, QName operationName)
        throws MessagingException {
        log.call(serviceName + "," + operationName);

        MessageExchangeImpl msg = createExchange();

        setDefaultMessageExchangeProperties(msg);

        msg.setService(serviceName);

        msg.setOperation(operationName);

        return new MessageExchangeDecorator(msg, Role.CONSUMER);
    }

    /**
     * Return a <code>MessageExchangeDecorator</code> for Consumer, as
     * <code>InOnlyExchange, InOutExchange,...</code>
     *
     * @see javax.jbi.messaging.MessageExchangeFactory#createExchange(java.net.URI)
     */
    public MessageExchange createExchange(URI pattern)
        throws MessagingException {
        log.call(pattern);

        if (!MessageExchangeImpl.IN_ONLY_PATTERN.equals(pattern)
            && !MessageExchangeImpl.ROBUST_IN_ONLY_PATTERN.equals(pattern)
            && !MessageExchangeImpl.IN_OUT_PATTERN.equals(pattern)
            && !MessageExchangeImpl.IN_OPTIONAL_OUT_PATTERN.equals(pattern)) {
            throw new MessagingException(
                "This Message Exchange Pattern is not recognized by JBI.");
        }

        return createExchangeDecorator(createExchange(), pattern);
    }

    /**
     * Return a <code>MessageExchangeDecorator</code>, as
     * <code>InOnlyExchange, InOutExchange,...</code>, or
     * <code>MessageExchangeDecorator</code> if pattern is unknown. The
     * 'observer' role is set with the current MessageExchangeImpl.getRole()
     * property.
     *
     * @see javax.jbi.messaging.MessageExchangeFactory#createExchange(java.net.URI)
     */
    public MessageExchange createExchangeDecorator(
        MessageExchangeImpl exchange, URI pattern) throws MessagingException {
        log.call(pattern);

        if (MessageExchangeImpl.IN_ONLY_PATTERN.equals(pattern)) {
            return new InOnlyImpl(exchange, exchange.getRole());
        } else if (MessageExchangeImpl.ROBUST_IN_ONLY_PATTERN.equals(pattern)) {
            return new RobustInOnlyImpl(exchange, exchange.getRole());
        } else if (MessageExchangeImpl.IN_OUT_PATTERN.equals(pattern)) {
            return new InOutImpl(exchange, exchange.getRole());
        } else if (MessageExchangeImpl.IN_OPTIONAL_OUT_PATTERN.equals(pattern)) {
            return new InOptionalOutImpl(exchange, exchange.getRole());
        } else {
            return new MessageExchangeDecorator(exchange, exchange.getRole());
        }
    }

    /**
     * Return a <code>MessageExchangeDecorator</code> for Consumer, as
     * <code>InOnly</code>.
     *
     * @see javax.jbi.messaging.MessageExchangeFactory#createInOnlyExchange()
     */
    public InOnly createInOnlyExchange() throws MessagingException {
        log.call();

        MessageExchangeImpl msg = createExchange();

        return new InOnlyImpl(msg, Role.CONSUMER);
    }

    /**
     * Return a <code>MessageExchangeDecorator</code> for Consumer, as
     * <code>InOptionalOut</code>.
     *
     * @see javax.jbi.messaging.MessageExchangeFactory#createInOptionalOutExchange()
     */
    public InOptionalOut createInOptionalOutExchange()
        throws MessagingException {
        log.call();

        MessageExchangeImpl msg = createExchange();

        return new InOptionalOutImpl(msg, Role.CONSUMER);
    }

    /**
     * Return a <code>MessageExchangeDecorator</code> for Consumer, as
     * <code>InOut</code>.
     *
     * @see javax.jbi.messaging.MessageExchangeFactory#createInOutExchange()
     */
    public InOut createInOutExchange() throws MessagingException {
        log.call();

        MessageExchangeImpl msg = createExchange();

        return new InOutImpl(msg, Role.CONSUMER);
    }

    /**
     * Return a <code>MessageExchangeDecorator</code> for Consumer, as
     * <code>RobustInOnly</code>.
     *
     * @see javax.jbi.messaging.MessageExchangeFactory#createRobustInOnlyExchange()
     */
    public RobustInOnly createRobustInOnlyExchange() throws MessagingException {
        log.call();

        MessageExchangeImpl msg = createExchange();

        return new RobustInOnlyImpl(msg, Role.CONSUMER);
    }

    public void setDefaultInterfaceName(QName defaultInterfaceName) {
        this.defaultInterfaceName = defaultInterfaceName;
    }

    public void setDefaultServiceEndpoint(ServiceEndpoint defaultServiceEndpoint) {
        this.defaultServiceEndpoint = defaultServiceEndpoint;
    }

    public void setDefaultServiceName(QName defaultServiceName) {
        this.defaultServiceName = defaultServiceName;
    }

    /**
     * Create a MessageExchangeImpl, which is the real object that is exchange
     * between the components
     *
     * @return
     * @throws MessagingException
     */
    protected MessageExchangeImpl createExchange() throws MessagingException {
        log.call();

        channel.checkDeliveryChannelIsOpened();

        MessageExchangeImpl msg = new MessageExchangeImpl(consumerEndpoint);

        setDefaultMessageExchangeProperties(msg);

        msg.setExchangeId(IDGenerator.getNewID());

        return msg;
    }

    // //////////////////////////////////////////////////////
    // specific methods
    // //////////////////////////////////////////////////////

    /**
     * set to the MessageExchangeImpl the defaultInterfaceName,
     * defaultServiceName and defaultServiceEndpoint, if they were specified
     * during the creation of this factory.
     *
     * @param msg
     */
    protected void setDefaultMessageExchangeProperties(MessageExchangeImpl msg) {
        log.call();

        if (defaultInterfaceName != null) {
            msg.setInterfaceName(defaultInterfaceName);
        }
        if (defaultServiceName != null) {
            msg.setService(defaultServiceName);
        }
        if (defaultServiceEndpoint != null) {
            msg.setEndpoint(defaultServiceEndpoint);
        }

    }
}
