/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: Registry.java 597 2006-06-13 09:57:29Z alouis $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.messaging.registry;

import java.util.List;

import javax.jbi.servicedesc.ServiceEndpoint;
import javax.naming.Context;
import javax.xml.namespace.QName;

import org.objectweb.petals.jbi.messaging.endpoint.AbstractEndpoint;
import org.objectweb.petals.jbi.messaging.endpoint.LinkedEndpoint;

/**
 * The registry is used to manage the endpoint storage
 * 
 * @version $Rev: 597 $ $Date: 2006-06-13 11:57:29 +0200 (mar., 13 juin 2006) $
 * @since Petals 1.0
 * @author alouis,Rafael Marins, ddesjardins
 */
public interface Registry {

    /**
     * Clean the new endpoints context
     * 
     * @throws RegistryException
     *             if a problem occurs
     */
    void cleanNewEndpoints() throws RegistryException;

    /**
     * Unregister a connection
     * 
     * @param connection
     *            connection to unregistered; must be non null
     * @throws RegistryException
     *             if the connection was not previously registered
     */
    void unregisterConnection(LinkedEndpoint connection)
        throws RegistryException;

    /**
     * Deregister the given external endpoint.
     * 
     * @param externalEndpoint
     *            the external endpoint to be deregistered; must be non-null
     * @throws RegistryException
     *             if the endpoint was not previously registered
     */
    void unregisterExternalEndpoint(ServiceEndpoint externalEndpoint)
        throws RegistryException;

    /**
     * Deactivates the given endpoint
     * 
     * @param endpoint
     *            the endpoint to be deactivated; must be non null
     * @throws RegistryException
     *             if the endpoint was not previously registered
     */
    void unregisterInternalEndpoint(AbstractEndpoint endpoint)
        throws RegistryException;

    /**
     * Get the external service endpoint for the named activated endpoint, if
     * any
     * 
     * @param service
     *            qualified name of the endpoint's service; must be non null
     * @param name
     *            name of the endpoint; must be non null
     * @return the named external endpoint, or null if the named endpoint is not
     *         activated
     */
    AbstractEndpoint getExternalEndpoint(QName service, String name)
        throws RegistryException;

    /**
     * Queries for external endpoints that implements the given interface
     * 
     * @param interfaceName
     *            qualified name of the interface that contains the endpoints
     * @return Array of external endpoints
     * 
     * Array of external endpoints that implements the qualified interface, if
     * the interface name is non null
     * 
     * Array of all the external endpoints, if the interface name is null
     * 
     * @throws RegistryException
     *             if a problem occurs
     */
    AbstractEndpoint[] getExternalEndpointsForInterface(QName interfaceName)
        throws RegistryException;

    /**
     * Queries for external endpoints that are part of the given service
     * 
     * @param serviceName
     *            qualified name of the service that contains the endpoints;
     *            must be non null
     * 
     * @return array of available external endpoints for the specified service
     *         name; must be non null; may be empty
     * 
     */
    AbstractEndpoint[] getExternalEndpointsForService(QName serviceName);

    /**
     * Get the internal service endpoint for the named activated endpoint, if
     * any. if the endpoint is a link, resolve it and return the real endpoint
     * 
     * @param service
     *            qualified name of the endpoint's service; must be non null
     * @param name
     *            name of the endpoint; must be non null
     * @return the named internal endpoint, or null if the named endpoint is not
     *         activated
     */
    AbstractEndpoint getInternalEndpoint(QName service, String name)
        throws RegistryException;

    /**
     * Get the internal service endpoint for the named activated endpoint, if
     * any
     * 
     * @param service
     *            qualified name of the endpoint's service; must be non null
     * @param name
     *            name of the endpoint; must be non null
     * @param resolveLink
     *            if the Endpoint is a link, resolve it (true) or return the
     *            link (false)
     * @return the named internal endpoint, or null if the named endpoint is not
     *         activated
     */
    AbstractEndpoint getInternalEndpoint(QName service, String name,
            boolean resolveLink) throws RegistryException;

    /**
     * Queries for internal endpoints that implements the given interface
     * 
     * @param interfaceName
     *            qualified name of the interface that contains the endpoints
     * @return Array of internal endpoints
     * 
     * Array of internal endpoints that implements the qualified interface, if
     * the interface name is non null
     * 
     * Array of all the external endpoints, if the interface name is null
     * 
     * @throws RegistryException
     *             if a problem occurs
     */
    AbstractEndpoint[] getInternalEndpointsForInterface(QName interfaceName)
        throws RegistryException;

    /**
     * Queries for internal endpoints that are part of the given service
     * 
     * @param serviceName
     *            qualified name of the service that contains the endpoints;
     *            must be non null
     * 
     * @return array of available internal endpoints for the specified service
     *         name; must be non null; may be empty
     * 
     */
    AbstractEndpoint[] getInternalEndpointsForService(QName serviceName);

    /**
     * Retrieve the users context that is used by components
     * 
     * @return the users JNDI context
     */
    Context getUsersContext();

    /**
     * Register a connection
     * 
     * @param connection
     *            connection to registered; must be non null
     * @throws RegistryException
     *             if the connection has already been registered or if a problem
     *             occurs
     */
    void registerConnection(LinkedEndpoint connection) throws RegistryException;

    /**
     * Register the given external endpoint
     * 
     * @param externalEndpoint
     *            the external endpoint to be registered; must be non null
     * @throws RegistryException
     *             if an external endpoint with the same name is already
     *             registered
     */
    void registerExternalEndpoint(ServiceEndpoint externalEndpoint)
        throws RegistryException;

    /**
     * Register the given internal endpoint
     * 
     * @param internalEndpoint
     *            the internal endpoint to be registered; must be non null
     * @throws RegistryException
     *             if an internal endpoint with the same name is already
     *             registered
     */
    void registerInternalEndpoint(AbstractEndpoint endpoint)
        throws RegistryException;

    /**
     * Retreive the list of all the new endpoints
     * 
     * @return list of new endpoints
     * @throws RegistryException
     *             if a problem occurs
     */
    List<AbstractEndpoint> retrieveNewEndpoints() throws RegistryException;

    /**
     * Update the endpoint, remove it from the new-endpoint context and update
     * the interfaces context
     * 
     * @param endpoint
     *            endpoint to validate; must be non null
     * @throws RegistryException
     *             if a problem occurs
     */
    void validateEndpoint(AbstractEndpoint endpoint) throws RegistryException;

}
