/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.jbi.messaging.routing;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Random;

import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;

import org.objectweb.petals.PetalsException;
import org.objectweb.petals.communication.topology.TopologyService;
import org.objectweb.petals.jbi.messaging.endpoint.InternalEndpoint;
import org.objectweb.petals.kernel.configuration.ConfigurationService;

/**
 * The enpoint chooser strategy.
 *
 * @author nsalatge - eBM WebSourcing
 * @author Roland NAUDIN - eBM WebSourcing
 *
 */
public final class EndpointChooserStrategy {

    /**
     * the highest strategy.
     */
    public static final String HIGHEST = "highest";

    /**
     * the load balancing (random strategy).
     */
    public static final String RANDOM = "random";

    /**
     * Constant used to specify the strategy to use.
     */
    public static final String PROPERTY_STRATEGY_PROTOCOLS = "org.objectweb.petals.routing.strategy";

    /**
     * Number of parameters defined in strategy.
     */
    public static final int NUMBER_STRATEGY_PARAMETERS = 4;

    /**
     * The strategy separator.
     */
    public static final String STRATEGY_SEPARATOR = ",";

    /**
     * The Topology service
     */
    private TopologyService topologyService;

    /**
     * The Configuration service
     */
    private ConfigurationService configurationService;

    /**
     * Default constructor.
     *
     */
    protected EndpointChooserStrategy(ConfigurationService configurationService,
        TopologyService topologyService) {
        this.topologyService = topologyService;
        this.configurationService = configurationService;
    }

    /**
     * Elects the address in the list of endpoints as the message recipient,
     * with the defined strategy. Ask the consumer and the provider if they are
     * Okay for the exchange.
     *
     * @param endpoints
     *            the list of endpoints to select an endpoint from
     * @param exchange
     *            the exchange
     * @param strategyParameters
     *            the list of parameters of the strategy. Must be non-null
     * @param r
     *            a random parameter. May be null.
     * @return the service endpoint selected or null
     * @throws RoutingException
     *             impossible to realize this strategy
     * @throws PetalsException
     * @throws PetalsException
     */
    public ServiceEndpoint choose(final List<ServiceEndpoint> endpoints,
        final MessageExchange exchange, final List<Object> strategyParameters,
        final Random r) throws RoutingException {

        String type = null;
        float localPonderation = 0;
        float remoteActivePonderation = 0;
        float remoteInactivePonderation = 0;

        try {
            if (strategyParameters != null
                && strategyParameters.size() == EndpointChooserStrategy.NUMBER_STRATEGY_PARAMETERS) {
                Iterator it = strategyParameters.iterator();
                type = ((String) it.next()).toLowerCase();
                localPonderation = (Float) it.next();
                remoteActivePonderation = (Float) it.next();
                remoteInactivePonderation = (Float) it.next();
            } else {
                throw new RoutingException(
                    "Invalid Parameters: "
                        + "4 parameters are required to configure the routing strategy");
            }
        } catch (ClassCastException e) {
            throw new RoutingException("Impossible to convert the "
                + "parameters to realize the routing strategy");
        }

        EndpointChooserStrategy.verifStrategyParameters(type, localPonderation,
            remoteActivePonderation, remoteInactivePonderation);

        ServiceEndpoint result = null;
        if (type.equals(EndpointChooserStrategy.HIGHEST)) {
            EndpointChooserStrategy.analyzePonderation(localPonderation,
                remoteActivePonderation, remoteInactivePonderation);
            result = loadHighestChoose(endpoints, exchange, localPonderation,
                remoteActivePonderation, remoteInactivePonderation);
        } else if (type.equals(EndpointChooserStrategy.RANDOM)) {
            EndpointChooserStrategy.analyzePonderation(localPonderation,
                remoteActivePonderation, remoteInactivePonderation);
            result = loadBalancingChoose(endpoints, exchange, localPonderation,
                remoteActivePonderation, remoteInactivePonderation, r);
        }

        return result;

    }

    /**
     * verify the strategy parameters.
     *
     * @param type
     *            the strategy type
     * @param localPonderation
     *            the local ponderation
     * @param remoteActivePonderation
     *            the remote active ponderation
     * @param remoteInactivePonderation
     *            the remote inactive ponderation
     * @throws RoutingException
     *             Impossible to find an endpoint with these ponderations
     */
    public static void verifStrategyParameters(final String type,
        final float localPonderation, final float remoteActivePonderation,
        final float remoteInactivePonderation) throws RoutingException {
        if ((!type.equals(EndpointChooserStrategy.HIGHEST))
            && (!type.equals(EndpointChooserStrategy.RANDOM))) {
            throw new RoutingException("the " + type
                + " routing strategy is unknown");
        }

        if ((localPonderation < 0) || (remoteActivePonderation < 0)
            || (remoteInactivePonderation < 0)) {
            throw new RoutingException(
                "impossible to have a negative parameter "
                    + "in routing strategy");
        }
    }

    /**
     * analyse the ponderations.
     *
     * @param l
     *            the local ponderation
     * @param ra
     *            the remote active ponderation
     * @param ri
     *            the remote inactive ponderation
     * @throws RoutingException
     *             Impossible to find an endpoint with these ponderations
     */
    private static void analyzePonderation(final float l, final float ra,
        final float ri) throws RoutingException {
        if ((l == 0) && (ra == 0) && (ri == 0)) {
            throw new RoutingException("Impossible to find an endpoint "
                + "with these ponderations: 0,0,0");
        }
    }

    /**
     * The random strategy. This strategy choose an endpoint in function of
     * defined ponderations. The endpoints that have the strongest ponderation
     * can be more easely choose in comparison with the others.
     *
     * @param topology
     *            the container topology. Must be non-null
     * @param endpoints
     *            the list of valid endpoints. May be null
     * @param localPonderation
     *            the local ponderation
     * @param remoteActivePonderation
     *            the remote active ponderation
     * @param remoteInactivePonderation
     *            the remote inactive ponderation
     * @param exchange
     *            the exchange. Must be non-null
     * @param r
     *            a random parameter. Must be non-null.
     * @return the service endpoit selected
     * @throws RoutingException
     *             impossible to realize this strategy
     */
    private ServiceEndpoint loadBalancingChoose(
        final List<ServiceEndpoint> endpoints, final MessageExchange exchange,
        final float localPonderation, final float remoteActivePonderation,
        final float remoteInactivePonderation, final Random r)
        throws RoutingException {

        if (exchange == null) {
            throw new NullPointerException("The exchange cannot be null");
        }
        if (r == null) {
            throw new NullPointerException("The random cannot be null");
        }

        ServiceEndpoint result = null;

        if (endpoints != null && endpoints.size() > 0) {

            Map<ServiceEndpoint, Float> ponderations = getPonderationEndpoints(
                endpoints, localPonderation, remoteActivePonderation,
                remoteInactivePonderation);

            float factor = EndpointChooserStrategy
                .sumPonderations(ponderations)
                * r.nextFloat();

            Iterator<ServiceEndpoint> it = endpoints.iterator();
            ServiceEndpoint provider = null;
            float idMin = 0;
            float idMax = 0;
            if (factor == 0) {
                result = it.next();
            }

            while ((result == null) && (it.hasNext())) {
                provider = it.next();
                idMax = idMin
                    + EndpointChooserStrategy.findPonderation(provider,
                        ponderations);
                if (idMin <= factor && factor < idMax) {
                    result = provider;
                }
                idMin = idMax;
            }

        }
        return result;
    }

    /**
     * find the ponderation of this provider.
     *
     * @param provider
     *            the provider. Must be non-null
     * @param ponderations
     *            the ponderations. Must be non-null
     * @return the finded ponderation. Must be-non null
     */
    private static float findPonderation(final ServiceEndpoint provider,
        final Map<ServiceEndpoint, Float> ponderations) {
        if (provider == null) {
            throw new NullPointerException("the provider cannot be null");
        }
        if (ponderations == null) {
            throw new NullPointerException("the ponderations cannot be null");
        }
        Float ponderation = ponderations.get(provider);
        if (ponderation == null) {
            throw new NullPointerException("the ponderation cannot be null");
        }
        return ponderation.floatValue();
    }

    /**
     * get the ponderation of all provider.
     *
     * @param endpoints
     *            the endpoints
     * @param localPonderation
     *            the local ponderation
     * @param remoteActivePonderation
     *            the remote active ponderation
     * @param remoteInactivePonderation
     *            the remote incative ponderation
     * @return all ponderations
     * @throws RoutingException
     *             impossible to contact the container topology
     */
    private Map<ServiceEndpoint, Float> getPonderationEndpoints(
        final List<ServiceEndpoint> endpoints, final float localPonderation,
        final float remoteActivePonderation,
        final float remoteInactivePonderation) throws RoutingException {

        Map<ServiceEndpoint, Float> result = new HashMap<ServiceEndpoint, Float>();
        try {
            for (ServiceEndpoint provider : endpoints) {
                if (((InternalEndpoint) provider).getContainerName().equals(
                    configurationService.getContainerConfiguration().getName())) {
                    // Local endpoint
                    result.put(provider, new Float(localPonderation));
                } else

                if (topologyService
                    .isContainerStarted(((InternalEndpoint) provider)
                        .getContainerName())) {
                    // Active remote endpoint
                    result.put(provider, new Float(remoteActivePonderation));
                } else {
                    // Inactive Remote endpoint
                    result.put(provider, new Float(remoteInactivePonderation));
                }
            }
        } catch (PetalsException e) {
            throw new RoutingException(e);
        }
        return result;
    }

    /**
     * calcul the sum of the ponderation of all endpoints.
     *
     * @param ponderations
     *            the ponderations
     * @return the sum of the ponderation of all endpoints
     */
    private static float sumPonderations(
        final Map<ServiceEndpoint, Float> ponderations) {
        float somme = 0;
        for (Float ponderation : ponderations.values()) {
            somme = somme + ponderation;
        }
        return somme;
    }

    /**
     * The highest strategy. This strategy choose the first endpoint in the list
     * that have the strongest ponderation.
     *
     * @param topology
     *            the Topology service. Must be non-null
     * @param endpoints
     *            the list of valid endpoints. May be null
     * @param localPonderation
     *            the local ponderation
     * @param remoteActivePonderation
     *            the remote active ponderation
     * @param remoteInactivePonderation
     *            the remote incative ponderation
     * @param exchange
     *            the exchange. Must be non-null
     * @return the service endpoit selected
     * @throws RoutingException
     *             impossible to contact the container topology
     */
    private ServiceEndpoint loadHighestChoose(
        final List<ServiceEndpoint> endpoints, final MessageExchange exchange,
        final float localPonderation, final float remoteActivePonderation,
        final float remoteInactivePonderation) throws RoutingException {

        if (exchange == null) {
            throw new NullPointerException("The exchange cannot be null");
        }

        ServiceEndpoint result = null;

        if (endpoints != null && endpoints.size() > 0) {
            float[] ponderationsOrder = EndpointChooserStrategy
                .sortPonderations(localPonderation, remoteActivePonderation,
                    remoteInactivePonderation);

            boolean trouve = false;
            int id = ponderationsOrder.length - 1;
            while ((!trouve) && (id >= 0)) {

                Map<ServiceEndpoint, Float> ponderations = getPonderationEndpoints(
                    endpoints, localPonderation, remoteActivePonderation,
                    remoteInactivePonderation);

                Iterator<ServiceEndpoint> it = endpoints.iterator();
                ServiceEndpoint provider = null;

                while ((result == null) && (it.hasNext())) {
                    provider = it.next();
                    if (ponderationsOrder[id] == EndpointChooserStrategy
                        .findPonderation(provider, ponderations)) {
                        result = provider;
                    }
                }
                id--;
            }
        }
        return result;
    }

    /**
     * Sort the ponderations in a ascending order.
     *
     * @param localPonderation
     *            the local ponderation
     * @param remoteActivePonderation
     *            the remote active ponderation
     * @param remoteInactivePonderation
     *            the remote incative ponderation
     * @return the sorted ponderations
     */
    private static float[] sortPonderations(final float localPonderation,
        final float remoteActivePonderation,
        final float remoteInactivePonderation) {
        float[] sortMax = new float[EndpointChooserStrategy.NUMBER_STRATEGY_PARAMETERS - 1];
        sortMax[0] = localPonderation;
        sortMax[1] = remoteActivePonderation;
        sortMax[2] = remoteInactivePonderation;
        Arrays.sort(sortMax);
        return sortMax;
    }
}
