/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.jbi.messaging.transport.local;

import java.util.Hashtable;
import java.util.concurrent.ArrayBlockingQueue;

import javax.jbi.messaging.MessageExchange;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.Service;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.petals.jbi.messaging.exchange.MessageExchangeImpl;
import org.objectweb.petals.jbi.messaging.transport.TransportException;
import org.objectweb.petals.jbi.messaging.transport.TransportProtocol;
import org.objectweb.petals.jbi.messaging.transport.TransportProtocolListener;
import org.objectweb.petals.jbi.messaging.transport.util.TransporterUtil;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.api.Logger;

/**
 * This class provides a local Transporter. The exchanges are stored in Java
 * queues and are never persisted. If a queue exceeds its maximum size, the
 * exchanges are refused by throwing an TransportProtocol exception.
 *
 * @author rnaudin
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = TransportProtocol.class))
public class LocalTransporter implements TransportProtocol {

    /**
     * Name used to identify the joram transporter when the "transporterToUse"
     * property is set.
     */
    public static final String TRANSPORTER_NAME = "local";

    /**
     * The maximum size of a the Java queue.
     */
    private static final int QUEUE_SIZE = 10000;

    /**
     * The map of the exchanges queues. One queue per installed component.
     */
    private Hashtable<String, ArrayBlockingQueue<MessageExchange>> exchangeQueuesMap;

    /**
     * The logger.
     */
    protected LoggingUtil log;

    /**
     * The Fractal component
     */
    @Service
    Component localTransporterComponent;

    /**
     * The fractal exposed interface of the component
     */
    TransportProtocol localTransportProtocol;

    /**
     * Monolog component logger
     */
    @Monolog(name = "logger")
    private Logger logger;

    @Requires(name = "dispatcher", signature = org.objectweb.petals.jbi.messaging.transport.TransportProtocolListener.class)
    private TransportProtocolListener dispatcher;

    @LifeCycle(on = LifeCycleType.START)
    public void start() throws Exception {
        log = new LoggingUtil(logger);
        log.call();
        exchangeQueuesMap = new Hashtable<String, ArrayBlockingQueue<MessageExchange>>();

        localTransportProtocol = (TransportProtocol) localTransporterComponent
                .getFcInterface("service");
    }

    @LifeCycle(on = LifeCycleType.STOP)
    public void stop() throws Exception {
        log.call();
        for (String componentName : exchangeQueuesMap.keySet()) {
            if (!exchangeQueuesMap.get(componentName).isEmpty()) {
                log.warning("The Local Transporter contains message destinated to the component '"
                        + componentName + "'. The message are losts.");
            }
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.PetalsLifeCycle#shutdown()
     */
    public void shutdown() throws Exception {
        log.call();
        // do nothing
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.jbi.transport.TransportProtocol#addDestination(java.lang.String)
     */
    public void addDestination(final String componentName) throws TransportException {
        log.call();
        exchangeQueuesMap.put(componentName, new ArrayBlockingQueue<MessageExchange>(QUEUE_SIZE));

    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.jbi.transport.TransportProtocol#removeDestination(java.lang.String)
     */
    public void removeDestination(final String componentName) throws TransportException {
        log.call();
        exchangeQueuesMap.remove(componentName);

    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.jbi.transport.TransportProtocol#receive(java.lang.String)
     */
    public MessageExchangeImpl receive(final String componentName) throws TransportException {
        log.start("Receive : " + componentName);

        MessageExchangeImpl exchange = (MessageExchangeImpl) exchangeQueuesMap.get(componentName)
                .poll();

        log.end();
        return exchange;
    }

    /**
     * Send a message exchange to the given destination. If the message is an
     * response of a synchronious invokation, the
     * <code>dispatcher.syncMessageReceived()</code> is called.
     *
     * @see org.objectweb.petals.jbi.transport.TransportProtocol#send(org.objectweb.petals.jbi.messaging.MessageExchangeImpl,
     *      java.lang.String, java.lang.String, long)
     */
    public void send(MessageExchangeImpl exchange, final String componentName,
            final String containerName, final long timeToLive) throws TransportException {
        log.start();

        ArrayBlockingQueue<MessageExchange> exchangesQueue = exchangeQueuesMap.get(componentName);

        if (exchangesQueue == null) {
            throw new TransportException("The destination bound to the component '" + componentName
                    + "' is not on the local container");
        }

        if (exchangesQueue.remainingCapacity() == 0) {
            throw new TransportException(
                    "The Local Transporter has reached its maximum capacity for the target component '"
                            + componentName + "'. The message exchange is rejected");
        }

        if (TransporterUtil.getSyncMode(exchange, true)) {
            dispatcher.syncMessageReceived(exchange);
        } else {
            exchangesQueue.add(exchange);
            dispatcher.messageAvailable(localTransportProtocol, componentName);
        }

        log.end();
    }
}
