/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.jbi.messaging.transport.util;

import java.io.File;
import java.io.IOException;
import java.net.ConnectException;
import java.net.UnknownHostException;
import java.util.List;

import javax.jms.ConnectionFactory;

import org.apache.commons.io.FileUtils;
import org.objectweb.joram.client.jms.Destination;
import org.objectweb.joram.client.jms.Queue;
import org.objectweb.joram.client.jms.Topic;
import org.objectweb.joram.client.jms.admin.AdminException;
import org.objectweb.joram.client.jms.admin.AdminModule;
import org.objectweb.joram.client.jms.local.LocalConnectionFactory;
import org.objectweb.petals.PetalsException;
import org.objectweb.petals.kernel.configuration.ContainerConfiguration;
import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.LoggerFactory;

import fr.dyade.aaa.agent.AgentServer;

/**
 * Utility class to control the JOrAM agent server.
 *
 * @author Christophe HAMERLING - eBMWebSourcing
 * @author Roland NAUDIN - eBMWebSourcing
 *
 */
public class JoramControl {

    /**
     * Instance of the Joram control class
     */
    private static JoramControl instance;

    /**
     * The local connection of this Joram agent server
     */
    private ConnectionFactory connectionFactory;

    /**
     * Singleton access
     *
     * @return
     */
    public static synchronized JoramControl getInstance() {
        if (instance == null) {
            instance = new JoramControl();
        }
        return instance;
    }

    /**
     * Creates a new instance of {@link JoramControl}
     *
     */
    private JoramControl() {
    }

    /**
     * Clean the workspace
     *
     */
    public final void cleanAgentServerWorkspace(File joramDir, boolean forceDelete) {
        for (File file : joramDir.listFiles()) {
            if (file.isDirectory()) {
                // We are in a Joram backup directory
                boolean isLock = false;
                for (File joramFile : file.listFiles()) {
                    if ("lock".equals(joramFile.getName())) {
                        // We are looking for a lock file
                        isLock = true;
                        break;
                    }
                }
                if (isLock || forceDelete) {
                    // This Joram server was not shutdown correctly
                    // We need to remove the backup directory
                    try {
                        FileUtils.forceDelete(file);
                    } catch (IOException e) {
                        // Do nothing
                    }
                }
            }
        }
    }

    /**
     * Initialize the agent server
     *
     * @param agentDir
     * @param id
     * @param factory
     * @throws Exception
     */
    private void doInitAgentServer(File agentDir, short id) throws Exception {
        System.setProperty(AgentServer.CFG_DIR_PROPERTY, agentDir
            .getAbsolutePath());

        AgentServer.init(id, agentDir.getAbsolutePath() + File.separator + id,
            getLoggerFactoryInstance());
    }

    /**
     * Test if the agent server is started
     *
     * @return
     */
    public final boolean isAgentServerRunning() {
        return 4 == AgentServer.getStatus();
    }

    /**
     * Start the agent server.
     *
     * @throws Exception
     *             the exception throwed by the joram agent
     */
    public final void startAgentServer() throws Exception {
        AgentServer.start();
    }

    /**
     * Stop the agent server
     *
     * @throws AdminException
     * @throws UnknownHostException
     * @throws NumberFormatException
     * @throws ConnectException
     *
     */
    public final void stopAgentServer(
        ContainerConfiguration containerConfiguration) throws ConnectException,
        NumberFormatException, UnknownHostException, AdminException {
        // stop the agent through the AdminModule to clearly stop it in the
        // JOrAM module
        connectToAdminModule(containerConfiguration);
        AdminModule.stopServer();
        disconnectFromAdminModule();
        // stop the agent
        AgentServer.stop();
    }

    /**
     * Add a Joram server to the Joram platform
     *
     * @param id
     * @param host
     * @param domain
     * @param domainPort
     * @param name
     * @param tcpPort
     * @throws ConnectException
     * @throws AdminException
     */
    public void addJoramServer(int id, String host, String domain,
        int domainPort, String name, int tcpPort) throws ConnectException,
        AdminException {
        AdminModule
            .addServer(
                id,
                host,
                domain,
                domainPort,
                name,
                new String[] {"org.objectweb.joram.mom.proxies.tcp.TcpProxyService"},
                new String[] {String.valueOf(tcpPort)});
    }

    /**
     *
     * @param containerInformation
     * @param joramDir
     * @throws PetalsException
     */
    public void initAgentServer(File joramDir, short containerId)
        throws PetalsException {
        if (!isAgentServerRunning()) {
            try {
                cleanAgentServerWorkspace(joramDir, false);
                doInitAgentServer(joramDir, containerId);
            } catch (Exception e) {
                fr.dyade.aaa.agent.AgentServer.stop(true);
                throw new PetalsException("Joram Agent can not be initialized",
                    e);
            }
        }
    }

    /**
     * Remove a container from JOrAM agent. Connects to the JOrAM module, remove
     * the server and disconnect from the admin module.
     *
     * @param containerConfiguration
     * @throws PetalsException
     */
    public void removeContainer(ContainerConfiguration containerConfiguration)
        throws PetalsException {
        if (containerConfiguration == null) {
            throw new IllegalArgumentException(
                "Container configuration cannot be null");
        }

        try {
            AdminModule.removeServer(containerConfiguration.getJoramId());
        } catch (ConnectException e) {
            throw new PetalsException(
                "Can not remove container from JORAM agent", e);
        } catch (AdminException e) {
            throw new PetalsException(
                "Can not remove container from JORAM agent", e);
        }
    }

    /**
     * Remove the local server.
     *
     * @param containerConfiguration
     * @throws PetalsException
     */
   public void removeLocalContainer(ContainerConfiguration containerConfiguration)
       throws PetalsException {
       if (containerConfiguration == null) {
           throw new IllegalArgumentException(
               "Container configuration cannot be null");
       }

       try {
           AdminModule.removeServer(containerConfiguration.getJoramId());

       } catch (ConnectException e) {
           throw new PetalsException(
               "Can not remove container from JORAM agent", e);
       } catch (AdminException e) {
           throw new PetalsException(
               "Can not remove container from JORAM agent", e);
       }
   }

    /**
     * Get a local connection factory
     *
     */
    public ConnectionFactory getConnectionFactory() {
        if (connectionFactory == null) {
            connectionFactory = LocalConnectionFactory.create();

            // FIXME optim multithread
            // connectionFactory.getParameters().multiThreadSync = true;
            // connectionFactory.getParameters().multiThreadSyncDelay = 1;
        }
        return connectionFactory;
    }

    /**
     * Connect the Admin module to the collocated server
     *
     * @param containerInformation
     * @throws AdminException
     * @throws UnknownHostException
     * @throws NumberFormatException
     * @throws ConnectException
     * @throws Exception
     *
     * @see AdminModule#collocatedConnect(String, int, String, String, int,
     *      String)
     */
    public void connectToAdminModule(
        ContainerConfiguration containerConfiguration) throws ConnectException,
        NumberFormatException, UnknownHostException, AdminException {
        if (containerConfiguration == null) {
            throw new IllegalArgumentException(
                "Container configuration can not be null");
        }

        AdminModule.collocatedConnect(containerConfiguration.getJoramLogin(),
            containerConfiguration.getJoramPassword());
    }

    /**
     * Disconnect from the JORAM admin module
     *
     */
    public void disconnectFromAdminModule() {
        AdminModule.disconnect();
    }

    /**
     * Create a new topic
     *
     * @param topicName
     *            the name of the topic to be created
     * @see Topic#create()
     */
    public Topic createTopic(String topicName) throws ConnectException,
        AdminException {
        Topic topic = Topic.create(topicName);
        topic.setFreeReading();
        topic.setFreeWriting();
        return topic;
    }

    /**
     * Create a new queue
     *
     * @param queueName
     *            the name of the queue to be created
     * @see Topic#create()
     */
    public Queue createQueue(String queueName) throws ConnectException,
        AdminException {
        Queue queue = Queue.create(queueName);
        queue.setFreeReading();
        queue.setFreeWriting();
        return queue;
    }

    /**
     * Check if the given destination exists.
     *
     * @param destination
     *            the destination to check
     */
    @SuppressWarnings("unchecked")
    public boolean isDestinationExist(Destination destination, int serverId)
        throws ConnectException, AdminException {
        List serverDestinations = AdminModule.getDestinations(serverId);

        for (Object serverDestination : serverDestinations) {
            if (((Destination) serverDestination).getName().equals(
                destination.getName())) {
                return true;
            }
        }

        return false;
    }

    /**
     * Get the logger factory to be used by JORAM. It must have been properly
     * initialized at the Petals server start.
     *
     * @return
     */
    private LoggerFactory getLoggerFactoryInstance() {
        return Monolog.getMonologFactory();
    }
}
