/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.jbi.messaging.transport.util;

import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;

import org.objectweb.petals.jbi.messaging.exchange.DeliveryChannelImpl;
import org.objectweb.petals.jbi.messaging.exchange.MessageExchangeDecorator;
import org.objectweb.petals.jbi.messaging.exchange.MessageExchangeImpl;

/**
 * @author garciap
 *
 */
public class TransporterUtil {

    public static final String SYNC_MODE = "syncMode";

    public static final String PROPERTY_SENDSYNC_PROVIDER = "javax.jbi.messaging.sendSync.provider";

    public static final String PROPERTY_SENDSYNC_CONSUMER = "javax.jbi.messaging.sendSync.consumer";

    /**
     * Set the properties to the message exchange to specify that a synchronious
     * response is expected
     *
     * @param exchange
     *
     * @throws MessagingException
     *             The <code>sync_send()</code> is forbidden for
     *             acknowledgment send
     */
    public static void setSendSyncProperties(MessageExchangeImpl exchange)
            throws MessagingException {

        if (!exchange.getStatus().equals(ExchangeStatus.ACTIVE)) {
            throw new MessagingException(
                    "Synchronious send is forbidden when sending an acknowledgement");
        }

        // specify the role of the requester of the sync exchange, used
        // internally
        if (MessageExchange.Role.PROVIDER.equals(exchange.getRole()))
            exchange.setProperty(PROPERTY_SENDSYNC_CONSUMER, "true");
        else
            exchange.setProperty(PROPERTY_SENDSYNC_PROVIDER, "true");

        // set the property as defined in the JBI specifications
        exchange.setProperty(DeliveryChannelImpl.PROPERTY_SENDSYNC, "true");
    }

    /**
     * Update the sync properties of the message exchange
     *
     * @param exchange
     */
    public static void updateSyncProperties(MessageExchangeImpl exchange) {
        if (MessageExchange.Role.PROVIDER.equals(exchange.getRole())) {
            // remove the SENDSYNC property for eventually futher exchanges
            exchange.setProperty(DeliveryChannelImpl.PROPERTY_SENDSYNC, null);
            exchange.setProperty(PROPERTY_SENDSYNC_PROVIDER, null);
        } else {
            // remove the SENDSYNC property for eventually futher exchanges
            exchange.setProperty(DeliveryChannelImpl.PROPERTY_SENDSYNC, null);
            exchange.setProperty(PROPERTY_SENDSYNC_CONSUMER, null);
        }
    }

    /**
     * Return the synchronized mode related to a message exchange.
     *
     * @param messageExchange
     *            the message exchange to check
     * @param isResponse
     *            true if it is property checking for a response message false
     *            otherwise
     * @return true if message is in CONSUMER role and it is a response or if
     *         message is in PROVIDER role and it isn't a response, false
     *         otherwise
     */
    public static boolean getSyncMode(MessageExchangeImpl messageExchange, boolean isResponse) {
        boolean syncMode = false;
        if ((MessageExchange.Role.CONSUMER.equals(messageExchange.getRole()) && isResponse)
                || (MessageExchange.Role.PROVIDER.equals(messageExchange.getRole()) && !isResponse)) {
            syncMode = (messageExchange.getProperty(TransporterUtil.PROPERTY_SENDSYNC_CONSUMER) != null && ((String) messageExchange
                    .getProperty(TransporterUtil.PROPERTY_SENDSYNC_CONSUMER)).toLowerCase().equals(
                    "true"));
        } else {
            syncMode = (messageExchange.getProperty(TransporterUtil.PROPERTY_SENDSYNC_PROVIDER) != null && ((String) messageExchange
                    .getProperty(TransporterUtil.PROPERTY_SENDSYNC_PROVIDER)).toLowerCase().equals(
                    "true"));
        }
        return syncMode;
    }

    /**
     * Return the string key associated with this exchange : exchangeId. It is
     * used to store synchronized exchanges. <br>
     * Note: if the exchange object is <code>MessageExchangeDecorator</code>,
     * we retreive the underlying <code>MessageExchangeImpl</code> object, to
     * avoid the checks made on getExchangeId().
     *
     * @param exchange
     * @return
     */
    public static String waitingExchangeKey(MessageExchange exchange) {
        MessageExchangeImpl meImpl = null;

        if (exchange instanceof MessageExchangeDecorator) {
            meImpl = ((MessageExchangeDecorator) exchange).getMessageExchange();
        } else {
            meImpl = (MessageExchangeImpl) exchange;
        }
        return meImpl.getExchangeId();
    }
}
