/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.kernel.server;

import java.io.IOException;
import java.util.Hashtable;
import java.util.Set;

import javax.management.MBeanServer;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.jmx.agent.Introspector;
import org.objectweb.fractal.util.Fractal;
import org.objectweb.petals.communication.jmx.JMXService;
import org.objectweb.petals.jbi.management.admin.AdminServiceMBean;
import org.objectweb.petals.jbi.management.deployment.DeploymentServiceMBean;
import org.objectweb.petals.jbi.management.installation.InstallationServiceMBean;
import org.objectweb.petals.kernel.admin.PetalsAdminServiceMBean;
import org.objectweb.petals.kernel.monitoring.router.Monitoring;
import org.objectweb.petals.kernel.monitoring.router.RouterMonitorImpl;
import org.objectweb.util.monolog.wrapper.remote.lib.MonologFactoryMBeanImpl;

/**
 * This class helps the Launcher to register MBean
 *
 * @author alouis
 *
 */
public class MBeanHelper {

    public static final String DOMAIN = "Petals";

    public static final String ADMIN_MBEAN = "Admin";

    public static final String DEPLOYMENT_MBEAN = "Deployment";

    public static final String INSTALLATION_MBEAN = "Installation";

    public static final String LOGGER_MBEAN = "Logger";

    public static final String MONITORING_MBEAN = "Monitoring";

    public static final String PETALS_ADMIN = "PetalsAdmin";

    /**
     * The FractalHelper singleton
     */
    private static FractalHelper fractalHelper;

    /**
     * The singleton internal instance
     */
    private static MBeanHelper mbeanHelper;

    private MBeanHelper() throws ADLException {
        fractalHelper = FractalHelper.getFractalHelper();
    }

    public static MBeanHelper getMBeanHelper() throws ADLException {
        if (mbeanHelper == null) {
            mbeanHelper = new MBeanHelper();
        }
        return mbeanHelper;
    }

    /**
     * Find the local jmx server in the petals component
     *
     * @param petalsComposite
     *            the petals component
     * @return the local jmx server
     * @throws NoSuchInterfaceException
     * @throws ADLException
     */
    public MBeanServer findLocalJMXServer(Component petalsComposite)
        throws NoSuchInterfaceException, ADLException {
        ContentController petalsContentController = Fractal
            .getContentController(petalsComposite);
        Component jmxService = fractalHelper.getComponentByName(
            petalsContentController, FractalHelper.JMX_COMPONENT);

        MBeanServer jmxServer = ((JMXService) jmxService
            .getFcInterface("service")).getLocalJMXServer();

        return jmxServer;
    }

    public void registerMBeans(Component petalsComposite) throws Exception {
        ContentController contentController = Fractal
            .getContentController(petalsComposite);

        MBeanServer jmxServer = findLocalJMXServer(petalsComposite);

        // register the fractal lifecycle controller
        LifeCycleController lcc = Fractal
            .getLifeCycleController(petalsComposite);

        Introspector.CURRENCY_TIME_LIMIT = "-1";

        jmxServer.registerMBean(Introspector.createMBean(lcc), new ObjectName(
            "FC/Petals@" + lcc.hashCode() + ":itf=lifecycle-controller"));

        // register AdminService
        registerComponent(contentController, FractalHelper.ADMIN_COMPONENT,
            jmxServer, AdminServiceMBean.class, DOMAIN + ":name=" + ADMIN_MBEAN
                + ",type=service");

        // register DeploymentService
        registerComponent(contentController,
            FractalHelper.DEPLOYMENT_COMPONENT, jmxServer,
            DeploymentServiceMBean.class, DOMAIN + ":name=" + DEPLOYMENT_MBEAN
                + ",type=service");

        // register InstallationService
        registerComponent(contentController,
            FractalHelper.INSTALLATION_COMPONENT, jmxServer,
            InstallationServiceMBean.class, DOMAIN + ":name="
                + INSTALLATION_MBEAN + ",type=service");

        // register Petals Admin
        registerComponent(contentController,
            FractalHelper.PETALSADMIN_COMPONENT, jmxServer,
            PetalsAdminServiceMBean.class, DOMAIN + ":name=" + PETALS_ADMIN
                + ",type=service");

        // register the monitoring tool
        Monitoring monitoring = new Monitoring();
        jmxServer.registerMBean(monitoring, new ObjectName(DOMAIN + ":name="
            + MONITORING_MBEAN + ",type=service"));

        // bind the fractal router monitoring to the monitoring tool
        Component router_monitor_component = FractalHelper.getFractalHelper()
            .getComponentByName(contentController,
                FractalHelper.ROUTER_MONITOR_COMPONENT);
        RouterMonitorImpl router_monitor = (RouterMonitorImpl) router_monitor_component
            .getFcInterface("/content");
        router_monitor.setMonitoring_util(monitoring);
        monitoring.setRouterMonitor(router_monitor);

        // register the Monolog
        jmxServer.registerMBean(Introspector
            .createMBean(new MonologFactoryMBeanImpl()), new ObjectName(DOMAIN
            + ":name=" + LOGGER_MBEAN + ",type=service"));
    }

    /**
     * Return the MBean name from the jmx server with the specified pattern
     *
     * @param pattern
     * @param jmxServer
     * @return the objectname, null if no object match
     * @throws MalformedObjectNameException
     * @throws NullPointerException
     * @throws IOException
     */
    @SuppressWarnings("unchecked")
    public ObjectName retrieveServiceMBean(String serviceName,
        MBeanServer jmxServer) throws MalformedObjectNameException,
        NullPointerException, IOException {

        ObjectName result = null;

        // crate the search filter
        Hashtable<String, String> attributes = new Hashtable<String, String>();
        attributes.put("name", serviceName);
        attributes.put("type", "service");
        ObjectName objName = new ObjectName("Petals", attributes);

        Set objNames = jmxServer.queryNames(objName, null);
        if (objNames != null && objNames.size() == 1) {
            result = (ObjectName) objNames.iterator().next();
        }
        return result;
    }

    /**
     * Register a fractal component-service to the jmx server. Assume that the
     * interface name of the fractal component is "service"
     *
     * @param contentController
     *            contentcontroller where the fractal comp can be found
     * @param fractalName
     *            the name of the fractal comp
     * @param jmxServer
     *            the jmx server to register to
     * @param mbeanClass
     *            the interface of the mbean
     * @param mbeanName
     *            the name of the mbean
     * @throws Exception
     */
    @SuppressWarnings("unchecked")
    protected void registerComponent(ContentController contentController,
        String fractalName, MBeanServer jmxServer, Class mbeanClass,
        String mbeanName) throws Exception {
        Component systemState = fractalHelper.getComponentByName(
            contentController, fractalName);
        Object object = systemState.getFcInterface("service");

        jmxServer.registerMBean(Introspector.createMBean(object, mbeanClass),
            new ObjectName(mbeanName));
    }

}
