/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.kernel.server;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Properties;

import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalContentException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.util.Fractal;
import org.objectweb.petals.PetalsException;
import org.objectweb.petals.communication.jndi.client.JNDIService;
import org.objectweb.petals.communication.topology.TopologyService;
import org.objectweb.petals.jbi.management.ManagementException;
import org.objectweb.petals.jbi.management.recovery.SystemRecoveryService;
import org.objectweb.petals.jbi.messaging.transport.Transporter;
import org.objectweb.petals.kernel.admin.PetalsAdminServiceInterface;
import org.objectweb.petals.kernel.configuration.ConfigurationService;
import org.objectweb.petals.kernel.configuration.ContainerConfiguration;
import org.objectweb.petals.kernel.configuration.DomainConfiguration;
import org.objectweb.petals.util.JNDIUtil;
import org.objectweb.petals.util.SystemUtil;
import org.objectweb.util.monolog.Monolog;

/**
 * This class allows to start and stop the Petals Server.
 *
 * @author rnaudin - eBMWebsourcing
 *
 */
public class PetalsServer {

    private static final String VERSION_ACTION = "getSystemInfo";

    private static final String STOP_ACTION = "stopContainer";

    private static final String SHUTDOWN_ACTION = "shutdownContainer";

    private static final String LOGGER_PROPERTIES_FILE_NAME = "loggers.properties";

    private static final String[] MONOLOG_FILE_HANDLERS = {"petalsLF",
        "joramLF", "dreamLF"};

    /**
     * Allow to start Fractal Explorer to observe the petals architecture
     */
    private boolean observer;

    /**
     * Listener called back when petals is started or stopped
     */
    private PetalsListener petalsListener;

    /**
     * Thread used to stop Petals in an isolate thread
     */
    private PetalsStopThread petalsStopThread;

    /**
     * The domain configuration
     */
    private DomainConfiguration domainConfiguration;

    /**
     * The container configuration
     */
    private ContainerConfiguration containerConfiguration;

    /**
     * The locker used to know if PEtALS engine is running
     */
    private Locker locker;

    /**
     * The fractalHelper singleton
     */
    private static FractalHelper fractalHelper;

    /**
     * The MBeanHelper singleton
     */
    private static MBeanHelper mbeanHelper;

    /**
     * Petals composite
     */
    private static Component petalsComposite = null;

    /**
     * Petals Content Controller
     */
    private static ContentController petalsContentController = null;

    /**
     * Get the content controller of petals
     *
     */
    public ContentController getPetalsContentController() {
        return petalsContentController;
    }

    /**
     * Creates a new instance of {@link PetalsServer}
     *
     * @throws PetalsException
     */
    public PetalsServer() throws PetalsException {
        this.petalsStopThread = new PetalsStopThread(this);
        this.locker = new Locker(SystemUtil.getPetalsInstallDirectory());
        try {
            fractalHelper = FractalHelper.getFractalHelper();
            mbeanHelper = MBeanHelper.getMBeanHelper();
        } catch (ADLException e) {
            throw new PetalsException("Failed to load fractal factory");
        }
    }

    /**
     *
     * @return
     */
    public PetalsStopThread getPetalsStopThread() {
        return this.petalsStopThread;
    }

    /**
     *
     * @param obs
     */
    public void setObserver(boolean obs) {
        this.observer = obs;
    }

    /**
     * Initialize petals composite
     *
     * @throws PetalsException
     *
     */
    private void initializePetalsComposite() throws PetalsException {
        try {
            petalsComposite = fractalHelper
                .createNewComponent(FractalHelper.PETALS_COMPOSITE);
            petalsContentController = Fractal
                .getContentController(petalsComposite);
        } catch (ADLException e) {
            e.printStackTrace();
            String msg = "Error creating petalsComposite.";
            throw new PetalsException(msg, e);
        } catch (NoSuchInterfaceException e) {
            System.out.println("NoSuchInterfaceException");
            e.printStackTrace();
            String msg = "Error creating petalsComposite.";
            throw new PetalsException(msg, e);
        }
    }

    /**
     * Initialize the environment for Monolog and start the Monolog factory
     *
     * @throws IOException
     */
    private void initializeMonolog() throws IOException {
        // get the loggers properties
        URL logConfUrl = this.getClass().getResource(
            "/" + LOGGER_PROPERTIES_FILE_NAME);
        if (logConfUrl == null) {
            throw new IOException("Failed to reach Monolog resource '"
                + LOGGER_PROPERTIES_FILE_NAME + "'");
        }
        Properties logProperties = new Properties();
        logProperties.load(logConfUrl.openStream());

        // configure the Monolog handlers
        configureMonologHandlers(logProperties);

        Monolog.getMonologFactory(logProperties);
    }

    /**
     * Configure the Monolog handlers
     *
     * @param logProperties
     */
    private void configureMonologHandlers(Properties logProperties) {
        // create logs directory is not present
        File logDir = new File(SystemUtil.getPetalsInstallDirectory(), "logs");
        logDir.mkdirs();

        // get the current date as a string
        Date currentDate = new Date(System.currentTimeMillis());
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        String dateString = dateFormat.format(currentDate);

        for (String handlerName : MONOLOG_FILE_HANDLERS) {
            // set the path names of the Monologs file handlers
            String handlerOutput = logProperties.getProperty("handler."
                + handlerName + ".output");
            if (handlerOutput != null) {
                handlerOutput = handlerOutput.replace("%d", dateString);
                logProperties.setProperty("handler." + handlerName + ".output",
                    new File(logDir, handlerOutput).getAbsolutePath());
            }

            // Monolog bug on the appendMode field
            String handlerAppendMode = logProperties.getProperty("handler"
                + handlerName + "appendMode");
            if (handlerAppendMode != null
                && (handlerAppendMode.equals("true") || handlerAppendMode
                    .equals("false"))) {
                System.setProperty(handlerAppendMode, handlerAppendMode);
            }
        }
    }

    /**
     * Start the Petals Fractal composite.
     *
     * @throws NoSuchInterfaceException
     * @throws IllegalLifeCycleException
     * @throws ADLException
     * @throws IllegalBindingException
     * @throws IllegalContentException
     * @throws PetalsException
     */
    private void startPetalsComposite() throws NoSuchInterfaceException,
        IllegalLifeCycleException, ADLException, IllegalContentException,
        IllegalBindingException, PetalsException {

        if (this.observer) {
            // Creation of the explorer
            fractalHelper.createExplorer(petalsComposite);
        }

        // start first the configuration component and the communication
        // composite
        Component configurationComponent = fractalHelper.getComponentByName(
            petalsContentController, FractalHelper.CONFIGURATION_COMPONENT);
        if (!fractalHelper.startComponent(configurationComponent)) {
            throw new PetalsException(
                "Failed to start PEtALS Fractal components");
        }

        Component communicationComposite = fractalHelper.getComponentByName(
            petalsContentController, FractalHelper.COMM_COMPOSITE);
        if (!fractalHelper.startComponent(communicationComposite)) {
            throw new PetalsException(
                "Failed to start PEtALS Fractal components");
        }

        // then update the TCPPort fractal attribute of the DreamTransporter
        ConfigurationService configurationService = (ConfigurationService) configurationComponent
            .getFcInterface("service");
        containerConfiguration = configurationService
            .getContainerConfiguration();
        domainConfiguration = configurationService.getDomainConfiguration();
        fractalHelper.setupDreamTransporter(petalsContentController,
            containerConfiguration.getDreamTCPPort());

        // finally, start the others components
        if (!fractalHelper.startComponent(petalsComposite)) {
            throw new PetalsException(
                "Failed to start PEtALS Fractal components");
        }
    }

    /**
     * Stop the Petals Fractal composite.
     *
     * @throws Exception
     */
    private void stopPetalsComposite() throws Exception {

        LifeCycleController lifeCycleController = Fractal
            .getLifeCycleController(petalsComposite);

        if (lifeCycleController.getFcState()
            .equals(LifeCycleController.STARTED)) {

            // First prepare the stop of the Dispatcher, to stop the ongoing
            // transfers
            Component dispatcherComponent = fractalHelper.getComponentByName(
                petalsContentController, FractalHelper.DISPATCHER_COMPONENT);
            Transporter dispatcher = (Transporter) dispatcherComponent
                .getFcInterface("service");
            dispatcher.prepareStop();

            // Then stop the Transport Protocols, to avoid blocking calls to the
            // interface
            // TransportProtocolListener.
            Component joramComponent = fractalHelper.getComponentByName(
                petalsContentController,
                FractalHelper.JORAMTRANSPORTER_COMPONENT);
            LifeCycleController joramLCController = Fractal
                .getLifeCycleController(joramComponent);
            joramLCController.stopFc();

            Component localComponent = fractalHelper.getComponentByName(
                petalsContentController,
                FractalHelper.LOCALTRANSPORTER_COMPONENT);
            LifeCycleController localLCController = Fractal
                .getLifeCycleController(localComponent);
            localLCController.stopFc();

            Component dreamComponent = fractalHelper.getComponentByName(
                petalsContentController,
                FractalHelper.DREAMTRANSPORTER_COMPONENT);
            LifeCycleController dreamLCController = Fractal
                .getLifeCycleController(dreamComponent);
            dreamLCController.stopFc();

            // Finally, stop the others components, we must stop the Composite
            // elements in an opposite order than their start order
            fractalHelper.stopComposite(petalsComposite);
        }
    }

    /**
     * Register to the Petals Admin service this Petals server instance
     *
     * @throws NoSuchInterfaceException
     * @throws PetalsException
     */
    private void registerPetalsServer() throws NoSuchInterfaceException,
        PetalsException {
        Component petalsAdminComponent = fractalHelper.getComponentByName(
            petalsContentController, FractalHelper.PETALSADMIN_COMPONENT);
        PetalsAdminServiceInterface petalsAdminService = (PetalsAdminServiceInterface) petalsAdminComponent
            .getFcInterface("service");

        petalsAdminService.setPetalsServer(this);
    }

    /**
     * Recover the system state. All state elements are recovered (components,
     * share libraries, service assemblies)
     *
     * @throws NoSuchInterfaceException
     * @throws ManagementException
     * @throws ADLException
     * @throws PetalsException
     */
    private void recoverSystem() throws NoSuchInterfaceException,
        ManagementException, ADLException, PetalsException {
        Component systemRecoveryComponent = fractalHelper.getComponentByName(
            petalsContentController, FractalHelper.SYSTEMRECOVERY_COMPONENT);
        SystemRecoveryService systemRecoveryService = (SystemRecoveryService) systemRecoveryComponent
            .getFcInterface("service");

        systemRecoveryService.recoverAllEntities();
    }

    // /**
    // *
    // * Initialize the SSL properties if the corresponding parameters are set
    // in
    // * the server.properties configuration file.
    // *
    // * @throws IOException
    // */
    // private void initializeSSL() throws IOException {
    //
    // if (!"".equals(SystemUtil.getTrustedkeystore())) {
    // System.setProperty("javax.net.ssl.trustStore", SystemUtil
    // .getTrustedkeystore());
    // }
    //
    // if (!"".equals(SystemUtil.getTrustedkeystorePassword())) {
    // System.setProperty("javax.net.ssl.trustStorePassword", SystemUtil
    // .getTrustedkeystorePassword());
    // }
    //
    // if (!"".equals(SystemUtil.getIdentitykeystore())) {
    // System.setProperty("javax.net.ssl.keyStore", SystemUtil
    // .getIdentitykeystore());
    // }
    //
    // if (!"".equals(SystemUtil.getIdentitykeystorePassword())) {
    // System.setProperty("javax.net.ssl.keyStorePassword", SystemUtil
    // .getIdentitykeystorePassword());
    // }
    // }

    /**
     * Set the Petals listener
     *
     * @param petalsListener
     */
    public void addStopListener(PetalsListener petalsListener) {
        this.petalsListener = petalsListener;
    }

    /**
     * Remove the Petals listener
     *
     */
    public void removeListener() {
        this.petalsListener = null;
    }

    /**
     * Prints the Petals version on standard output. We connect to the JMX
     * server of the server that is running with the server.properties entries.
     *
     * @throws PetalsException
     */
    public void getServerVersion() throws PetalsException {

        try {
            // get the jmxServer of this container
            MBeanServer mbeanServer = mbeanHelper
                .findLocalJMXServer(petalsComposite);

            // search the Admin mbean
            ObjectName result = mbeanHelper.retrieveServiceMBean(
                MBeanHelper.ADMIN_MBEAN, mbeanServer);

            // call Stop
            if (result != null) {
                String version = (String) mbeanServer.invoke(result,
                    VERSION_ACTION, new Object[0], new String[0]);

                System.out.println(version);

            }
        } catch (Exception exception) {
            throw new PetalsException(exception);
        }
    }

    public String getContainerConfiguration() throws PetalsException {
        if (containerConfiguration == null) {
            throw new PetalsException(
                "The container configuration is not properly set");
        }

        return containerConfiguration.toString();
    }

    /**
     * Browse the JNDI server contents.
     *
     * @return
     * @throws PetalsException
     */
    public String browseJNDI() throws PetalsException {
        if (containerConfiguration == null) {
            throw new PetalsException(
                "The container configuration is not properly set");
        }

        String result = null;
        Component jndiComponent = fractalHelper.getComponentByName(
            petalsContentController, FractalHelper.JNDI_COMPONENT);
        try {
            JNDIService jndiService = (JNDIService) jndiComponent
                .getFcInterface("service");
            InitialContext initialContext = jndiService.getInitialContext();
            if (domainConfiguration.getJndiHost() == null) {
                result = JNDIUtil.browseJNDI(initialContext, (String) null, 0);
            } else {
                result = JNDIUtil.browseJNDI(initialContext,
                    domainConfiguration.getJndiHost(), domainConfiguration
                        .getJndiPort());
            }

        } catch (NoSuchInterfaceException e) {
            throw new PetalsException(e);
        } catch (NamingException e) {
            throw new PetalsException(e);
        }

        return result;
    }

    /**
     * Initialize the PEtALS environment. Init Qname, JMX if necessarry,
     * Monolog, Fractal and SSL.
     *
     * @param embedded
     *            {@ code true} if the PEtALS server is embedded in another
     *            environment
     */
    public void init(boolean embedded) throws PetalsException {

        // set the compatible SUID for the Qname
        System.setProperty(
            "com.sun.xml.namespace.QName.useCompatibleSerialVersionUID", "1.0");

        if (!embedded) {
            System.setProperty("javax.management.builder.initial",
                "mx4j.server.MX4JMBeanServerBuilder");
        }

        try {
            // initialize MONOLOG
            this.initializeMonolog();

            // initialize Petals Fractal composite
            this.initializePetalsComposite();
        } catch (Exception exception) {
            // clean up the petals server
            throw new PetalsException("Problem while initializing Petals",
                exception);
        }
    }

    /**
     * Start the Petals Server. Start the fractal components. Register the
     * different component, then try to recover the Petals container.
     *
     * @throws PetalsException
     */
    public void start() throws PetalsException {

        try {
            // start Petals platform
            this.startPetalsComposite();

            // register the Petals server in the Petals admin.
            this.registerPetalsServer();

            // expose JBI MBean object in the JMX Server
            mbeanHelper.registerMBeans(petalsComposite);

            // recover all entities
            this.recoverSystem();

            this.startupDone();

            // add the lock file
            this.locker.lock();
        } catch (Exception exception) {
            // clean up the petals server

            if (petalsComposite != null) {
                try {
                    fractalHelper.stopComponent(petalsComposite);
                } catch (Exception e) {
                    // do nothing
                }
            }

            throw new PetalsException("Problem while starting Petals",
                exception);
        }

        // try {
        // this.initializeSSL();
        // } catch (IOException e) {
        // throw new PetalsException("SSL Properties cannot be correctly set!");
        // }

        // notify the listener if there is one
        if (this.petalsListener != null) {
            this.petalsListener.onPetalsStarted();
        }
    }

    /**
     * Trigger the stop or shutdown statement. We connect to the JMX server,
     * locally or remotely, and invoke the statement.
     *
     * @param shutdown
     *            shutdown if <code>, <code>stop</code> if false
     *
     * @throws PetalsException
     */
    public void triggerStop(boolean shutdown) throws PetalsException {
        String action = (shutdown ? SHUTDOWN_ACTION : STOP_ACTION);

        MBeanServer jmxServer = null;
        ObjectName petalsAdminMBeanName = null;
        try {
            jmxServer = mbeanHelper.findLocalJMXServer(petalsComposite);
            petalsAdminMBeanName = new ObjectName(MBeanHelper.DOMAIN + ":name="
                + MBeanHelper.PETALS_ADMIN + ",type=service");
            jmxServer.isRegistered(petalsAdminMBeanName);
        } catch (Exception e) {
            String msg = "Petals admin can not be accessed.";
            throw new PetalsException(msg, e);
        }

        try {
            // call the stop procedure and wait the thread stop to be processed
            jmxServer.invoke(petalsAdminMBeanName, action, new Object[0],
                new String[0]);
            this.petalsStopThread.join();
        } catch (Exception e) {
            throw new PetalsException(e);
        }
    }

    /**
     * Stop or Shutdown the Petals server.
     *
     * @throws PetalsException
     */
    public void stop() throws PetalsException {
        try {
            this.stopPetalsComposite();
        } catch (Exception e) {
            throw new PetalsException(e);
        }

        this.locker.unlock();

        // notify the listener if there is one
        if (this.petalsListener != null) {
            this.petalsListener.onPetalsStopped();
        }

        // NOTE : unreachable part if the listener exists the system, case of
        // standalone

    }

    /**
     *
     * @return
     */
    public final boolean isLocked() {
        return this.locker.isLocked();
    }

    /**
     * Set the local container state to started. The local container state is in
     * unknow state until all is really started.
     *
     * @throws Exception
     */
    private void startupDone() throws Exception {
        Component topologyComponent = fractalHelper.getComponentByName(
            petalsContentController, FractalHelper.TOPOLOGY_COMPONENT);

        if (topologyComponent == null) {
            throw new NullPointerException(
                "The network fractal component is null");
        } else {
            TopologyService service = (TopologyService) topologyComponent
                .getFcInterface("service");
            service.setContainerState(containerConfiguration.getName(),
                ContainerConfiguration.ContainerState.STARTED);
        }
    }
}
