/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 eBM Websourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: RepositoryServiceImpl.java 5347 2007-09-27 10:08:51Z rnaudin $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.platform.repository;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.URI;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.io.filefilter.PrefixFileFilter;
import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.petals.PetalsException;
import org.objectweb.petals.kernel.configuration.ConfigurationService;
import org.objectweb.petals.kernel.configuration.ContainerConfiguration;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.petals.util.ParameterCheckHelper;
import org.objectweb.petals.util.SystemUtil;
import org.objectweb.petals.util.ZipUtil;
import org.objectweb.util.monolog.api.Logger;

/**
 * Platform Component Repository service implementation.
 *
 * @version $Rev: 5347 $ $Date: 2007-09-27 12:08:51 +0200 (jeu, 27 sep 2007) $
 * @since Petals 1.0
 * @author <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 * @author alouis - EBM Websourcing
 * @author ofabre - EBM Websourcing
 * @author ddesjardins - eBMWebsourcing
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = org.objectweb.petals.platform.repository.RepositoryService.class))
public class RepositoryServiceImpl implements RepositoryService {

    /**
     * The date pattern
     */
    private static final String DATE_PATTERN = "MM-dd-yyyy-hhmmssSSS";

    private static final int DATE_PATTERN_LENGTH = 20;

    /**
     * Path to component install directory
     */
    private static final String INSTALL_PATH = "install";

    /**
     * Path to components directory into petals repository
     */
    private static final String COMPONENTS_PATH = "components";

    /**
     * Path to components directory into petals repository
     */
    private static final String SHAREDLIBS_PATH = "shared-libraries";

    /**
     * Path to lost+found directory into petals repository
     */
    private static final String LOSTPLUSFOUND_PATH = "lost+found";

    /**
     * Path to components directory into petals repository
     */
    private static final String SERVICEASSEMBLIES_PATH = "service-assemblies";

    /**
     * Path to component working directory
     */
    private static final String WORK_PATH = "work";

    /**
     * The complete repository directory path.
     */
    private File repositoryDirectory = null;

    /**
     * Entity types (components, shared libs or service assemblies)
     */
    public static enum EntityType {
        COMPONENT_TYPE, SL_TYPE, SA_TYPE
    };

    /**
     * logger wrapper
     */
    protected LoggingUtil log;

    /**
     * Monolog Logger instance.
     */
    @Monolog(name = "logger")
    protected Logger logger;

    /**
     * DateFormat used for repository folders
     */
    private SimpleDateFormat folderDateFormat;

    /**
     * The Configuration service fractal component
     */
    @Requires(name = "configuration", signature = org.objectweb.petals.kernel.configuration.ConfigurationService.class)
    private ConfigurationService configurationService;

    /**
     * The container configuration
     */
    private ContainerConfiguration containerConfiguration;

    /**
     * Copy an entity package into petals repository
     *
     * @param entityId
     *            {@link String} unique entity id that identifies an unique
     *            entity directory into the petals repository.
     * @param tempFile
     *            {@link File} the entity package file to add into the
     *            repository.
     * @param entityType
     *            the entity type: component, shared lib or service assembly
     * @return installation root {@link URI}
     * @throws PetalsException
     *             if entity installation or work root can't be created or if
     *             entity files can't be copied in the installation root
     */
    private URI addPackage(String entityId, File tempFile, EntityType entityType)
        throws PetalsException {

        File rootFile = createEntityRootFile(entityId, entityType);

        File installationRoot = createEntityInstallRootFolder(rootFile);

        File workRoot = null;
        try {
            workRoot = createEntityWorkRootFolder(rootFile);
        } catch (PetalsException e) {
            try {
                FileUtils.forceDelete(installationRoot);
            } catch (IOException e1) {
                // Nothing to do, just log
                log.warning("Residual incomplete component folder");
            }
            throw e;
        }

        try {
            FileUtils.copyDirectory(tempFile, installationRoot);
            FileUtils.forceDelete(tempFile);
        } catch (IOException e) {
            try {
                FileUtils.forceDelete(installationRoot);
                FileUtils.forceDelete(workRoot);
            } catch (IOException e1) {
                // Nothing to do, just log
                log.warning("Residual incomplete component folder");
            }
            throw new PetalsException(e);
        }

        return installationRoot.toURI();
    }

    private File createEntityWorkRootFolder(File rootFile)
        throws PetalsException {
        File workRoot = new File(rootFile, WORK_PATH);
        if (!workRoot.mkdirs()) {
            throw new PetalsException("Failed to create work dir: "
                + workRoot.getAbsolutePath());
        }
        return workRoot;
    }

    private File createEntityInstallRootFolder(File rootFile)
        throws PetalsException {
        File installationRoot = new File(rootFile, INSTALL_PATH);

        if (!installationRoot.mkdirs()) {
            throw new PetalsException(
                "Failed to create installation root dir: "
                    + installationRoot.getAbsolutePath());
        }
        return installationRoot;
    }

    private File createEntityRootFile(String entityId, EntityType entityType)
        throws PetalsException {
        File rootFile = null;

        switch (entityType) {
            case COMPONENT_TYPE:
                rootFile = createNewComponentRoot(entityId);
                break;
            case SA_TYPE:
                rootFile = createNewServiceAssemblyRoot(entityId);
                break;
            case SL_TYPE:
                rootFile = createNewSharedLibRoot(entityId);
                break;
        }
        return rootFile;
    }

    public URI addComponentPackage(String componentID, File tempFile)
        throws PetalsException {
        ParameterCheckHelper.isNullOrEmptyParameter(componentID, "componentID");
        ParameterCheckHelper.isNullParameter(tempFile, "tempFile");
        return addPackage(componentID, tempFile, EntityType.COMPONENT_TYPE);
    }

    public URI addSharedLibPackage(String sharedLibID, File tempFile)
        throws PetalsException {
        ParameterCheckHelper.isNullOrEmptyParameter(sharedLibID, "sharedLibID");
        ParameterCheckHelper.isNullParameter(tempFile, "tempFile");
        return addPackage(sharedLibID, tempFile, EntityType.SL_TYPE);
    }

    public URI addServiceAssemblyPackage(String serviceAssemblyID, File tempFile)
        throws PetalsException {
        ParameterCheckHelper.isNullOrEmptyParameter(serviceAssemblyID,
            "serviceAssemblyID");
        ParameterCheckHelper.isNullParameter(tempFile, "tempFile");
        return addPackage(serviceAssemblyID, tempFile, EntityType.SA_TYPE);
    }

    /**
     * @see org.objectweb.petals.platform.repository.RepositoryService#explodeSUIntoSAInstallFolder(String,
     *      URI, String)
     */
    public URI explodeSUIntoSAInstallFolder(String suId, URI suZipLocation,
        String saId) throws PetalsException {
        File installationRoot = createSUInstallRootFolder(suId, saId);
        try {
            File zipArchive = new File(suZipLocation);
            ZipUtil.unzipArchive(zipArchive, installationRoot);
        } catch (IOException e) {
            try {
                FileUtils.forceDelete(installationRoot);
            } catch (IOException e1) {
                // Nothing to do, just log
                log.warning("Residual incomplete component folder");
            }
            throw new PetalsException(e);
        }
        return installationRoot.toURI();
    }

    /**
     * Create the SU install root into the SA install root
     *
     * @param suId
     *            the service unit unique identifier
     * @param saId
     *            the service assembly unique identifier
     * @return the SU install root File
     * @throws PetalsException
     *             if SU install root can't be created
     */
    private File createSUInstallRootFolder(String suId, String saId)
        throws PetalsException {
        File installationRoot = new File(getServiceAssemblyInstallRoot(saId),
            suId);

        if (!installationRoot.exists() && !installationRoot.mkdirs()) {
            throw new PetalsException(
                "Failed to create installation root dir: "
                    + installationRoot.getAbsolutePath());
        }
        return installationRoot;
    }

    public File getComponentsDirectory() {
        return new File(getRepositoryDirectory(), File.separator
            + COMPONENTS_PATH);

    }

    public File getServiceAssembliesDirectory() {
        return new File(getRepositoryDirectory(), File.separator
            + SERVICEASSEMBLIES_PATH);

    }

    public File getSharedLibsDirectory() {
        return new File(getRepositoryDirectory(), File.separator
            + SHAREDLIBS_PATH);
    }

    public String getLostPlusFoundDirectory() {
        return SystemUtil.getPetalsInstallDirectory().getAbsolutePath()
            + File.separator + LOSTPLUSFOUND_PATH;
    }

    public File getComponentInstallRoot(String componentId)
        throws PetalsException {
        String path = getComponentRoot(componentId) + File.separator
            + INSTALL_PATH;

        return new File(path);

    }

    public File getComponentRoot(String componentId) throws PetalsException {
        return getEntityRoot(componentId, EntityType.COMPONENT_TYPE);
    }

    private String[] filterPathsOnlength(String componentId,
        String[] componentsFilePaths) {
        List<String> goodFilePaths = new ArrayList<String>();

        for (String path : componentsFilePaths) {
            if (path.length() == componentId.length() + DATE_PATTERN_LENGTH + 1) {
                goodFilePaths.add(path);
            }
        }

        return goodFilePaths.toArray(new String[0]);
    }

    /**
     * Retrieve the most recent path in an array of folder path, according to
     * the included date information.
     *
     * @param filePaths
     *            the file paths to compare
     * @param entityId
     *            an entity (component, shared lib or service assembly) id
     * @return the most recent file path, null if given filePaths array is null
     * @throws ParseException
     *             if an error occurs during Date extraction
     */
    private String searchForMostRecentFilePath(String[] filePaths,
        String entityId) throws ParseException {
        String mostRecentPath = null;
        Date mostRecentDate = null;
        if (filePaths != null) {
            for (String path : filePaths) {
                String baseName = FilenameUtils.getName(path);
                Date currentDate = extractDateFromName(baseName, entityId);
                if (mostRecentDate != null) {
                    if (currentDate.after(mostRecentDate)) {
                        mostRecentDate = currentDate;
                        mostRecentPath = path;
                    }
                } else {
                    mostRecentDate = currentDate;
                    mostRecentPath = path;
                }
            }
        }
        return mostRecentPath;
    }

    /**
     * Extract a Date from a file name.
     *
     * @param baseName
     *            a file name
     * @param entityId
     *            an entity (component, shared lib or service assembly) id
     * @return the file Date, non null
     * @throws ParseException
     *             if an error occurs during Date extraction
     */
    private Date extractDateFromName(String baseName, String entityId)
        throws ParseException {
        String dateString = baseName.substring(baseName.indexOf(entityId)
            + entityId.length() + 1);

        return folderDateFormat.parse(dateString);
    }

    private File createNewComponentRoot(String componentId)
        throws PetalsException {
        String path = getComponentsDirectory() + File.separator + componentId
            + "-"
            + folderDateFormat.format(new Date(System.currentTimeMillis()));

        return new File(path);
    }

    public File getComponentWorkRoot(String componentId) throws PetalsException {
        return new File(getComponentRoot(componentId), WORK_PATH);

    }

    public File getSharedLibRoot(String slId) throws PetalsException {
        return getEntityRoot(slId, EntityType.SL_TYPE);
    }

    private File createNewSharedLibRoot(String slId) throws PetalsException {
        String path = getSharedLibsDirectory() + File.separator + slId + "-"
            + folderDateFormat.format(new Date(System.currentTimeMillis()));

        return new File(path);
    }

    public File getSharedLibWorkRoot(String slId) throws PetalsException {
        String path = getSharedLibRoot(slId) + File.separator + WORK_PATH;
        return new File(path);

    }

    public File getSharedLibInstallRoot(String slId) throws PetalsException {
        String path = getSharedLibRoot(slId) + File.separator + INSTALL_PATH;
        return new File(path);
    }

    private File getEntityRoot(String entityId, EntityType entityType)
        throws PetalsException {
        File result = null;
        File entitiesDirectory = null;

        switch (entityType) {
            case COMPONENT_TYPE:
                entitiesDirectory = getComponentsDirectory();
                break;
            case SA_TYPE:
                entitiesDirectory = getServiceAssembliesDirectory();
                break;
            case SL_TYPE:
                entitiesDirectory = getSharedLibsDirectory();
                break;
        }

        String[] entityFilePaths = entitiesDirectory.list(new PrefixFileFilter(
            entityId));

        String[] entityFilePathsWithGoodLength = filterPathsOnlength(entityId,
            entityFilePaths);

        String selectedPath = null;
        try {
            selectedPath = searchForMostRecentFilePath(
                entityFilePathsWithGoodLength, entityId);
        } catch (ParseException e) {
            throw new PetalsException(
                "Can't retrieve entity root folder for the following entity: "
                    + entityId, e);
        }
        if (selectedPath != null) {
            result = new File(entitiesDirectory, selectedPath);
        }
        return result;
    }

    public File getServiceAssemblyRoot(String saId) throws PetalsException {
        return getEntityRoot(saId, EntityType.SA_TYPE);
    }

    private File createNewServiceAssemblyRoot(String saId)
        throws PetalsException {
        String path = getServiceAssembliesDirectory() + File.separator + saId
            + "-"
            + folderDateFormat.format(new Date(System.currentTimeMillis()));

        return new File(path);
    }

    public File getServiceAssemblyWorkRoot(String saId) throws PetalsException {
        String path = getServiceAssemblyRoot(saId) + File.separator + WORK_PATH;
        return new File(path);
    }

    public File getServiceAssemblyInstallRoot(String saId)
        throws PetalsException {
        String path = getServiceAssemblyRoot(saId) + File.separator
            + INSTALL_PATH;
        return new File(path);
    }

    /**
     * @throws PetalsException
     * @see org.objectweb.petals.platform.repository.RepositoryService#removePackage(java.lang.String)
     */
    private boolean removePackage(String entityID, EntityType entityType)
        throws PetalsException {
        boolean removed = true;

        File entityRoot = null;

        switch (entityType) {
            case COMPONENT_TYPE:
                entityRoot = getComponentRoot(entityID);
                break;
            case SA_TYPE:
                entityRoot = getServiceAssemblyRoot(entityID);
                break;
            case SL_TYPE:
                entityRoot = getSharedLibRoot(entityID);
                break;
        }

        try {
            FileUtils.deleteDirectory(entityRoot);
        } catch (IOException e) {
            log.warning("Failed to delete old Petals entity folder. "
                + "This folder will be deleted on a future Petals restarting.");
        }

        return removed;
    }

    public boolean removeComponentPackage(String componentId)
        throws PetalsException {
        return removePackage(componentId, EntityType.COMPONENT_TYPE);
    }

    public boolean removeSharedLibPackage(String slId) throws PetalsException {
        return removePackage(slId, EntityType.SL_TYPE);
    }

    public boolean removeServiceAssemblyPackage(String saId)
        throws PetalsException {
        return removePackage(saId, EntityType.SA_TYPE);
    }

    @LifeCycle(on = LifeCycleType.START)
    protected void start() throws FileNotFoundException {
        log = new LoggingUtil(logger);
        log.start();

        folderDateFormat = new SimpleDateFormat(DATE_PATTERN);
        containerConfiguration = configurationService
            .getContainerConfiguration();

        // Initialization
        /*
         * Check if a repository folder path is specified in the
         * 'server.properties' file. If it is, create it as specified
         */
        repositoryDirectory = new File(containerConfiguration
            .getRepositoryDirectoryPath());

        try {
        if (!repositoryDirectory.exists()) {
            throw new FileNotFoundException(
                "The repository directory does not exist:"
                    + containerConfiguration.getRepositoryDirectoryPath());
        }

        if (!getComponentsDirectory().exists())
            throw new FileNotFoundException(
                "The component repository of the server.properties "
                    + "file must be a valid file: " + getComponentsDirectory());

        if (!getServiceAssembliesDirectory().exists())
            throw new FileNotFoundException(
                "The service assemblies repository of the server.properties "
                    + "file must be a valid file: "
                    + getServiceAssembliesDirectory());

        if (!getSharedLibsDirectory().exists())
            throw new FileNotFoundException(
                "The shared libraries repository of the server.properties "
                    + "file must be a valid file: " + getSharedLibsDirectory());
        } catch (FileNotFoundException e) {
            // log the error here as Fractal layer is removing the Exception stack
            log.error("Failed to start Repository service", e);
            throw e;
        }

        log.end();
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() {
        log.start();

        logger = null;

        log.end();
    }

    /**
     * Return the petals repository directory. It will be created when this
     * method is called for the first time
     *
     * @throws FileNotFoundException
     *
     * @throws RuntimeException
     *             if the specified or default repository can't be created
     */
    public File getRepositoryDirectory() {
        return repositoryDirectory;
    }

}
