/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SystemUtil.java 446 2006-05-19 14:13:50Z ddesjardins $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.util;

import java.io.File;
import java.net.SocketException;
import java.net.URI;
import java.net.URL;

/**
 * This utility class provides facilities to get system information.
 *
 * @version $Rev: 446 $ $Date: 2006-05-19 16:13:50 +0200 (ven, 19 mai 2006) $
 * @since Petals 1.0
 * @author alouis, Rafael Marins
 * @author ddesjardins - eBMWebsourcing
 * @author Marc Dutoo - Open Wide
 * @author chamerling - eBMWebsourcing
 * @author roland NAUDIN - eBMWebsourcing
 */
public final class SystemUtil {

    public static final String ENVIRONMENT_INSTALL_PATH = "PETALS_HOME";

    public static final String PROPERTY_INSTALL_PATH = "petals.home";

    private static File petalsHome;

    /**
     * Creates a new instance of {@link SystemUtil}
     *
     */
    private SystemUtil() {
        super();

    }

    /**
     * Get the PEtALS home directory.
     *
     * @return The PEtALS home directory
     */
    public static File getPetalsInstallDirectory() {

        if (petalsHome == null) {
            // first try to resolve install directory set in the environment
            petalsHome = resolveInstallDirectoryFromEnvironment();
            // otherwise, guess install directory using the framework
            if (petalsHome == null) {
                petalsHome = resolveInstallDirectoryFromFramework();
            }

            if (petalsHome == null) {
                throw new RuntimeException(
                    "Petals installation directory can not be found. "
                        + "Set a PETALS_HOME variable or use petals.home JAVA property");
            }
        }

        return petalsHome;
    }

    /**
     * Try to get the petals'home environment variable : petals.home system
     * property, or PETALS_HOME env variable
     *
     * @return the related directory, or null
     */
    private static File resolveInstallDirectoryFromEnvironment() {
        File directory = null;
        String petalsHome = System.getProperty(PROPERTY_INSTALL_PATH);
        if (petalsHome == null || petalsHome.trim().equals("")) {
            petalsHome = System.getenv(ENVIRONMENT_INSTALL_PATH);
        }
        // not null or empty
        if (petalsHome != null && !petalsHome.trim().equals("")) {
            File theHome = new File(petalsHome);
            // exists and it's a directory
            if (theHome.exists() || theHome.isDirectory()) {
                directory = theHome;
            }
        }
        return directory;
    }

    /**
     * Try to find the current install directory
     *
     * @return the directory or null if a problem occured
     */
    private static File resolveInstallDirectoryFromFramework() {
        // guess from the location of the current JAR
        File directory = null;
        URL url = SystemUtil.class.getProtectionDomain().getCodeSource()
            .getLocation();
        if (url != null) {
            try {
                URI baseURI = new URI(url.toString()).resolve("..");
                directory = new File(baseURI);
            } catch (Exception ignored) {
                ignored.printStackTrace();
            }
        }
        return directory;
    }

    /**
     * Resolve the "localhost" IP address to the real IP First, use
     * InetAddress.getLocalHost(). If the returned IP is a "127.0.xxx" like IP,
     * call the system command "ipconfig" or "ifconfig" and parse the result to
     * extract the IP.
     *
     * @return the real IP address of the machine, or 127.0.0.1 if the
     *         resolution failed.
     */
    public static String resolveLocalIP() {
        String hostIP = null;
        try {
            hostIP = NetworkUtil.getIPv4Address();
        } catch (SocketException e1) {
            hostIP = NetworkUtil.IPV4_LOOPBACK_ADDRESS;
        }
        return hostIP;
    }
}
